// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/driver/device.c $
// $Rev: 53399 $
// $Date: 2023-07-12 12:04:34 -0500 (Wed, 12 Jul 2023) $

// HPDI32: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model	Vendor	Device	SubVen	SubDev	type

	{ "HPDI32",	0x10B5, 0x9656, 0x10B5, 0x2705,	GSC_DEV_TYPE_HPDI32	},	// 64-bit
	{ "HPDI32",	0x10B5, 0x9080, 0x10B5, 0x2400,	GSC_DEV_TYPE_HPDI32	},	// 32-bit
	{ "HPDI32",	0x10B5, 0x9080, 0x10B5, 0x9080,	GSC_DEV_TYPE_HPDI32	},	// 32-bit, old devices
	{ NULL }
};



//*****************************************************************************
static void _bus_width_compute(dev_data_t* dev)
{
	u16	sid;

	sid	= os_reg_pci_rx_u16(dev, 1, 2);

	if (sid == 0x9656)
		dev->cache.bus_width	= 64;
	else
		dev->cache.bus_width	= 32;
}



//*****************************************************************************
static void _clock_max_compute(dev_data_t* dev)
{
	u16	sid;

	sid	= os_reg_pci_rx_u16(dev, 1, 2);

	if (sid == 0x9656)
		dev->cache.clock_max	= 50000000L;
	else
		dev->cache.clock_max	= 25000000L;
}



//*****************************************************************************
static void _dma_compute(dev_data_t* dev)
{
	if (dev->cache.gsc_fsr_32 & D4)
		dev->cache.dmdma_ch_1	= 1;
	else
		dev->cache.dmdma_ch_1	= 0;
}



//*****************************************************************************
static void _fifo_reset_fast_compute(dev_data_t* dev)
{
	// Fast FIFO Resets are supported on devices that use SRAM instead of
	// physical FIFOs. With SRAM, FIFO Resets will normally be completed before
	// software can check that the Tx/Rx FIFO Reset bit clears itself. With
	// physical FIFOs, resets may take longer to perform.

	dev->cache.fifo_reset_fast	= 0;

	switch (dev->cache.model)
	{
		default:

			break;

		case HPDI32_QUERY_MODEL_HPDI32B:

			// Model B uses SRAM.
			dev->cache.fifo_reset_fast	= 1;
			break;

		case HPDI32_QUERY_MODEL_HPDI32AL:
		case HPDI32_QUERY_MODEL_HPDI32ALT:

			if ((dev->cache.gsc_frr_32 & 0xFF0000) >= 0x0E0000)
			{
				// Rev E and later use SRAM.
				dev->cache.fifo_reset_fast	= 1;
			}

			break;
	}
}

//*****************************************************************************
static void _fifos_compute(dev_data_t* dev)
{
	u32	bsr;
	int	i;
	int	k;
	u16	sid;

	if (dev->cache.gsc_fsr_32 & D0)
	{
		dev->cache.reg_rfsr		= 1;
		dev->cache.reg_tfsr		= 1;
		dev->cache.fifo_size_rx	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_rfsr_32);
		dev->cache.fifo_size_tx	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_tfsr_32);
	}
	else
	{
		dev->cache.reg_rfsr		= 0;
		dev->cache.reg_tfsr		= 0;
		dev->cache.fifo_size_rx	= 0;
		dev->cache.fifo_size_tx	= 0;

		// We'll manually compute the Tx FIFO size and
		// use that as teh Rx FIFO size as well.
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, 0x7);	// Reset the device and the FIFOs.

		for (k = 1; k <= 128; k++)
		{
			for (i = 0; i < 1024; i++)
				os_reg_mem_tx_u32(dev, dev->vaddr.gsc_fdr_32, 0);

			bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);

			if ((bsr & D11) == 0)
			{
				dev->cache.fifo_size_rx	= k * 1024;
				dev->cache.fifo_size_tx	= k * 1024;
				break;
			}
		}

		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, 0x7);	// Reset the device and the FIFOs.
	}

	if (dev->cache.gsc_fsr_32 & D1)
	{
		dev->cache.reg_rfwr	= 1;
		dev->cache.reg_tfwr	= 1;
	}
	else
	{
		dev->cache.reg_rfwr	= 0;
		dev->cache.reg_tfwr	= 0;
	}

	sid	= os_reg_pci_rx_u16(dev, 1, 2);

	if (dev->cache.reg_rfwr == 0)
		dev->cache.reg_fwr_update	= 0;
	else if (sid == 0x9080)
		dev->cache.reg_fwr_update	= 1;
	else if (dev->cache.model == HPDI32_QUERY_MODEL_HPDI32AL)
		dev->cache.reg_fwr_update	= 0;
	else if (dev->cache.gsc_fsr_32 & D8)
		dev->cache.reg_fwr_update	= 1;
	else
		dev->cache.reg_fwr_update	= 0;
}



//*****************************************************************************
static void _form_factor_compute(dev_data_t* dev)
{
	u32	form;

	form	= GSC_FIELD_DECODE(dev->cache.gsc_frr_32, 27, 24);

	if (dev->cache.gsc_frr_32 & HPDI32_FRR_CONFORMANT)
	{
		if (form > HPDI32_QUERY_FF_PC104P)
			form	= HPDI32_QUERY_FF_UNKNOWN;

		dev->cache.form_factor	= form;
	}
	else
	{
		if (dev->cache.gsc_frr_32 & D24)	// 32-bit device: PMC form factor
			dev->cache.form_factor	= HPDI32_QUERY_FF_PMC;
		else
			dev->cache.form_factor	= HPDI32_QUERY_FF_PCI;
	}
}



//*****************************************************************************
static void _fsr_compute(dev_data_t* dev)
{
	if (dev->cache.gsc_frr_32 & D31)
	{
		dev->cache.reg_fsr	= 1;
	}
	else
	{
		dev->cache.reg_fsr		= 0;
		dev->cache.gsc_fsr_32	= 0;
	}
}



//*****************************************************************************
static void _gpio_compute(dev_data_t* dev)
{
	if (dev->cache.gsc_fsr_32 & D3)
		dev->cache.gpio_0_5	= 1;
	else
		dev->cache.gpio_0_5	= 0;

	if (dev->cache.gsc_fsr_32 & D6)
		dev->cache.gpio_6	= 1;
	else
		dev->cache.gpio_6	= 0;

	if (dev->cache.gsc_fsr_32 & D9)
		dev->cache.gpio_d32	= 1;
	else
		dev->cache.gpio_d32	= 0;

	dev->cache.reg_acr	= dev->cache.gpio_d32;
	dev->cache.reg_gir	= dev->cache.gpio_d32;
	dev->cache.reg_gor	= dev->cache.gpio_d32;
}



//*****************************************************************************
static void _irq_compute(dev_data_t* dev)
{
	if (dev->cache.gsc_fsr_32 & D2)
	{
		dev->cache.reg_ielr	= 1;
		dev->cache.reg_ihlr	= 1;
	}
	else
	{
		dev->cache.reg_ielr	= 0;
		dev->cache.reg_ihlr	= 0;
	}
}



//*****************************************************************************
static void _model_compute(dev_data_t* dev)
{
	u16	sid;
	int	sub_id;

	sid		= os_reg_pci_rx_u16(dev, 1, 2);
	sub_id	= GSC_FIELD_DECODE(dev->cache.gsc_frr_32, 15, 8);

	dev->cache.model	= HPDI32_QUERY_MODEL_HPDI32;
	dev->model			= "HPDI32";

	if (sid == 0x9060)
	{
	}
	else if (sid == 0x9080)
	{
		if (sub_id == 0x02)
		{
			dev->cache.model	= HPDI32_QUERY_MODEL_HPDI32A;
			dev->model			= "HPDI32A";

			if ((dev->cache.gsc_frr_32 & 0xFF0000) >= 0x030000)
			{
				// The HPDI32B devices are Rev C or later.
				dev->cache.model	= HPDI32_QUERY_MODEL_HPDI32B;
				dev->model			= "HPDI32B";
			}
		}
	}
	else if (sid == 0x9656)
	{
		if (dev->cache.form_factor == HPDI32_QUERY_FF_PCI)
		{
			dev->cache.model	= HPDI32_QUERY_MODEL_HPDI32AL;
			dev->model			= "HPDI32AL";
		}
		else if (dev->cache.form_factor == HPDI32_QUERY_FF_PMC)
		{
			dev->cache.model	= HPDI32_QUERY_MODEL_HPDI32ALT;
			dev->model			= "HPDI32ALT";
		}
	}
}



//*****************************************************************************
static void _over_under_run_compute(dev_data_t* dev)
{
	if (dev->cache.gsc_fsr_32 & D5)
		dev->cache.over_under_run	= 1;
	else
		dev->cache.over_under_run	= 0;
}



//*****************************************************************************
static void _proc_id_str_compute(dev_data_t* dev)
{
	u32	bsr;

	if (dev->cache.user_jumpers)
	{
		bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
		bsr	= (bsr & 0x30000) >> 16;
		sprintf(dev->proc_id_str, "0x%lX", (long) bsr);
	}
	else
	{
		strcpy(dev->proc_id_str, "none");
	}
}



//*****************************************************************************
static void _reg_tcdr_compute(dev_data_t* dev)
{
	u32		a;
	u32		b;
	VADDR_T	va;

	if (dev->cache.model > HPDI32_QUERY_MODEL_HPDI32)
	{
		dev->cache.reg_tcdr	= 1;
	}
	else
	{
		va	= GSC_VADDR(dev, 0x38);
		a	= os_reg_mem_rx_u32(dev, va);
		os_reg_mem_tx_u32(dev, va, 0);
		b	= os_reg_mem_rx_u32(dev, va);
		os_reg_mem_tx_u32(dev, va, a);

		if ((b & 0xFFFF) == 0)
			dev->cache.reg_tcdr	= 1;
		else
			dev->cache.reg_tcdr	= 0;
	}
}



//*****************************************************************************
static void _single_cycle_disable_compute(dev_data_t* dev)
{
	if (dev->cache.gsc_fsr_32 & D7)
		dev->cache.single_cycle_disable	= 1;
	else
		dev->cache.single_cycle_disable	= 0;
}



//*****************************************************************************
static void _tristate_te_re_compute(dev_data_t* dev)
{
	u32	bcr;

	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);
	bcr	|= D31;
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, bcr);
	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (bcr & D31)
	{
		dev->cache.tristate_te_re	= 1;
		bcr	&= ~D31;
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, bcr);
	}
	else
	{
		dev->cache.tristate_te_re	= 0;
	}
}



//*****************************************************************************
static void _tx_auto_stop_compute(dev_data_t* dev)
{
	if (dev->cache.gsc_fsr_32 & D6)
		dev->cache.tx_auto_stop	= 1;
	else
		dev->cache.tx_auto_stop	= 0;
}



//*****************************************************************************
static void _user_jumpers_compute(dev_data_t* dev)
{
	if (dev->cache.bus_width == 64)
	{
		// The User Jumpers are supported on all 64-bit devices.
		dev->cache.user_jumpers	= 2;
	}
	else if (dev->cache.gsc_frr_32 & HPDI32_FRR_CONFORMANT)
	{
		// User Jumpers are unsupported on 32-bit PMC devices
		// with the updated FRR format.

		if (dev->cache.form_factor == HPDI32_QUERY_FF_PCI)
		{
			dev->cache.user_jumpers	= 2;
		}
		else
		{
			dev->cache.user_jumpers	= 0;
		}
	}
	else if (dev->cache.gsc_frr_32 & D24)	// 32-bit device: PMC form factor
	{
		// User Jumpers are unsupported on 32-bit PMC devices
		// with the older FRR format.
		dev->cache.user_jumpers	= 0;
	}
	else
	{
		dev->cache.user_jumpers	= 2;
	}
}



//*****************************************************************************
static void _xcvr_compute(dev_data_t* dev)
{
	u32	bsr;

	if ((dev->cache.model == HPDI32_QUERY_MODEL_HPDI32A) ||
		(dev->cache.model == HPDI32_QUERY_MODEL_HPDI32B))
	{
		dev->cache.bsr_d18_xcvr	= 1;
		bsr						= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
		dev->cache.xcvr_type	= (bsr & 0x40000)
								? HPDI32_QUERY_XCVR_PECL
								: HPDI32_QUERY_XCVR_RS485;
	}
	else
	{
		dev->cache.bsr_d18_xcvr	= 0;
		dev->cache.xcvr_type	= HPDI32_QUERY_XCVR_UNKNOWN;
	}
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Do everything needed to setup and use the given device.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	u32	dma0;
	u32	dma1;
	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(HPDI32_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_acr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_ACR));
		dev->vaddr.gsc_bcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_BCR));
		dev->vaddr.gsc_bsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_BSR));
		dev->vaddr.gsc_fdr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_FDR));
		dev->vaddr.gsc_frr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_FRR));
		dev->vaddr.gsc_fsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_FSR));
		dev->vaddr.gsc_icr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_ICR));
		dev->vaddr.gsc_ielr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_IELR));
		dev->vaddr.gsc_ihlr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_IHLR));
		dev->vaddr.gsc_isr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_ISR));
		dev->vaddr.gsc_rar_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_RAR));
		dev->vaddr.gsc_rfsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_RFSR));
		dev->vaddr.gsc_rfwr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_RFWR));
		dev->vaddr.gsc_rlcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_RLCR));
		dev->vaddr.gsc_tar_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_TAR));
		dev->vaddr.gsc_tfsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_TFSR));
		dev->vaddr.gsc_tfwr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_TFWR));

		dev->vaddr.gsc_rscr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_RSCR));
		dev->vaddr.gsc_tcdr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_TCDR));
		dev->vaddr.gsc_tlilcr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_TLILCR));
		dev->vaddr.gsc_tlvlcr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_TLVLCR));
		dev->vaddr.gsc_tsvlcr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(HPDI32_GSC_TSVLCR));

		// Data cache initialization.

		dev->cache.gsc_frr_32		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_frr_32);
		dev->cache.gsc_fsr_32		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_fsr_32);

		dev->cache.tx_clock_div_max	= 0xFFFF;
		dev->cache.tx_clock_div_min	= 0;

		_fsr_compute(dev);					// 1
		_form_factor_compute(dev);			// 2
		_model_compute(dev);				// 3
		_fifos_compute(dev);				// 4
		_bus_width_compute(dev);			// 5
		_single_cycle_disable_compute(dev);
		_dma_compute(dev);
		_gpio_compute(dev);
		_irq_compute(dev);
		_over_under_run_compute(dev);
		_tx_auto_stop_compute(dev);
		_user_jumpers_compute(dev);
		_clock_max_compute(dev);
		_xcvr_compute(dev);
		_reg_tcdr_compute(dev);
		_tristate_te_re_compute(dev);
		_fifo_reset_fast_compute(dev);		// after model computed
		_proc_id_str_compute(dev);			// last

		// Initialize additional resources.
		ret	= dev_irq_create(dev);
		if (ret)	break;

		ret	= dev_io_create(dev);
		if (ret)	break;

		if (dev->cache.dmdma_ch_1)
		{
			dma0	= GSC_DMA_SEL_STATIC
					| GSC_DMA_CAP_BMDMA_READ
					| GSC_DMA_CAP_DMDMA_READ;
			dma1	= GSC_DMA_SEL_STATIC
					| GSC_DMA_CAP_BMDMA_WRITE
					| GSC_DMA_CAP_DMDMA_WRITE;
		}
		else
		{
			dma0	= GSC_DMA_SEL_DYNAMIC
					| GSC_DMA_CAP_BMDMA_READ
					| GSC_DMA_CAP_BMDMA_WRITE
					| GSC_DMA_CAP_DMDMA_READ
					| GSC_DMA_CAP_DMDMA_WRITE;
			dma1	= GSC_DMA_SEL_STATIC
					| GSC_DMA_CAP_BMDMA_READ
					| GSC_DMA_CAP_BMDMA_WRITE;
		}

		ret	= gsc_dma_create(dev, dma0, dma1);
		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		gsc_dma_destroy(dev);
		dev_io_destroy(dev);
		dev_irq_destroy(dev);
		gsc_bar_destroy(&dev->bar);
		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);
		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


