// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/driver/ioctl.c $
// $Rev: 53399 $
// $Date: 2023-07-12 12:04:34 -0500 (Wed, 12 Jul 2023) $

// HPDI32: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	INIT_START	0x1



// prototypes *****************************************************************

static int _rx_fifo_ae(dev_data_t* dev, s32* arg);
static int _rx_fifo_af(dev_data_t* dev, s32* arg);
static int _tx_fifo_ae(dev_data_t* dev, s32* arg);
static int _tx_fifo_af(dev_data_t* dev, s32* arg);



// variables ******************************************************************

static const s32	_cmd_state_options[]	=
{
	HPDI32_CABLE_CMD_STATE_0,
	HPDI32_CABLE_CMD_STATE_1,
	-1	// terminate list
};



//*****************************************************************************
static int _arg_s32_list_mask_reg(
	dev_data_t*	dev,
	s32*		arg,	// aligned with bit 0
	const s32*	list,
	u32			mask,	// aligned with register bits
	u32			reg,
	int			end)
{
	int			i;
	u16			offset;
	int			ret		= -EINVAL;
	u32			v;
	VADDR_T		vaddr;

	offset	= GSC_REG_OFFSET(reg);
	vaddr	= GSC_VADDR(dev, offset);

	if (arg[0] == -1)
	{
		ret	= 0;
		v	= os_reg_mem_rx_u32(dev, vaddr);
		v	&= mask;
		v	>>= end;

		for (i = 0; list[i] >= 0; i++)
		{
			if (list[i] == v)
			{
				arg[0]	= v;
				break;
			}
		}
	}
	else
	{
		for (i = 0; list[i] >= 0; i++)
		{
			if (list[i] == arg[0])
			{
				ret	= 0;
				v	= os_reg_mem_rx_u32(dev, vaddr);
				v	&= ~mask;
				v	|= (list[i] << end);
				os_reg_mem_tx_u32(dev, vaddr, v);
				break;
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	switch (arg[0])
	{
		default:							arg[0]	= HPDI32_IOCTL_QUERY_ERROR;			break;
		case HPDI32_QUERY_BSR_D18_XCVR:		arg[0]	= dev->cache.bsr_d18_xcvr;			break;
		case HPDI32_QUERY_BUS_WIDTH:		arg[0]	= dev->cache.bus_width;				break;
		case HPDI32_QUERY_CLOCK_MAX:		arg[0]	= dev->cache.clock_max;				break;
		case HPDI32_QUERY_COUNT:			arg[0]	= HPDI32_IOCTL_QUERY_LAST;			break;
		case HPDI32_QUERY_DEVICE_TYPE:		arg[0]	= dev->board_type;					break;
		case HPDI32_QUERY_DMDMA_1:			arg[0]	= dev->cache.dmdma_ch_1;			break;
		case HPDI32_QUERY_FIFO_RESET_FAST:	arg[0]	= dev->cache.fifo_reset_fast;		break;
		case HPDI32_QUERY_FIFO_SIZE_RX:		arg[0]	= dev->cache.fifo_size_rx;			break;
		case HPDI32_QUERY_FIFO_SIZE_TX:		arg[0]	= dev->cache.fifo_size_tx;			break;
		case HPDI32_QUERY_FORM_FACTOR:		arg[0]	= dev->cache.form_factor;			break;
		case HPDI32_QUERY_GPIO_0_5:			arg[0]	= dev->cache.gpio_0_5;				break;
		case HPDI32_QUERY_GPIO_6:			arg[0]	= dev->cache.gpio_6;				break;
		case HPDI32_QUERY_GPIO_D32:			arg[0]	= dev->cache.gpio_d32;				break;
		case HPDI32_QUERY_MODEL:			arg[0]	= dev->cache.model;					break;
		case HPDI32_QUERY_OVER_UNDER_RUN:	arg[0]	= dev->cache.over_under_run;		break;
		case HPDI32_QUERY_REG_ACR:			arg[0]	= dev->cache.reg_acr;				break;
		case HPDI32_QUERY_REG_FSR:			arg[0]	= dev->cache.reg_fsr;				break;
		case HPDI32_QUERY_REG_FWR_UPDATE:	arg[0]	= dev->cache.reg_fwr_update;		break;
		case HPDI32_QUERY_REG_GIR:			arg[0]	= dev->cache.reg_gir;				break;
		case HPDI32_QUERY_REG_GOR:			arg[0]	= dev->cache.reg_gor;				break;
		case HPDI32_QUERY_REG_IELR:			arg[0]	= dev->cache.reg_ielr;				break;
		case HPDI32_QUERY_REG_IHLR:			arg[0]	= dev->cache.reg_ihlr;				break;
		case HPDI32_QUERY_REG_RFSR:			arg[0]	= dev->cache.reg_rfsr;				break;
		case HPDI32_QUERY_REG_RFWR:			arg[0]	= dev->cache.reg_rfwr;				break;
		case HPDI32_QUERY_REG_TCDR:			arg[0]	= dev->cache.reg_tcdr;				break;
		case HPDI32_QUERY_REG_TFSR:			arg[0]	= dev->cache.reg_tfsr;				break;
		case HPDI32_QUERY_REG_TFWR:			arg[0]	= dev->cache.reg_tfwr;				break;
		case HPDI32_QUERY_SINGLE_CYC_DIS:	arg[0]	= dev->cache.single_cycle_disable;	break;
		case HPDI32_QUERY_TRISTATE_TE_RE:	arg[0]	= dev->cache.tristate_te_re;		break;
		case HPDI32_QUERY_TX_AUTO_STOP:		arg[0]	= dev->cache.tx_auto_stop;			break;
		case HPDI32_QUERY_TX_CLOCK_DIV_MAX:	arg[0]	= dev->cache.tx_clock_div_max;		break;
		case HPDI32_QUERY_TX_CLOCK_DIV_MIN:	arg[0]	= dev->cache.tx_clock_div_min;		break;
		case HPDI32_QUERY_USER_JUMPERS:		arg[0]	= dev->cache.user_jumpers;			break;
		case HPDI32_QUERY_XCVR_TYPE:		arg[0]	= dev->cache.xcvr_type;				break;
	}

	return(0);
}



//*****************************************************************************
static int _tx_auto_stop(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_AUTO_STOP_NO,
		HPDI32_AUTO_STOP_YES,
		-1	// terminate list
	};

	int	ret;

	// This refers to the Tx Start Auto Clear Disable feature.

	if (dev->cache.tx_auto_stop)
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 6, 6);
	}
	else
	{
		if (arg[0] == -1)
		{
			// The firmware auto-clears the Tx Start bit when the Tx FIFO goes empty.
			ret		= 0;
			arg[0]	= HPDI32_AUTO_STOP_YES;
		}
		else
		{
			// The feature is not configurable.
			arg[0]	= -1;
			ret		= 0;
		}
	}

	return(ret);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	s32		almost;
	int		i;
	long	ms		= 2;
	u32		reg;
	int		ret		= 0;
	s32		set;
	int		tmp;
	VADDR_T	va		= dev->vaddr.gsc_bcr_32;

	// The HPDI32 doesn't have an Initialization Done interrupt so we don't
	// involve interrupts here.

	// Initiate initialization.
	os_reg_mem_tx_u32(dev, va, INIT_START);

	// Manually wait for completion.
	tmp	= gsc_poll_u32(dev, ms, va, INIT_START, 0);

	if (tmp)
	{
		ret	= ret ? ret : tmp;
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion.
	reg	= os_reg_mem_rx_u32(dev, va);

	if (reg & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Initialize the software settings.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_sw_init)
			{
				(dev->io.io_streams[i]->dev_io_sw_init)(dev, dev->io.io_streams[i]);
			}
		}
	}

	// Manually set the FIFO Almost Empty and Almost Full values.
	// This is done because some old devices don't set these.
	almost	= 15;
	almost	= _rx_fifo_ae(dev, &almost);
	ret		= ret ? ret : almost;

	almost	= 16;
	almost	= _rx_fifo_af(dev, &almost);
	ret		= ret ? ret : almost;

	almost	= 15;
	almost	= _tx_fifo_ae(dev, &almost);
	ret		= ret ? ret : almost;

	almost	= 16;
	almost	= _tx_fifo_af(dev, &almost);
	ret		= ret ? ret : almost;

	if (dev->cache.tx_auto_stop)
	{
		set	= HPDI32_AUTO_STOP_NO;
		ret	= _tx_auto_stop(dev, &set);
	}

	if (dev->cache.single_cycle_disable)
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D11, D11);

	return(ret);
}



//*****************************************************************************
static int _cable_cmd_mode_x(dev_data_t* dev, s32* arg, int index)
{
	static const s32	options[]	=
	{
		HPDI32_CABLE_CMD_MODE_FC,
		HPDI32_CABLE_CMD_MODE_IN,
		HPDI32_CABLE_CMD_MODE_OUT_LO,
		HPDI32_CABLE_CMD_MODE_OUT_HI,
		-1	// terminate list
	};

	int	end;
	u32	mask	= 0x1010000 << index;
	int	ret;

	if ((index >= 0) && (index <= 5) && (dev->cache.gpio_0_5 == 0))
	{
		// This GPIO option isn't supported,
		// so the signal has the Flow Control functionality.
		arg[0]	= HPDI32_CABLE_CMD_MODE_FC;
		ret		= 0;
	}
	else if ((index == 6) && (dev->cache.gpio_6 == 0))
	{
		// This GPIO option isn't supported,
		// so the signal has the Flow Control functionality.
		arg[0]	= HPDI32_CABLE_CMD_MODE_FC;
		ret		= 0;
	}
	else
	{
		end	= 16 + index;
		ret	= _arg_s32_list_mask_reg(dev, arg, options, mask, HPDI32_GSC_BCR, end);
	}

	return(ret);
}



//*****************************************************************************
static int _cable_cmd_mode_0(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _cable_cmd_mode_x(dev, arg, 0);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_mode_1(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _cable_cmd_mode_x(dev, arg, 1);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_mode_2(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _cable_cmd_mode_x(dev, arg, 2);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_mode_3(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _cable_cmd_mode_x(dev, arg, 3);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_mode_4(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _cable_cmd_mode_x(dev, arg, 4);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_mode_5(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _cable_cmd_mode_x(dev, arg, 5);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_mode_6(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _cable_cmd_mode_x(dev, arg, 6);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_state_0(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, _cmd_state_options, dev->vaddr.gsc_bsr_32, 0, 0);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_state_1(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, _cmd_state_options, dev->vaddr.gsc_bsr_32, 1, 1);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_state_2(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, _cmd_state_options, dev->vaddr.gsc_bsr_32, 2, 2);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_state_3(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, _cmd_state_options, dev->vaddr.gsc_bsr_32, 3, 3);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_state_4(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, _cmd_state_options, dev->vaddr.gsc_bsr_32, 4, 4);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_state_5(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, _cmd_state_options, dev->vaddr.gsc_bsr_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _cable_cmd_state_6(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, _cmd_state_options, dev->vaddr.gsc_bsr_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _rx_auto_start(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_AUTO_START_NO,
		HPDI32_AUTO_START_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.auto_start);
	return(ret);
}



//*****************************************************************************
static int _rx_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_RX_ENABLE_NO,
		HPDI32_RX_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _rx_line_count(dev_data_t* dev, s32* arg)
{
	int	ret;

	// The value is returned to the user as a u32, but we're using an s32 here.
	arg[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFFFF, dev->vaddr.gsc_rlcr_32, 31, 0);
	return(ret);
}



//*****************************************************************************
static int _rx_status_count(dev_data_t* dev, s32* arg)
{
	int	ret;

	// The value is returned to the user as a u32, but we're using an s32 here.
	arg[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFFFF, dev->vaddr.gsc_rscr_32, 31, 0);
	return(ret);
}



//*****************************************************************************
static int _rx_fifo_reset(dev_data_t* dev, void* arg)
{
	#undef	RESET
	#define	RESET	D2
	#undef	STATUS
	#define	STATUS	(HPDI32_BSR_RX_OVERRUN | HPDI32_BSR_RX_UNDERRUN)

	u32	bcr;
	int	i;
	int	ret		= 0;

	if (dev->cache.fifo_reset_fast)
	{
		// The Rx FIFO is SRAM based so things should go very quickly.
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, 0xFFFFFFFF, RESET);

		for (i = 1;; i++)
		{
			bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

			if ((bcr & RESET) == 0)
				break;

			if (i >= 100)
			{
				ret	= -EINVAL;
				printf("%s: The Rx FIFO took too long to reset.\n", DEV_NAME);
				break;
			}
		}
	}
	else
	{
		// The Rx FIFO is a physical FIFO chip and may take longer to reset.
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, 0xFFFFFFFF, HPDI32_BCR_RX_FIFO_RESET);
		os_time_us_delay(10);
		gsc_poll_u32(dev, 10, dev->vaddr.gsc_bcr_32, HPDI32_BCR_RX_FIFO_RESET, 0);
	}

	// Clear the Overrun and Underrun status bits. They should clear within one
	// register read cycle. We don't poll for completion as they could
	// conceivably become set if we're suspended and I/O activity continues.
	// Were this to occur, the polling could needlessly continue till it
	// timesout.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bsr_32, 0xFFFFFFFF, STATUS);

	return(ret);
}



//*****************************************************************************
static int _rx_fifo_ae(dev_data_t* dev, s32* arg)
{
	int	ret;
	s32	v	= arg[0];

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_rar_32, 15, 0);

	// Reset the FIFO so the setting will take affect.

	if ((ret == 0) && (v >= 0))
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, 0xFFFFFFFF, HPDI32_BCR_RX_FIFO_RESET);
		os_time_us_delay(10);
		gsc_poll_u32(dev, 10, dev->vaddr.gsc_bcr_32, HPDI32_BCR_RX_FIFO_RESET, 0);
	}

	return(ret);
}



//*****************************************************************************
static int _rx_fifo_af(dev_data_t* dev, s32* arg)
{
	int	ret;
	s32	v	= arg[0];

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_rar_32, 31,16);

	// Reset the FIFO so the setting will take affect.

	if ((ret == 0) && (v >= 0))
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, 0xFFFFFFFF, HPDI32_BCR_RX_FIFO_RESET);
		os_time_us_delay(10);
		gsc_poll_u32(dev, 10, dev->vaddr.gsc_bcr_32, HPDI32_BCR_RX_FIFO_RESET, 0);
	}

	return(ret);
}



//*****************************************************************************
static int _rx_fifo_overrun(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_FIFO_ERROR_NO,
		HPDI32_FIFO_ERROR_YES,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.over_under_run)
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 23, 23);

		if ((ret == 0) && (arg[0] != -1))
		{
			arg[0]	= -1;
			ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 23, 23);
		}
	}
	else
	{
		arg[0]	= -1;
		ret		= 0;
	}

	return(ret);
}



//*****************************************************************************
static int _rx_fifo_status(dev_data_t* dev, s32* arg)
{
	u32	bsr;

	bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);

	if ((bsr & HPDI32_BSR_RX_NOT_FULL) == 0)
		arg[0]	= HPDI32_FIFO_STATUS_FULL;
	else if ((bsr & HPDI32_BSR_RX_NOT_AF) == 0)
		arg[0]	= HPDI32_FIFO_STATUS_ALMOST_FULL;
	else if ((bsr & HPDI32_BSR_RX_NOT_EMPTY) == 0)
		arg[0]	= HPDI32_FIFO_STATUS_EMPTY;
	else if ((bsr & HPDI32_BSR_RX_NOT_AE) == 0)
		arg[0]	= HPDI32_FIFO_STATUS_ALMOST_EMPTY;
	else
		arg[0]	= HPDI32_FIFO_STATUS_MEDIUM;

	return(0);
}



//*****************************************************************************
static int _rx_fifo_underrun(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_FIFO_ERROR_NO,
		HPDI32_FIFO_ERROR_YES,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.over_under_run)
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 22, 22);

		if ((ret == 0) && (arg[0] != -1))
		{
			arg[0]	= -1;
			ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 22, 22);
		}
	}
	else
	{
		arg[0]	= -1;
		ret		= 0;
	}

	return(ret);
}



//*****************************************************************************
static int _rx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_read_abort_active_xfer(dev, &dev->io.rx);
	return(0);
}



//*****************************************************************************
static int _rx_io_data_size(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_IO_DATA_SIZE_8_BITS,
		HPDI32_IO_DATA_SIZE_16_BITS,
		HPDI32_IO_DATA_SIZE_32_BITS,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.data_size_bits);
	dev->io.rx.bytes_per_sample	= dev->io.rx.data_size_bits / 8;

	return(ret);
}



//*****************************************************************************
static int _rx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.rx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _rx_io_overrun(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_IO_ERROR_IGNORE,
		HPDI32_IO_ERROR_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.overrun_check);
	return(ret);
}



//*****************************************************************************
static int _rx_io_pio_threshold(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret		= gsc_s32_range_var(arg, 0, S32_MAX, &dev->io.rx.pio_threshold);
	return(ret);
}



//*****************************************************************************
static int _rx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			HPDI32_IO_TIMEOUT_MIN,
			HPDI32_IO_TIMEOUT_INFINITE,
			&dev->io.rx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _rx_io_underrun(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_IO_ERROR_IGNORE,
		HPDI32_IO_ERROR_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.underrun_check);
	return(ret);
}



//*****************************************************************************
static int _tx_auto_start(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_AUTO_START_NO,
		HPDI32_AUTO_START_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.tx.auto_start);
	return(ret);
}



//*****************************************************************************
static int _tx_clock_divider(dev_data_t* dev, s32* arg)
{
	int	ret;

	if (dev->cache.reg_tcdr)
	{
		ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_tcdr_32, 15, 0);
	}
	else
	{
		arg[0]	= -1;
		ret		= 0;
	}

	return(ret);
}



//*****************************************************************************
static int _tx_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_TX_ENABLE_NO,
		HPDI32_TX_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 4, 4);
	return(ret);
}



//*****************************************************************************
static int _tx_flow_control(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_TX_FLOW_CONTROL_STOP,
		HPDI32_TX_FLOW_CONTROL_START,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 8, 8);
	return(ret);
}



//*****************************************************************************
static int _tx_line_val_off_cnt(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_tlilcr_32, 15, 0);
	return(ret);
}



//*****************************************************************************
static int _tx_line_val_on_cnt(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, S32_MAX, dev->vaddr.gsc_tlvlcr_32, 31, 0);
	return(ret);
}



//*****************************************************************************
static int _tx_remote_throttle(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_TX_REMOTE_THROTTLE_NO,
		HPDI32_TX_REMOTE_THROTTLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 9, 9);
	return(ret);
}



//*****************************************************************************
static int _tx_status_val_cnt(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, S32_MAX, dev->vaddr.gsc_tsvlcr_32, 31, 0);
	return(ret);
}



//*****************************************************************************
static int _tx_status_val_mir(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_TX_STATUS_VAL_MIR_NO,
		HPDI32_TX_STATUS_VAL_MIR_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 7, 7);
	return(ret);
}



//*****************************************************************************
static int _tx_fifo_reset(dev_data_t* dev, void* arg)
{
	#undef	RESET
	#define	RESET	D1
	#undef	STATUS
	#define	STATUS	HPDI32_BSR_TX_OVERRUN

	u32	bcr;
	int	i;
	int	ret		= 0;

	if (dev->cache.fifo_reset_fast)
	{
		// The Tx FIFO is SRAM based so things should go very quickly.
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, 0xFFFFFFFF, RESET);

		for (i = 1;; i++)
		{
			bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

			if ((bcr & RESET) == 0)
				break;

			if (i >= 100)
			{
				ret	= -EINVAL;
				printf("%s: The Tx FIFO took too long to reset.\n", DEV_NAME);
				break;
			}
		}
	}
	else
	{
		// The Tx FIFO is a physical FIFO chip and may take longer to reset.
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, 0xFFFFFFFF, HPDI32_BCR_TX_FIFO_RESET);
		os_time_us_delay(10);
		gsc_poll_u32(dev, 10, dev->vaddr.gsc_bcr_32, HPDI32_BCR_TX_FIFO_RESET, 0);
	}

	// Clear the Overrun status bit. It should clear within one register read
	// cycle. We don't poll for completion as it could conceivably become set
	// if we're suspended and I/O activity continues. Were this to occur,
	// the polling could needlessly continue till it timesout.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bsr_32, 0xFFFFFFFF, STATUS);
	gsc_poll_u32(dev, 10, dev->vaddr.gsc_bsr_32, STATUS, 0);
	return(ret);
}



//*****************************************************************************
static int _tx_fifo_ae(dev_data_t* dev, s32* arg)
{
	int	ret;
	s32	v	= arg[0];

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_tar_32, 15, 0);

	// Reset the FIFO so the setting will take affect.

	if ((ret == 0) && (v >= 0))
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, 0xFFFFFFFF, HPDI32_BCR_TX_FIFO_RESET);
		os_time_us_delay(10);
		gsc_poll_u32(dev, 10, dev->vaddr.gsc_bcr_32, HPDI32_BCR_TX_FIFO_RESET, 0);
	}

	return(ret);
}



//*****************************************************************************
static int _tx_fifo_af(dev_data_t* dev, s32* arg)
{
	int	ret;
	s32	v	= arg[0];

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_tar_32, 31,16);

	// Reset the FIFO so the setting will take affect.

	if ((ret == 0) && (v >= 0))
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, 0xFFFFFFFF, HPDI32_BCR_TX_FIFO_RESET);
		os_time_us_delay(10);
		gsc_poll_u32(dev, 10, dev->vaddr.gsc_bcr_32, HPDI32_BCR_TX_FIFO_RESET, 0);
	}

	return(ret);
}



//*****************************************************************************
static int _tx_fifo_overrun(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_FIFO_ERROR_NO,
		HPDI32_FIFO_ERROR_YES,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.over_under_run)
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 21, 21);

		if ((ret == 0) && (arg[0] != -1))
		{
			arg[0]	= -1;
			ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 21, 21);
		}
	}
	else
	{
		arg[0]	= -1;
		ret		= 0;
	}

	return(ret);
}



//*****************************************************************************
static int _tx_fifo_status(dev_data_t* dev, s32* arg)
{
	u32	bsr;

	bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);

	if ((bsr & HPDI32_BSR_TX_NOT_FULL) == 0)
		arg[0]	= HPDI32_FIFO_STATUS_FULL;
	else if ((bsr & HPDI32_BSR_TX_NOT_AF) == 0)
		arg[0]	= HPDI32_FIFO_STATUS_ALMOST_FULL;
	else if ((bsr & HPDI32_BSR_TX_NOT_EMPTY) == 0)
		arg[0]	= HPDI32_FIFO_STATUS_EMPTY;
	else if ((bsr & HPDI32_BSR_TX_NOT_AE) == 0)
		arg[0]	= HPDI32_FIFO_STATUS_ALMOST_EMPTY;
	else
		arg[0]	= HPDI32_FIFO_STATUS_MEDIUM;

	return(0);
}



//*****************************************************************************
static int _tx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_write_abort_active_xfer(dev, &dev->io.tx);
	return(0);
}



//*****************************************************************************
static int _tx_io_data_size(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_IO_DATA_SIZE_8_BITS,
		HPDI32_IO_DATA_SIZE_16_BITS,
		HPDI32_IO_DATA_SIZE_32_BITS,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.tx.data_size_bits);
	dev->io.tx.bytes_per_sample	= dev->io.tx.data_size_bits / 8;

	return(ret);
}



//*****************************************************************************
static int _tx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.tx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _tx_io_pio_threshold(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret		= gsc_s32_range_var(arg, 0, S32_MAX, &dev->io.tx.pio_threshold);
	return(ret);
}



//*****************************************************************************
static int _tx_io_overrun(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_IO_ERROR_IGNORE,
		HPDI32_IO_ERROR_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.tx.overrun_check);
	return(ret);
}



//*****************************************************************************
static int _tx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			HPDI32_IO_TIMEOUT_MIN,
			HPDI32_IO_TIMEOUT_INFINITE,
			&dev->io.tx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _irq_config_edge(dev_data_t* dev, s32* arg)
{
	u32	ielr;
	int	ret;

	if (dev->cache.reg_ielr == 0)
	{
		arg[0]	= -1;
		ret		= 0;
	}
	else if (arg[0] == -1)
	{
		ret		= 0;
		ielr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ielr_32);
		arg[0]	= ielr & 0xFFFF;
	}
	else if ((arg[0] & 0xFFFF) == arg[0])
	{
		ret		= 0;
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_ielr_32, arg[0], 0xFFFF);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _irq_config_high(dev_data_t* dev, s32* arg)
{
	u32	ihlr;
	int	ret;

	if (dev->cache.reg_ihlr == 0)
	{
		arg[0]	= -1;
		ret		= 0;
	}
	else if (arg[0] == -1)
	{
		ret		= 0;
		ihlr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ihlr_32);

		arg[0]	= ihlr & 0xFFFF;
	}
	else if ((arg[0] & 0xFFFF) == arg[0])
	{
		ret		= 0;
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_ihlr_32, arg[0], 0xFFFF);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _irq_enable_ioctl(dev_data_t* dev, s32* arg)
{
	u32	icr;
	int	ret;

	if (arg[0] == -1)
	{
		ret	= 0;
		icr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_icr_32);
		arg[0]	= icr & 0xFFFF;
	}
	else if ((arg[0] & 0xFFFF) == arg[0])
	{
		ret		= 0;
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_icr_32, arg[0], 0xFFFF);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _user_jumpers(dev_data_t* dev, s32* arg)
{
	int	ret;

	if (dev->cache.user_jumpers == 0)
	{
		ret		= 0;
		arg[0]	= 0;
	}
	else
	{
		arg[0]	= -1;
		ret		= gsc_s32_range_reg(dev, arg, 0, 0x3, dev->vaddr.gsc_bsr_32, 17, 16);
	}

	return(ret);
}



//*****************************************************************************
static int _tristate_te_re(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_TRISTATE_TE_RE_NO,
		HPDI32_TRISTATE_TE_RE_YES,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.tristate_te_re)
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 31, 31);
	}
	else
	{
		arg[0]	= -1;
		ret		= 0;
	}

	return(ret);
}



//*****************************************************************************
static int _rx_io_bmdma_thresh(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret		= gsc_s32_range_var(arg, 0, dev->cache.fifo_size_rx, &dev->io.rx.bmdma_threshold);
	return(ret);
}



//*****************************************************************************
static int _tx_io_bmdma_thresh(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret		= gsc_s32_range_var(arg, 0, dev->cache.fifo_size_tx, &dev->io.tx.bmdma_threshold);
	return(ret);
}



//*****************************************************************************
static int _gpio_d32_output(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32_GPIO_D32_OUTPUT_DISABLE,
		HPDI32_GPIO_D32_OUTPUT_ENABLE,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.gpio_d32)
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_acr_32, 0, 0);
	}
	else
	{
		arg[0]	= -1;
		ret		= 0;
	}

	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ HPDI32_IOCTL_REG_READ,			(void*) gsc_reg_read_ioctl		},
	{ HPDI32_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ HPDI32_IOCTL_REG_MOD,				(void*) gsc_reg_mod_ioctl		},
	{ HPDI32_IOCTL_QUERY,				(void*) _query					},
	{ HPDI32_IOCTL_INITIALIZE,			(void*) initialize_ioctl		},
	{ HPDI32_IOCTL_CABLE_CMD_MODE_0,	(void*) _cable_cmd_mode_0		},
	{ HPDI32_IOCTL_CABLE_CMD_MODE_1,	(void*) _cable_cmd_mode_1		},
	{ HPDI32_IOCTL_CABLE_CMD_MODE_2,	(void*) _cable_cmd_mode_2		},
	{ HPDI32_IOCTL_CABLE_CMD_MODE_3,	(void*) _cable_cmd_mode_3		},
	{ HPDI32_IOCTL_CABLE_CMD_MODE_4,	(void*) _cable_cmd_mode_4		},
	{ HPDI32_IOCTL_CABLE_CMD_MODE_5,	(void*) _cable_cmd_mode_5		},
	{ HPDI32_IOCTL_CABLE_CMD_MODE_6,	(void*) _cable_cmd_mode_6		},
	{ HPDI32_IOCTL_CABLE_CMD_STATE_0,	(void*) _cable_cmd_state_0		},
	{ HPDI32_IOCTL_CABLE_CMD_STATE_1,	(void*) _cable_cmd_state_1		},
	{ HPDI32_IOCTL_CABLE_CMD_STATE_2,	(void*) _cable_cmd_state_2		},
	{ HPDI32_IOCTL_CABLE_CMD_STATE_3,	(void*) _cable_cmd_state_3		},
	{ HPDI32_IOCTL_CABLE_CMD_STATE_4,	(void*) _cable_cmd_state_4		},
	{ HPDI32_IOCTL_CABLE_CMD_STATE_5,	(void*) _cable_cmd_state_5		},
	{ HPDI32_IOCTL_CABLE_CMD_STATE_6,	(void*) _cable_cmd_state_6		},
	{ HPDI32_IOCTL_RX_AUTO_START,		(void*) _rx_auto_start			},
	{ HPDI32_IOCTL_RX_ENABLE,			(void*) _rx_enable				},
	{ HPDI32_IOCTL_RX_LINE_COUNT,		(void*) _rx_line_count			},
	{ HPDI32_IOCTL_RX_STATUS_COUNT,		(void*) _rx_status_count		},
	{ HPDI32_IOCTL_RX_FIFO_AE,			(void*) _rx_fifo_ae				},
	{ HPDI32_IOCTL_RX_FIFO_AF,			(void*) _rx_fifo_af				},
	{ HPDI32_IOCTL_RX_FIFO_OVERRUN,		(void*) _rx_fifo_overrun		},
	{ HPDI32_IOCTL_RX_FIFO_RESET,		(void*) _rx_fifo_reset			},
	{ HPDI32_IOCTL_RX_FIFO_STATUS,		(void*) _rx_fifo_status			},
	{ HPDI32_IOCTL_RX_FIFO_UNDERRUN,	(void*) _rx_fifo_underrun		},
	{ HPDI32_IOCTL_RX_IO_ABORT,			(void*) _rx_io_abort			},
	{ HPDI32_IOCTL_RX_IO_DATA_SIZE,		(void*) _rx_io_data_size		},
	{ HPDI32_IOCTL_RX_IO_MODE,			(void*) _rx_io_mode				},
	{ HPDI32_IOCTL_RX_IO_OVERRUN,		(void*) _rx_io_overrun			},
	{ HPDI32_IOCTL_RX_IO_PIO_THRESHOLD,	(void*) _rx_io_pio_threshold	},
	{ HPDI32_IOCTL_RX_IO_TIMEOUT,		(void*) _rx_io_timeout			},
	{ HPDI32_IOCTL_RX_IO_UNDERRUN,		(void*) _rx_io_underrun			},
	{ HPDI32_IOCTL_TX_AUTO_START,		(void*) _tx_auto_start			},
	{ HPDI32_IOCTL_TX_AUTO_STOP,		(void*) _tx_auto_stop			},
	{ HPDI32_IOCTL_TX_CLOCK_DIVIDER,	(void*) _tx_clock_divider		},
	{ HPDI32_IOCTL_TX_ENABLE,			(void*) _tx_enable				},
	{ HPDI32_IOCTL_TX_FLOW_CONTROL,		(void*) _tx_flow_control		},
	{ HPDI32_IOCTL_TX_LINE_VAL_OFF_CNT,	(void*) _tx_line_val_off_cnt	},
	{ HPDI32_IOCTL_TX_LINE_VAL_ON_CNT,	(void*) _tx_line_val_on_cnt		},
	{ HPDI32_IOCTL_TX_REMOTE_THROTTLE,	(void*) _tx_remote_throttle		},
	{ HPDI32_IOCTL_TX_STATUS_VAL_CNT,	(void*) _tx_status_val_cnt		},
	{ HPDI32_IOCTL_TX_STATUS_VAL_MIR,	(void*) _tx_status_val_mir		},
	{ HPDI32_IOCTL_TX_FIFO_AE,			(void*) _tx_fifo_ae				},
	{ HPDI32_IOCTL_TX_FIFO_AF,			(void*) _tx_fifo_af				},
	{ HPDI32_IOCTL_TX_FIFO_OVERRUN,		(void*) _tx_fifo_overrun		},
	{ HPDI32_IOCTL_TX_FIFO_RESET,		(void*) _tx_fifo_reset			},
	{ HPDI32_IOCTL_TX_FIFO_STATUS,		(void*) _tx_fifo_status			},
	{ HPDI32_IOCTL_TX_IO_ABORT,			(void*) _tx_io_abort			},
	{ HPDI32_IOCTL_TX_IO_DATA_SIZE,		(void*) _tx_io_data_size		},
	{ HPDI32_IOCTL_TX_IO_MODE,			(void*) _tx_io_mode				},
	{ HPDI32_IOCTL_TX_IO_PIO_THRESHOLD,	(void*) _tx_io_pio_threshold	},
	{ HPDI32_IOCTL_TX_IO_OVERRUN,		(void*) _tx_io_overrun			},
	{ HPDI32_IOCTL_TX_IO_TIMEOUT,		(void*) _tx_io_timeout			},
	{ HPDI32_IOCTL_IRQ_CONFIG_EDGE,		(void*) _irq_config_edge		},
	{ HPDI32_IOCTL_IRQ_CONFIG_HIGH,		(void*) _irq_config_high		},
	{ HPDI32_IOCTL_IRQ_ENABLE,			(void*) _irq_enable_ioctl		},
	{ HPDI32_IOCTL_TRISTATE_TE_RE,		(void*) _tristate_te_re			},
	{ HPDI32_IOCTL_USER_JUMPERS,		(void*) _user_jumpers			},
	{ HPDI32_IOCTL_WAIT_EVENT,			(void*) gsc_wait_event_ioctl	},
	{ HPDI32_IOCTL_WAIT_CANCEL,			(void*) gsc_wait_cancel_ioctl	},
	{ HPDI32_IOCTL_WAIT_STATUS,			(void*) gsc_wait_status_ioctl	},
	{ HPDI32_IOCTL_RX_IO_BMDMA_THRESH,	(void*) _rx_io_bmdma_thresh		},
	{ HPDI32_IOCTL_TX_IO_BMDMA_THRESH,	(void*) _tx_io_bmdma_thresh		},
	{ HPDI32_IOCTL_GPIO_D32_OUTPUT,		(void*) _gpio_d32_output		},

	{ -1, NULL }
};


