// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/driver/irq.c $
// $Rev: 43908 $
// $Date: 2018-11-01 16:25:02 -0500 (Thu, 01 Nov 2018) $

// HPDI32: Device Driver: source file

#include "main.h"



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	u32	icr;
	u32	ielr;
	u32	isr;
	u32	level;
	u32	mask;

	icr		= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_icr_32);
	isr		= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_isr_32);
	mask	= icr & isr & 0xFFFF;

	if (mask)
	{
		// Service the respective interrupts.
		os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_isr_32, mask);

		if (dev->cache.reg_ielr)
		{
			// Disable the LEVEL triggerred interrupts.
			ielr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_ielr_32);
			level	= ~ielr & mask;

			if (level)
				os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_icr_32, 0, level);
		}
		else
		{
			// Disable the interrupts.
			os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_icr_32, 0, mask);
		}

		// Resume any threads awiting any of these interrupts.
		gsc_wait_resume_irq_gsc(dev, mask);
	}
	else
	{
		// We don't know the source of the interrupt.
		gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_icr_32, 0);			// Disable IRQs.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_isr_32, 0xFFFFFFFF);	// Clear IRQs.
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_icr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_icr_32, 0);			// Disable IRQs.

	if (dev->vaddr.gsc_isr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_isr_32, 0xFFFFFFFF);	// Clear IRQs.

	gsc_irq_destroy(dev);
}


