// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/b2btest/cable_command_4.c $
// $Rev: 51573 $
// $Date: 2022-10-04 17:45:10 -0500 (Tue, 04 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _config(int fd)
{
	int	errs	= 0;

	// Frame Valid is assumed to be functional.
	errs	+= hpdi32_initialize		(fd, -1, 0);
	errs	+= hpdi32_tristate_te_re	(fd, -1, 0, HPDI32_TRISTATE_TE_RE_YES,	NULL);
	errs	+= hpdi32_cable_cmd_mode_0	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_FC,	NULL);	// Frame Valid/GPIO 6
	errs	+= hpdi32_cable_cmd_mode_1	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Line Valid/GPIO 0
	errs	+= hpdi32_cable_cmd_mode_2	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Status Valid/GPIO 1
	errs	+= hpdi32_cable_cmd_mode_3	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Rx Ready/GPIO 2
	errs	+= hpdi32_cable_cmd_mode_4	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_FC,	NULL);	// Tx Ready/GPIO 3
	errs	+= hpdi32_cable_cmd_mode_5	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Tx Enabled/GPIO 4
	errs	+= hpdi32_cable_cmd_mode_6	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Rx Enabled/GPIO 5

	return(errs);
}



//*****************************************************************************
static int _tx_ready_test(int fd_tx)
{
	int	errs	= 0;
	s32	state;

	os_sleep_ms(2);

	// Verify that Tx Ready is high at the receiver device.
	os_sleep_ms(2);
	errs	+= hpdi32_cable_cmd_state_4(fd_tx, -1, 0, &state);

	if (state != HPDI32_CABLE_CMD_STATE_1)
		errs++;

	// Prepare the Tx FIFO in the transmit device.
	errs	+= hpdi32_reg_write(fd_tx, -1, 0, HPDI32_GSC_FDR, 0);

	// Verify that Tx Ready is low at the receiver device.
	os_sleep_ms(2);
	errs	+= hpdi32_cable_cmd_state_4(fd_tx, -1, 0, &state);

	if (state != HPDI32_CABLE_CMD_STATE_0)
		errs++;

	return(errs);
}



//*****************************************************************************
static int _flow_control_test(int fd_tx, int fd_rx)
{
	int	errs	= 0;
	int	i;

	gsc_label("Tx Ready");

	for (i = 0; (errs == 0) && (i < 10); i++)
	{
		errs	+= _config(fd_tx);
		errs	+= _config(fd_rx);

		errs	+= hpdi32_tx_enable(fd_tx, -1, 0, HPDI32_TX_ENABLE_YES, NULL);
		errs	+= hpdi32_rx_enable(fd_rx, -1, 0, HPDI32_RX_ENABLE_YES, NULL);

		errs	+= _tx_ready_test(fd_tx);

		errs	+= hpdi32_initialize(fd_tx, -1, 0);
		errs	+= hpdi32_initialize(fd_rx, -1, 0);
	}

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



//*****************************************************************************
static int _gpio_test(int fd_tx, int fd_rx)
{
	int	errs	= 0;
	s32	gpio_rx	= 0;
	s32	gpio_tx	= 0;
	int	i;
	s32	state;

	gsc_label("GPIO 3");

	errs	+= hpdi32_query(fd_tx, -1, 0, HPDI32_QUERY_GPIO_0_5, &gpio_tx);
	errs	+= hpdi32_query(fd_rx, -1, 0, HPDI32_QUERY_GPIO_0_5, &gpio_rx);

	if ((gpio_tx == 0) || (gpio_rx == 0))
	{
		printf(	"%s  (GPIO 3 not supported by both devices.)\n",
				errs ? "FAIL <---" : "SKIPPED");
	}
	else
	{
		errs	+= _config(fd_tx);
		errs	+= _config(fd_rx);
		errs	+= hpdi32_cable_cmd_mode_4(fd_tx, -1, 0, HPDI32_CABLE_CMD_MODE_IN, NULL);
		errs	+= hpdi32_cable_cmd_mode_4(fd_rx, -1, 0, HPDI32_CABLE_CMD_MODE_IN, NULL);

		for (i = 0; i < 10; i++)
		{
			// GPIO out Low
			errs	+= hpdi32_cable_cmd_mode_4(fd_tx, -1, 0, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
			os_sleep_ms(2);
			errs	+= hpdi32_cable_cmd_state_4(fd_rx, -1, 0, &state);

			if (state != HPDI32_CABLE_CMD_STATE_0)
				errs++;

			// GPIO out Low
			errs	+= hpdi32_cable_cmd_mode_4(fd_tx, -1, 0, HPDI32_CABLE_CMD_MODE_OUT_HI, NULL);
			os_sleep_ms(2);
			errs	+= hpdi32_cable_cmd_state_4(fd_rx, -1, 0, &state);

			if (state != HPDI32_CABLE_CMD_STATE_1)
				errs++;
		}

		errs	= errs ? 1 : 0;
		printf("%s\n", errs ? "FAIL <---" : "PASS");
	}

	return(errs);
}



//*****************************************************************************
int cable_command_4_test(int fd_tx, int fd_rx)
{
	int	errs	= 0;

	gsc_label("Cable Command 4");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _flow_control_test(fd_tx, fd_rx);
	errs	+= _gpio_test(fd_tx, fd_rx);

	gsc_label_level_dec();
	return(errs);
}


