// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/b2btest/cable_data.c $
// $Rev: 51573 $
// $Date: 2022-10-04 17:45:10 -0500 (Tue, 04 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _config(int fd)
{
	int	errs	= 0;

	// Frame Valid is assumed to be functional.
	errs	+= hpdi32_initialize		(fd, -1, 0);
	errs	+= hpdi32_tristate_te_re	(fd, -1, 0, HPDI32_TRISTATE_TE_RE_YES,	NULL);
	errs	+= hpdi32_cable_cmd_mode_0	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_FC,	NULL);	// Frame Valid/GPIO 6
	errs	+= hpdi32_cable_cmd_mode_1	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Line Valid/GPIO 0
	errs	+= hpdi32_cable_cmd_mode_2	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Status Valid/GPIO 1
	errs	+= hpdi32_cable_cmd_mode_3	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Rx Ready/GPIO 2
	errs	+= hpdi32_cable_cmd_mode_4	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Tx Ready/GPIO 3
	errs	+= hpdi32_cable_cmd_mode_5	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Tx Enabled/GPIO 4
	errs	+= hpdi32_cable_cmd_mode_6	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_IN,	NULL);	// Rx Enabled/GPIO 5

	errs	+= hpdi32_tx_status_val_cnt	(fd, -1, 0, 1, NULL);
	return(errs);
}



//*****************************************************************************
static int _transfer_test(int fd_tx, int fd_rx)
{
	int	errs	= 0;
	u32	fdr;
	int	i;
	u32	qty		= 0;

	os_sleep_ms(10);

	// Prepare the Rx device
	errs	+= hpdi32_rx_fifo_reset(fd_rx, -1, 0);

	// Prefill the Tx device.
	errs	+= hpdi32_tx_flow_control(fd_tx, -1, 0, HPDI32_TX_FLOW_CONTROL_STOP, NULL);

	// Transmit a walking "1".

	for (i = 0; i < 32; i++)
		errs	+= hpdi32_reg_write(fd_tx, -1, 0, HPDI32_GSC_FDR, (u32) (0x1 << i));

	// Transmit a walking "0".

	for (i = 0; i < 32; i++)
		errs	+= hpdi32_reg_write(fd_tx, -1, 0, HPDI32_GSC_FDR, (u32) ~(0x1 << i));

	// Send the data from te Tx device to the Rx device.
	errs	+= hpdi32_tx_flow_control(fd_tx, -1, 0, HPDI32_TX_FLOW_CONTROL_START, NULL);
	os_sleep_ms(10);

	// Verify tha data was received.
	errs	+= hpdi32_reg_read(fd_rx, -1, 0, HPDI32_GSC_RFWR, &qty);

	if (qty != 64)
		errs++;

	// Verify the received data.
	// Verify a walking "1".

	for (i = 0; i < 32; i++)
	{
		errs	+= hpdi32_reg_read(fd_rx, -1, 0, HPDI32_GSC_FDR, &fdr);

		if (fdr != ((u32) (0x1 << i)))
			errs++;
	}

	// Verify a walking "0".

	for (i = 0; i < 32; i++)
	{
		errs	+= hpdi32_reg_read(fd_rx, -1, 0, HPDI32_GSC_FDR, &fdr);

		if (fdr != ((u32) ~(0x1 << i)))
			errs++;
	}

	return(errs);
}



//*****************************************************************************
static int _streaming_test(int fd_tx, int fd_rx)
{
	int	errs	= 0;
	int	i;

	gsc_label("Streaming Data");

	for (i = 0; (errs == 0) && (i < 10); i++)
	{
		errs	+= _config(fd_tx);
		errs	+= _config(fd_rx);

		errs	+= hpdi32_tx_clock_divider	(fd_tx, -1, 0, 10, NULL);
		errs	+= hpdi32_tx_enable			(fd_tx, -1, 0, HPDI32_TX_ENABLE_YES, NULL);
		errs	+= hpdi32_rx_enable			(fd_rx, -1, 0, HPDI32_RX_ENABLE_YES, NULL);

		errs	+= _transfer_test(fd_tx, fd_rx);

		errs	+= hpdi32_initialize(fd_tx, -1, 0);
		errs	+= hpdi32_initialize(fd_rx, -1, 0);
	}

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



//*****************************************************************************
static int _gpio_test(int fd_tx, int fd_rx)
{
	int	errs	= 0;
	s32	gpio_rx	= 0;
	s32	gpio_tx	= 0;
	int	i;
	u32	rx_gir;
	u32	tx;
	u32	tx_gir;
	u32	tx_gor;

	gsc_label("GPIO D32");

	errs	+= hpdi32_query(fd_tx, -1, 0, HPDI32_QUERY_GPIO_D32, &gpio_tx);
	errs	+= hpdi32_query(fd_rx, -1, 0, HPDI32_QUERY_GPIO_D32, &gpio_rx);

	if ((gpio_tx == 0) && (gpio_rx == 0))
	{
		printf(	"%s  (GPIO D32 not supported by either device.)\n",
				errs ? "FAIL <---" : "SKIPPED");
	}
	else if ((gpio_tx == 0) || (gpio_rx == 0))
	{
		printf(	"%s  (GPIO D32 not supported by both devices.)\n",
				errs ? "FAIL <---" : "SKIPPED");
	}
	else
	{
		// Enable the D32 GPIO feature.
		errs	+= hpdi32_initialize(fd_tx, -1, 0);
		errs	+= hpdi32_initialize(fd_rx, -1, 0);
		errs	+= hpdi32_gpio_d32_output(fd_tx, -1, 0, HPDI32_GPIO_D32_OUTPUT_ENABLE, NULL);

		// Test a walking "1" pattern.

		for (i = 0; i < 32; i++)
		{
			tx		= (u32) (0x1 << i);
			errs	+= hpdi32_reg_write(fd_tx, -1, 0, HPDI32_GSC_GOR, tx);
			os_sleep_ms(2);
			errs	+= hpdi32_reg_read(fd_tx, -1, 0, HPDI32_GSC_GOR, &tx_gor);
			errs	+= hpdi32_reg_read(fd_tx, -1, 0, HPDI32_GSC_GIR, &tx_gir);
			errs	+= hpdi32_reg_read(fd_rx, -1, 0, HPDI32_GSC_GIR, &rx_gir);

			if ((tx != tx_gor) || (tx != tx_gir) || (tx != rx_gir))
				errs++;
		}

		// Test a walking "0" pattern.

		for (i = 0; i < 32; i++)
		{
			tx		= (u32) ~(0x1 << i);
			errs	+= hpdi32_reg_write(fd_tx, -1, 0, HPDI32_GSC_GOR, tx);
			os_sleep_ms(2);
			errs	+= hpdi32_reg_read(fd_tx, -1, 0, HPDI32_GSC_GOR, &tx_gor);
			errs	+= hpdi32_reg_read(fd_tx, -1, 0, HPDI32_GSC_GIR, &tx_gir);
			errs	+= hpdi32_reg_read(fd_rx, -1, 0, HPDI32_GSC_GIR, &rx_gir);

			if ((tx != tx_gor) || (tx != tx_gir) || (tx != rx_gir))
				errs++;
		}

		errs	= errs ? 1 : 0;
		printf("%s\n", errs ? "FAIL <---" : "PASS");
	}

	return(errs);
}



//*****************************************************************************
int cable_data_test(int fd_tx, int fd_rx)
{
	int	errs	= 0;

	gsc_label("Cable Data");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _streaming_test(fd_tx, fd_rx);
	errs	+= _gpio_test(fd_tx, fd_rx);

	gsc_label_level_dec();
	return(errs);
}


