// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/rx_simple/main.c $
// $Rev: 53386 $
// $Date: 2023-07-12 11:16:32 -0500 (Wed, 12 Jul 2023) $

// HPDI32: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	int	_index		= 0;
static	int	_tx_sim		= 0;



//*****************************************************************************
static int _parse_args(int argc, char** argv)
{
	char	c;
	int		errs	= 0;
	int		i;
	int		j;
	int		k;

	printf("rx_simple: A simple 'rx' sample application\n");
	printf("USAGE: rx_simple <-sim> <#>\n");
	printf("  -sim  Simulate a Tx device providing data. DETACH CABLE!\n");
	printf("  #     The index of the device to test.\n");

	printf("Arguments: ");
	printf("(%d argument%s)\n", argc - 1, ((argc - 1) == 1) ? "" : "s");

	for (i = 0; i < argc; i++)
	{
		printf("  Argument %d: %s\n", i, argv[i]);
	}

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-sim") == 0)
		{
			_tx_sim	= 1;
			continue;
		}

		j	= sscanf(argv[i], "%d%c", &k, &c);

		if ((j == 1) && (k >= 0))
		{
			_index	= k;
			continue;
		}

		errs	= 1;
		printf("ERROR: invalid argument: %s\n", argv[i]);
		break;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	int	fd		= -1;
	int	errs;
	int	ret;

	errs	= _parse_args(argc, argv);
	printf("\n");

	if (errs == 0)
	{
		// Initialize the API.
		printf("HPDI32 API Initialize: ");
		ret	= hpdi32_init();

		if (ret == 0)
		{
			printf("PASS\n");
		}
		else
		{
			printf("FAIL <---  (ret = %d)\n", ret);
			errs++;
		}
	}

	if (errs == 0)
	{
		// Open access to the device.
		printf("Opening HPDI32 #%d: ", (int) _index);
		ret	= hpdi32_open(_index, 0, &fd);

		if (ret == 0)
		{
			printf("PASS\n");
		}
		else
		{
			errs++;
			printf("FAIL <---\n");
		}
	}

	if (errs == 0)
	{
		// Perform the test.
		errs	+= perform_tests(fd, _tx_sim);

		// Close access to the device.
		printf("Closing HPDI32 #%d: ", _index);
		ret	= hpdi32_close(fd);

		if (ret == 0)
		{
			printf("PASS\n");
		}
		else
		{
			errs++;
			printf("FAIL <---  (ret = %d)\n", ret);
		}
	}

	// Report the results.
	printf("\nRESULTS: %s\n", errs ? "FAIL <---" : "PASS");

	// Exit the applicaton.
	ret	= errs ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


