// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/rx_simple/perform.c $
// $Rev: 52316 $
// $Date: 2022-12-20 15:49:05 -0600 (Tue, 20 Dec 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32	buffer[256L * 1024];	// 1MB of data.



/******************************************************************************
*
*	Function:	_config
*
*	Purpose:
*
*		Configure things in preparation for the test.
*
*	Arguments:
*
*		fd		The handle of the device to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _config(int fd)
{
	s32	arg;
	int	errs	= 0;
	int	ret;

	printf("Configuring:\n");

	// Initialize the device.
	printf("  Initialize: ");
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_INITIALIZE, NULL);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Select Demand Mode DMA.
	printf("  Rx I/O Mode: DMDMA: ");
	arg	= GSC_IO_MODE_DMDMA;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_IO_MODE, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Tristate these signals in case we are connected to another HPDI32.
	printf("  Tx/Rx Tristate Mode: Enable: ");
	arg	= HPDI32_TRISTATE_TE_RE_YES;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_TRISTATE_TE_RE, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Enable the receiver.
	printf("  Rx Enable: Yes: ");
	arg	= HPDI32_RX_ENABLE_YES;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_ENABLE, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_tx_sim_enable
*
*	Purpose:
*
*		Configure the HPDI32 to simulate presence of another device sending us
*		data. This is done by configuring the flow control signals as GPIO
*		inputs. Doing this disables the flow control functionality. We then
*		enable the transmitter, which enables the Tx Clock. With this
*		configuration the data is received at the rate of the Tx Clock.
*
*	Arguments:
*
*		fd		The handle of the device to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _tx_sim_enable(int fd)
{
	s32	arg;
	int	errs	= 0;
	int	ret;

	// Here, we simulate the presence of a connected transmitting device. We
	// do this by configuring the transmitter so the Tx Clock is active full
	// time and all of the other flow control signals are GPIO Inputs.

	printf("Tx Simulation: DISCONNECT ANY REMOTE DEVICE!\n");

	// We reduce the Tx clock so we don't get an Rx Overrun error.
	printf("  Tx Clock Divider: 2: ");
	arg	= 2;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_CLOCK_DIVIDER, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Set the Frame Valid signal to GPIO Input.
	printf("  Frame Valid: GPIO Input: ");
	arg	= HPDI32_CABLE_CMD_MODE_IN;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_CABLE_CMD_MODE_0, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Set the Line Valid signal to GPIO Input.
	printf("  Line Valid: GPIO Input: ");
	arg	= HPDI32_CABLE_CMD_MODE_IN;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_CABLE_CMD_MODE_1, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Set the Status Valid signal to GPIO Input.
	printf("  Status Valid: GPIO Input: ");
	arg	= HPDI32_CABLE_CMD_MODE_IN;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_CABLE_CMD_MODE_2, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Set the Rx Ready signal to GPIO Input.
	printf("  Rx Ready: GPIO Input: ");
	arg	= HPDI32_CABLE_CMD_MODE_IN;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_CABLE_CMD_MODE_3, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Set the Tx Ready signal to GPIO Input.
	printf("  Tx Ready: GPIO Input: ");
	arg	= HPDI32_CABLE_CMD_MODE_IN;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_CABLE_CMD_MODE_4, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Enable the Transmitter to start the Tx Clock.
	printf("  Tx Enable: Yes: ");
	arg	= HPDI32_TX_ENABLE_YES;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_ENABLE, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_rx_data
*
*	Purpose:
*
*		Receive the data.
*
*	Arguments:
*
*		fd		The handle of the device to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _rx_data(int fd)
{
	int			errs	= 0;
	const char*	factor	= "";
	size_t		get		= sizeof(buffer);
	int			i;
	int			ret;
	long long	total	= 0;

	printf("Receiving Data:\n");

	for (i = 0; i < 256; i++)
	{
		if ((i % 64) == 0)
			printf("\n  ");

		printf(".");
		fflush(stdout);
		ret	= hpdi32_read(fd, buffer, get);

		if (ret > 0)
			total	+= ret;

		if (ret < 0)
		{
			if (total == 0)
			{
				// We assume this even though the results are other than success.
				printf(" ABORTED <---  (No Rx Clock at the cable interface)\n");
			}
			else
			{
				printf("FAIL <---  (error = %d)\n", ret);
				errs++;
			}

			break;
		}

		if (ret != get)
		{
			printf(	" FAIL <--- (Requested %ld bytes but read only %ld bytes.)",
					(long) get,
					(long) ret);
			errs++;
			break;
		}
	}

	printf("\n\n");

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "K";
	}

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "M";
	}

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "G";
	}

	printf("Read %lld%s Bytes\n", total, factor);
	return(errs);
}



//*****************************************************************************
int perform_tests(int fd, int tx_sim)
{
	int		errs		= 0;

	memset(buffer, 0, sizeof(buffer));

	errs	+= _config(fd);

	if ((errs == 0) && (tx_sim))
		errs	+= _tx_sim_enable(fd);

	if (errs == 0)
		errs	+= _rx_data(fd);

	return(errs);
}


