// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/cable_command_2.c $
// $Rev: 53397 $
// $Date: 2023-07-12 11:59:33 -0500 (Wed, 12 Jul 2023) $

// HPDI32: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	D2			(0x1 << 2)



//*****************************************************************************
static int _write_data(int fd, int verbose, void* src, size_t bytes)
{
	int		errs;
	size_t	sent;

	memset(src, 0, bytes);
	errs	= hpdi32_write_util(fd, -1, verbose, src, bytes, &sent);

	if (sent != bytes)
	{
		if (verbose)
		{
			gsc_label("Write Failure");
			printf(	"FAIL <---  (Wrote %ld bytes instead of %ld.)\n",
					(long) sent,
					(long) bytes);
		}

		errs++;
	}

	return(errs);
}



//*****************************************************************************
static int _tx_status_valid_off_test(int fd, int verbose)
{
	u32				bsr		= 0;
	long			change	= 0;
	int				errs	= 0;
	int				level	= -1;
	os_time_ns_t	now;
	int				ret;
	os_time_ns_t	start;

	gsc_label("Tx Status Valid Off");

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize			(fd, -1, verbose);
	errs	+= hpdi32_tx_enable				(fd, -1, verbose, HPDI32_TX_ENABLE_NO, NULL);
	errs	+= hpdi32_tx_clock_divider		(fd, -1, verbose, 0xFFFF, NULL);
	errs	+= hpdi32_tx_status_val_cnt		(fd, -1, verbose, 1024, NULL);
	errs	+= hpdi32_cable_cmd_mode_0		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_cable_cmd_mode_1		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_cable_cmd_mode_2		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_FC, NULL);

	if (verbose)
		gsc_label("Monitoring");

	os_time_get_ns(&start);

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (change)
			{
				errs++;

				if (verbose)
				{
					printf(	"FAIL <---  (The level changed %ld time%s in one second.)\n",
							change,
							(change == 1) ? "" : "s");
				}
			}
			else
			{
				if (verbose)
				{
					printf(	"PASS  (The level stayed constant for one second. BSR 0x%lX.)\n",
							(long) bsr);
				}
			}

			break;
		}

		ret	= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (ret)
		{
			if (verbose)
				printf("FAIL <---  (Reading BSR.)\n");

			errs++;
			break;
		}

		if (level < 0)
		{
			if (bsr & D2)
				level	= 1;
			else
				level	= 0;
		}
		else if (level)
		{
			if ((bsr & D2) == 0)
			{
				change++;
				level	= 0;
			}
		}
		else
		{
			if (bsr & D2)
			{
				change++;
				level	= 1;
			}
		}
	}

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
static int _tx_status_valid_idle_test(int fd, int verbose)
{
	u32				bsr		= 0;
	long			change	= 0;
	int				errs	= 0;
	int				level	= -1;
	os_time_ns_t	now;
	int				ret;
	os_time_ns_t	start;

	gsc_label("Tx Status Valid Idle");

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize			(fd, -1, verbose);
	errs	+= hpdi32_tx_enable				(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
	errs	+= hpdi32_tx_clock_divider		(fd, -1, verbose, 0xFFFF, NULL);
	errs	+= hpdi32_tx_status_val_cnt		(fd, -1, verbose, 1024, NULL);
	errs	+= hpdi32_cable_cmd_mode_0		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_cable_cmd_mode_1		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_cable_cmd_mode_2		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_FC, NULL);

	if (verbose)
		gsc_label("Monitoring");

	os_time_get_ns(&start);

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (change)
			{
				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (The level changed %ld time%s in one second.)\n",
							change,
							(change == 1) ? "" : "s");
				}
			}
			else if (level)
			{
				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (The level was high for one second. BSR 0x%lX.)\n",
							(long) bsr);
				}
			}
			else
			{
				if (verbose)
					printf("PASS  (The level stayed low for one second.)\n");
			}

			break;
		}

		ret	= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (ret)
		{
			if (verbose)
				printf("FAIL <---  (Reading BSR.)\n");

			errs++;
			break;
		}

		if (level < 0)
		{
			if (bsr & D2)
				level	= 1;
			else
				level	= 0;
		}
		else if (level)
		{
			if ((bsr & D2) == 0)
			{
				change++;
				level	= 0;
			}
		}
		else
		{
			if (bsr & D2)
			{
				change++;
				level	= 1;
			}
		}
	}

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
static int _tx_status_valid_active_test(int fd, int verbose)
{
	u32				bsr		= 0;
	u32				buf[1024];
	long			change	= 0;
	int				errs	= 0;
	u32				frr;
	int				level	= -1;
	s32				model;
	os_time_ns_t	now;
	int				ret;
	os_time_ns_t	start;

	gsc_label("Tx Status Valid Active");

	for (;;)		// A convenience loop.
	{
		errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_MODEL, &model);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (Model query failed.)\n");
			else
				printf("FAIL <---\n");

			break;
		}

		if ((model == HPDI32_QUERY_MODEL_HPDI32) ||
			(model == HPDI32_QUERY_MODEL_HPDI32AL))
		{
			printf("SKIPPED  (Not supported on this device.)\n");
			break;
		}

		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_FRR, &frr);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (FRR read failed.)\n");
			else
				printf("FAIL <---\n");

			break;
		}

		if ((model == HPDI32_QUERY_MODEL_HPDI32ALT) && ((frr & 0xFF) < 0x05))
		{
			printf("SKIPPED  (Not supported on this device.)\n");
			break;
		}

		if (verbose)
			printf("\n");

		gsc_label_level_inc();

		errs	+= hpdi32_initialize			(fd, -1, verbose);
		errs	+= hpdi32_tx_enable				(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
		errs	+= hpdi32_tx_clock_divider		(fd, -1, verbose, 0x1F, NULL);
		errs	+= hpdi32_tx_status_val_cnt		(fd, -1, verbose, 1, NULL);
		errs	+= hpdi32_cable_cmd_mode_0		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
		errs	+= hpdi32_cable_cmd_mode_1		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
		errs	+= hpdi32_cable_cmd_mode_2		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_FC, NULL);
		errs	+= hpdi32_tx_clock_divider		(fd, -1, verbose, 0xFFF, NULL);
		errs	+= _write_data					(fd, verbose, buf, sizeof(buf));
		errs	+= hpdi32_tx_flow_control		(fd, -1, verbose, HPDI32_TX_FLOW_CONTROL_START, NULL);

		if (verbose)
			gsc_label("Monitoring");

		os_time_get_ns(&start);

		for (;;)	// monitor loop
		{
			os_time_get_ns(&now);

			if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
			{
				if (change == 0)
				{
					errs	= 1;

					if (verbose)
					{
						printf(	"FAIL <---  (The level stayed constant for one second. BSR 0x%lX.)\n",
								(long) bsr);
					}
				}
				else
				{
					if (verbose)
						printf("PASS  (For one second the level did not stay constant.)\n");
				}

				break;
			}

			ret	= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

			if (ret)
			{
				if (verbose)
					printf("FAIL <---  (Reading BSR.)\n");

				errs++;
				break;
			}

			if (level < 0)
			{
				if (bsr & D2)
					level	= 1;
				else
					level	= 0;
			}
			else if (level)
			{
				if ((bsr & D2) == 0)
				{
					change++;
					level	= 0;
				}
			}
			else
			{
				if (bsr & D2)
				{
					change++;
					level	= 1;
				}
			}
		}

		gsc_label_level_dec();

		if (verbose == 0)
			printf("%s\n", errs ? "FAIL <---" : "PASS");

		break;
	}

	return(errs);
}



//*****************************************************************************
static int _bsr_d0_status(int fd, int verbose, s32 cable_mode)
{
	u32	bsr;
	int	errs	= 0;

	if (cable_mode != HPDI32_CABLE_CMD_MODE_IN)
	{
		if (verbose)
			gsc_label("BSR.D2 Status Bit");

		errs	= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (Reading BSR.)\n");
		}
		else if (cable_mode == HPDI32_CABLE_CMD_MODE_OUT_LO)
		{
			if (bsr & D2)
			{
				if (verbose)
				{
					printf(	"FAIL <---  (The level is 1, but expect 0. BSR 0x%lX.)\n",
							(long) bsr);
				}

				errs++;
			}
			else
			{
				if (verbose)
					printf("PASS  (BSR.D2 is low.)\n");
			}
		}
		else if (cable_mode == HPDI32_CABLE_CMD_MODE_OUT_HI)
		{
			if ((bsr & D2) == 0)
			{
				if (verbose)
				{
					printf(	"FAIL <---  (The level is 0, but expect 1. BSR 0x%lX.)\n",
							(long) bsr);
				}

				errs++;
			}
			else
			{
				if (verbose)
					printf("PASS  (BSR.D2 is high.)\n");
			}
		}
	}

	return(errs);
}



//*****************************************************************************
static int _rx_fifo_status(int fd, int verbose)
{
	int	errs;
	u32	rfwr;

	if (verbose)
		gsc_label("Rx FIFO State");

	errs	= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_RFWR, &rfwr);

	if (errs)
	{
		if (verbose)
			printf("FAIL <---  (Reading RFWR.)\n");
	}
	else if (rfwr)
	{
		errs++;

		if (verbose)
		{
			printf(	"FAIL <---  (Rx FIFO is not empty. RFWR 0x%lX.)\n",
					(long) rfwr);
		}
	}
	else
	{
		if (verbose)
			printf(	"PASS  (Rx FIFO is empty.)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _tx_status_valid_gpio_test(
	int			fd,
	int			verbose,
	const char*	title,
	s32			cable_mode)
{
	u32				buf[1024];
	int				errs	= 0;
	os_time_ns_t	now;
	int				ret;
	os_time_ns_t	start;
	u32				tfwr	= 0;

	gsc_label(title);

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize			(fd, -1, verbose);
	errs	+= hpdi32_tx_enable				(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
	errs	+= hpdi32_tx_status_val_cnt		(fd, -1, verbose, 1, NULL);
	errs	+= hpdi32_cable_cmd_mode_0		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_cable_cmd_mode_1		(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_cable_cmd_mode_2		(fd, -1, verbose, cable_mode, NULL);
	errs	+= hpdi32_tx_clock_divider		(fd, -1, verbose, 0x1F, NULL);
	errs	+= _write_data					(fd, verbose, buf, sizeof(buf));
	errs	+= hpdi32_tx_flow_control		(fd, -1, verbose, HPDI32_TX_FLOW_CONTROL_START, NULL);

	if (verbose)
		gsc_label("Monitoring");

	os_time_get_ns(&start);

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (verbose)
			{
				printf(	"FAIL <---  (Tx FIFO did not empty within one second. TFWR %ld.)\n",
						(long) tfwr);
			}

			errs	= 1;
			break;
		}

		ret	= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_TFWR, &tfwr);

		if (ret)
		{
			if (verbose)
				printf("FAIL <---  (Reading TFWR.)\n");

			errs++;
			break;
		}

		if (tfwr == 0)
		{
			if (verbose)
				printf("PASS  (Tx FIFO emptied within one second.)\n");

			break;
		}
	}

	errs	+= _bsr_d0_status(fd, verbose, cable_mode);
	errs	+= _rx_fifo_status(fd, verbose);

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
static int _tx_status_valid_gpio_input_test(int fd, int verbose)
{
	int	errs;

	errs	= _tx_status_valid_gpio_test(
					fd,
					verbose,
					"Tx Status Valid GPIO Input",
					HPDI32_CABLE_CMD_MODE_IN);
	return(errs);
}



//*****************************************************************************
static int _tx_status_valid_gpio_output_0_test(int fd, int verbose)
{
	int	errs;

	errs	= _tx_status_valid_gpio_test(
					fd,
					verbose,
					"Tx Status Valid GPIO Output 0",
					HPDI32_CABLE_CMD_MODE_OUT_LO);
	return(errs);
}



//*****************************************************************************
static int _tx_status_valid_gpio_output_1_test(int fd, int verbose)
{
	int	errs;

	errs	= _tx_status_valid_gpio_test(
					fd,
					verbose,
					"Tx Status Valid GPIO Output 1",
					HPDI32_CABLE_CMD_MODE_OUT_HI);
	return(errs);
}



//*****************************************************************************
static int _gpio_output_test(int fd, int verbose)
{
	int	errs	= 0;
	u32	frr;
	s32	high;
	int	i;
	s32	low;
	s32	model;

	gsc_label("GPIO Output");

	for (;;)		// A convenience loop.
	{
		errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_MODEL, &model);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (Model query failed.)\n");
			else
				printf("FAIL <---\n");

			break;
		}

		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_FRR, &frr);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (FRR read failed.)\n");
			else
				printf("FAIL <---\n");

			break;
		}

		if ((model == HPDI32_QUERY_MODEL_HPDI32ALT) && ((frr & 0xFF) < 0x05))
		{
			printf("SKIPPED  (Not supported on this device.)\n");
			break;
		}

		for (i = 0; i < 10; i++)
		{
			errs	+= hpdi32_cable_cmd_mode_2(fd, -1, 0, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
			errs	+= hpdi32_cable_cmd_state_2(fd, -1, 0, &low);
			errs	+= hpdi32_cable_cmd_mode_2(fd, -1, 0, HPDI32_CABLE_CMD_MODE_OUT_HI, NULL);
			errs	+= hpdi32_cable_cmd_state_2(fd, -1, 0, &high);

			if (errs)
			{
				printf("FAIL <---  (Cable Command 2 Mode/State request.)\n");
				errs	= 1;
				break;
			}

			if (low != HPDI32_CABLE_CMD_STATE_0)
			{
				printf("FAIL <---  (Cable Command 2 state not reported as low.)\n");
				errs++;
				break;
			}

			if (high != HPDI32_CABLE_CMD_STATE_1)
			{
				printf("FAIL <---  (Cable Command 2 state not reported as high.)\n");
				errs++;
				break;
			}
		}

		if (errs == 0)
			printf("PASS\n");

		break;
	}

	return(errs);
}



//*****************************************************************************
int cable_command_2_test(int fd, int verbose)
{
	int	errs		= 0;
	s32	gpio_0_5	= 0;
	s32	gpio_6		= 0;

	gsc_label("Cable Command 2");

	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_GPIO_0_5, &gpio_0_5);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_GPIO_6, &gpio_6);

	if (errs)
	{
		printf("FAIL <---  (GPIO query failures.)\n");
	}
	else if (gpio_6 == 0)
	{
		printf("SKIPPED  (GPIO 6 not supported.)\n");
	}
	else if (gpio_0_5 == 0)
	{
		printf("SKIPPED  (GPIO 0-5 not supported.)\n");
	}
	else
	{
		printf("\n");

		gsc_label_level_inc();

		errs	+= _tx_status_valid_off_test(fd, verbose);
		errs	+= _tx_status_valid_idle_test(fd, verbose);
		errs	+= _tx_status_valid_active_test(fd, verbose);
		errs	+= _tx_status_valid_gpio_input_test(fd, verbose);
		errs	+= _tx_status_valid_gpio_output_0_test(fd, verbose);
		errs	+= _tx_status_valid_gpio_output_1_test(fd, verbose);
		errs	+= _gpio_output_test(fd, verbose);

		gsc_label_level_dec();
	}

	return(errs);
}


