// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/cable_command_3.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	D15			(0x1 << 15)



//*****************************************************************************
static int _rx_ready_remote_throttle_bypass_test(int fd, int verbose)
{
	u32				bsr		= 0;
	int				errs	= 0;
	os_time_ns_t	now;
	os_time_ns_t	start;

	gsc_label("Rx Ready Remote Throttle Bypass");

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize		(fd, -1, verbose);
	errs	+= hpdi32_cable_cmd_mode_0	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_HI, NULL);
	errs	+= hpdi32_cable_cmd_mode_1	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_cable_cmd_mode_2	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_cable_cmd_mode_3	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_FC, NULL);
	errs	+= hpdi32_tx_auto_start		(fd, -1, verbose, HPDI32_AUTO_START_NO, NULL);
	errs	+= hpdi32_tx_auto_stop		(fd, -1, verbose, HPDI32_AUTO_STOP_NO, NULL);
	errs	+= hpdi32_tx_remote_throttle(fd, -1, verbose, HPDI32_TX_REMOTE_THROTTLE_YES, NULL);
	errs	+= hpdi32_tx_enable			(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
	errs	+= hpdi32_rx_enable			(fd, -1, verbose, HPDI32_RX_ENABLE_YES, NULL);

	if (verbose)
		gsc_label("Monitoring");

	os_time_get_ns(&start);

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (verbose)
				printf(	"FAIL <---  (The Rx FIFO did not fill in one second.)\n");

			errs++;
			break;
		}

		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (Reading BSR.)\n");

			break;
		}

		if ((bsr & D15) == 0)
		{
			if (verbose)
				printf(	"PASS  (The Rx FIFO filled in one second.)\n");

			break;
		}
	}

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
static int _rx_ready_remote_throttle_off_test(int fd, int verbose)
{
	u32				bsr			= 0;
	u32				buf[1024]	= { 0 };
	int				errs		= 0;
	os_time_ns_t	now;
	int				sent;
	os_time_ns_t	start;

	gsc_label("Rx Ready Remote Throttle Off");

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize		(fd, -1, verbose);
	errs	+= hpdi32_cable_cmd_mode_3	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_FC, NULL);
	errs	+= hpdi32_tx_auto_start		(fd, -1, verbose, HPDI32_AUTO_START_YES, NULL);
	errs	+= hpdi32_tx_auto_stop		(fd, -1, verbose, HPDI32_AUTO_STOP_NO, NULL);
	errs	+= hpdi32_tx_remote_throttle(fd, -1, verbose, HPDI32_TX_REMOTE_THROTTLE_NO, NULL);
	errs	+= hpdi32_tx_io_mode		(fd, -1, verbose, GSC_IO_MODE_PIO, NULL);
	errs	+= hpdi32_tx_io_timeout		(fd, -1, verbose, 0, NULL);
	errs	+= hpdi32_tx_io_overrun		(fd, -1, verbose, HPDI32_IO_ERROR_IGNORE, NULL);
	errs	+= hpdi32_tx_enable			(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
	errs	+= hpdi32_rx_enable			(fd, -1, verbose, HPDI32_RX_ENABLE_YES, NULL);

	if (verbose)
		gsc_label("Monitoring");

	os_time_get_ns(&start);

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (verbose == 0)
				printf(	"FAIL <---  (The Rx FIFO did not fill in one second.)\n");

			errs++;
			break;
		}

		sent	= hpdi32_write(fd, buf, sizeof(buf));

		if (sent < 0)
		{
			if (verbose == 0)
				printf(	"FAIL <---  (Write request returned %d.)\n", sent);

			errs++;
			break;
		}

		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (Reading BSR.)\n");

			break;
		}

		if ((bsr & D15) == 0)
		{
			if (verbose)
				printf("PASS  (The Rx FIFO filled within one second.)\n");

			break;
		}
	}

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
static int _rx_ready_remote_throttle_on_test(int fd, int verbose)
{
	u32				bsr			= 0;
	u32				buf[1024]	= { 0 };
	int				errs		= 0;
	os_time_ns_t	now;
	int				sent;
	os_time_ns_t	start;

	gsc_label("Rx Ready Remote Throttle On");

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize		(fd, -1, verbose);
	errs	+= hpdi32_cable_cmd_mode_3	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_FC, NULL);
	errs	+= hpdi32_tx_auto_start		(fd, -1, verbose, HPDI32_AUTO_START_YES, NULL);
	errs	+= hpdi32_tx_auto_stop		(fd, -1, verbose, HPDI32_AUTO_STOP_NO, NULL);
	errs	+= hpdi32_tx_remote_throttle(fd, -1, verbose, HPDI32_TX_REMOTE_THROTTLE_YES, NULL);
	errs	+= hpdi32_tx_io_mode		(fd, -1, verbose, GSC_IO_MODE_PIO, NULL);
	errs	+= hpdi32_tx_io_timeout		(fd, -1, verbose, 0, NULL);
	errs	+= hpdi32_tx_io_overrun		(fd, -1, verbose, HPDI32_IO_ERROR_IGNORE, NULL);
	errs	+= hpdi32_tx_enable			(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
	errs	+= hpdi32_rx_enable			(fd, -1, verbose, HPDI32_RX_ENABLE_YES, NULL);

	if (verbose)
		gsc_label("Monitoring");

	os_time_get_ns(&start);

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (verbose)
			{
				printf(	"FAIL <---  (The Rx FIFO filled within one second. BSR 0x%lX.)\n",
						(long) bsr);
			}

			errs++;
			break;
		}

		sent	= hpdi32_write(fd, buf, sizeof(buf));

		if (sent < 0)
		{
			if (verbose)
				printf(	"FAIL <---  (Write request returned %d.)\n", sent);

			errs++;
			break;
		}

		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (Reading BSR.)\n");

			break;
		}

		if ((bsr & D15) == 0)
		{
			if (verbose)
				printf("PASS  (The Rx FIFO did not fill within one second.)\n");

			break;
		}
	}

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
static int _gpio_out_low_remote_throttle_test(int fd, int verbose)
{
	u32				bsr			= 0;
	u32				buf[1024]	= { 0 };
	int				errs		= 0;
	os_time_ns_t	now;
	int				sent;
	os_time_ns_t	start;

	gsc_label("GPIO Out Low Remote Throttle");

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize		(fd, -1, verbose);
	errs	+= hpdi32_cable_cmd_mode_3	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_tx_auto_start		(fd, -1, verbose, HPDI32_AUTO_START_YES, NULL);
	errs	+= hpdi32_tx_auto_stop		(fd, -1, verbose, HPDI32_AUTO_STOP_NO, NULL);
	errs	+= hpdi32_tx_remote_throttle(fd, -1, verbose, HPDI32_TX_REMOTE_THROTTLE_YES, NULL);
	errs	+= hpdi32_tx_io_mode		(fd, -1, verbose, GSC_IO_MODE_PIO, NULL);
	errs	+= hpdi32_tx_io_timeout		(fd, -1, verbose, 0, NULL);
	errs	+= hpdi32_tx_io_overrun		(fd, -1, verbose, HPDI32_IO_ERROR_IGNORE, NULL);
	errs	+= hpdi32_tx_enable			(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
	errs	+= hpdi32_rx_enable			(fd, -1, verbose, HPDI32_RX_ENABLE_YES, NULL);

	if (verbose)
		gsc_label("Monitoring");

	os_time_get_ns(&start);

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (verbose)
			{
				printf(	"FAIL <---  (The Rx FIFO did not fill within one second. BSR 0x%lX.)\n",
						(long) bsr);
			}

			errs++;
			break;
		}

		sent	= hpdi32_write(fd, buf, sizeof(buf));

		if (sent < 0)
		{
			if (verbose)
				printf(	"FAIL <---  (Write request returned %d.)\n", sent);

			errs++;
			break;
		}

		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (Reading BSR.)\n");

			break;
		}

		if ((bsr & D15) == 0)
		{
			if (verbose)
				printf("PASS (The Rx FIFO filled within one second.)\n");

			break;
		}
	}

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
static int _gpio_out_high_remote_throttle_test(int fd, int verbose)
{
	u32				bsr			= 0;
	u32				buf[1024]	= { 0 };
	int				errs		= 0;
	os_time_ns_t	now;
	int				sent;
	os_time_ns_t	start;

	gsc_label("GPIO Out High Remote Throttle");

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize		(fd, -1, verbose);
	errs	+= hpdi32_cable_cmd_mode_3	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_HI, NULL);
	errs	+= hpdi32_tx_auto_start		(fd, -1, verbose, HPDI32_AUTO_START_YES, NULL);
	errs	+= hpdi32_tx_auto_stop		(fd, -1, verbose, HPDI32_AUTO_STOP_NO, NULL);
	errs	+= hpdi32_tx_remote_throttle(fd, -1, verbose, HPDI32_TX_REMOTE_THROTTLE_YES, NULL);
	errs	+= hpdi32_tx_io_mode		(fd, -1, verbose, GSC_IO_MODE_PIO, NULL);
	errs	+= hpdi32_tx_io_timeout		(fd, -1, verbose, 0, NULL);
	errs	+= hpdi32_tx_io_overrun		(fd, -1, verbose, HPDI32_IO_ERROR_IGNORE, NULL);
	errs	+= hpdi32_tx_enable			(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
	errs	+= hpdi32_rx_enable			(fd, -1, verbose, HPDI32_RX_ENABLE_YES, NULL);

	if (verbose)
		gsc_label("Monitoring");

	os_time_get_ns(&start);

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (verbose)
			{
				printf(	"FAIL <---  (The Rx FIFO did not fill within one second. BSR 0x%lX.)\n",
						(long) bsr);
			}

			errs++;
			break;
		}

		sent	= hpdi32_write(fd, buf, sizeof(buf));

		if (sent < 0)
		{
			if (verbose)
				printf(	"FAIL <---  (Write request returned %d.)\n", sent);

			errs++;
			break;
		}

		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (Reading BSR.)\n");

			break;
		}

		if ((bsr & D15) == 0)
		{
			if (verbose)
				printf("PASS  (The Rx FIFO filled within one second.)\n");

			break;
		}
	}

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
static int _gpio_output_test(int fd, int verbose)
{
	int	errs	= 0;
	s32	high;
	int	i;
	s32	low;

	gsc_label("GPIO Output");

	for (i = 0; i < 10; i++)
	{
		errs	+= hpdi32_cable_cmd_mode_3(fd, -1, 0, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
		errs	+= hpdi32_cable_cmd_state_3(fd, -1, 0, &low);
		errs	+= hpdi32_cable_cmd_mode_3(fd, -1, 0, HPDI32_CABLE_CMD_MODE_OUT_HI, NULL);
		errs	+= hpdi32_cable_cmd_state_3(fd, -1, 0, &high);

		if (errs)
		{
			printf("FAIL <---  (Cable Command 3 Mode/State request.)\n");
			errs	= 1;
			break;
		}

		if (low != HPDI32_CABLE_CMD_STATE_0)
		{
			printf("FAIL <---  (Cable Command 3 state not reported as low.)\n");
			errs++;
			break;
		}

		if (high != HPDI32_CABLE_CMD_STATE_1)
		{
			printf("FAIL <---  (Cable Command 3 state not reported as high.)\n");
			errs++;
			break;
		}
	}

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
int cable_command_3_test(int fd, int verbose)
{
	int	errs		= 0;
	s32	gpio_0_5	= 0;
	s32	gpio_6		= 0;

	gsc_label("Cable Command 3");

	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_GPIO_0_5, &gpio_0_5);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_GPIO_6, &gpio_6);

	if (errs)
	{
		printf("FAIL <---  (GPIO query failures.)\n");
	}
	else if (gpio_6 == 0)
	{
		printf("SKIPPED  (GPIO 6 not supported.)\n");
	}
	else if (gpio_0_5 == 0)
	{
		printf("SKIPPED  (GPIO 0-5 not supported.)\n");
	}
	else
	{
		printf("\n");

		gsc_label_level_inc();

		errs	+= _rx_ready_remote_throttle_bypass_test(fd, verbose);
		errs	+= _rx_ready_remote_throttle_off_test(fd, verbose);
		errs	+= _rx_ready_remote_throttle_on_test(fd, verbose);
		errs	+= _gpio_out_low_remote_throttle_test(fd, verbose);
		errs	+= _gpio_out_high_remote_throttle_test(fd, verbose);
		errs	+= _gpio_output_test(fd, verbose);

		gsc_label_level_dec();
	}

	return(errs);
}


