// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/cable_command_6.c $
// $Rev: 49212 $
// $Date: 2021-03-15 16:37:14 -0500 (Mon, 15 Mar 2021) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _rx_enabled_no_test(int fd, int verbose)
{
	int	errs	= 0;
	s32	state;
	int	i;

	gsc_label("Rx Enabled - No");

	for (i = 0; i < 10; i++)
	{
		errs	+= hpdi32_initialize		(fd, -1, 0);
		errs	+= hpdi32_rx_enable			(fd, -1, 0, HPDI32_RX_ENABLE_NO, NULL);
		errs	+= hpdi32_cable_cmd_mode_6	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_FC, NULL);
		errs	+= hpdi32_cable_cmd_state_6	(fd, -1, 0, &state);

		if (errs)
		{
			printf("FAIL <---  (Device configuration.)\n");
			break;
		}

		if (state != HPDI32_CABLE_CMD_STATE_0)
		{
			printf("FAIL <---  (State not reported as low.)\n");
			errs++;
			break;
		}
	}

	if (errs == 0)
		printf("PASS  (State is low as receiver is disabled.)\n");

	return(errs);
}



//*****************************************************************************
static int _rx_enabled_yes_test(int fd, int verbose)
{
	int	errs	= 0;
	s32	state;
	int	i;

	gsc_label("Rx Enabled - Yes");

	for (i = 0; i < 10; i++)
	{
		errs	+= hpdi32_initialize		(fd, -1, 0);
		errs	+= hpdi32_rx_enable			(fd, -1, 0, HPDI32_RX_ENABLE_YES, NULL);
		errs	+= hpdi32_cable_cmd_mode_6	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_FC, NULL);
		errs	+= hpdi32_cable_cmd_state_6	(fd, -1, 0, &state);

		if (errs)
		{
			printf("FAIL <---  (Device configuration.)\n");
			break;
		}

		if (state != HPDI32_CABLE_CMD_STATE_1)
		{
			printf("FAIL <---  (State not reported as high.)\n");
			errs++;
			break;
		}
	}

	if (errs == 0)
		printf("PASS  (State is high as receiver is enabled.)\n");

	return(errs);
}



//*****************************************************************************
static int _rx_enabled_gpio_out_low_test(int fd, int verbose)
{
	int	errs	= 0;
	s32	state;
	int	i;

	gsc_label("Rx Enabled GPIO Output Low");

	for (i = 0; i < 10; i++)
	{
		errs	+= hpdi32_initialize		(fd, -1, 0);
		errs	+= hpdi32_rx_enable			(fd, -1, 0, HPDI32_RX_ENABLE_YES, NULL);
		errs	+= hpdi32_cable_cmd_mode_6	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
		errs	+= hpdi32_cable_cmd_state_6	(fd, -1, 0, &state);

		if (errs)
		{
			printf("FAIL <---  (Device configuration.)\n");
			break;
		}

		if (state != HPDI32_CABLE_CMD_STATE_0)
		{
			printf("FAIL <---  (State not reported as low.)\n");
			errs++;
			break;
		}
	}

	if (errs == 0)
		printf("PASS  (State is low even though receiver is enabled.)\n");

	return(errs);
}



//*****************************************************************************
static int _rx_enabled_gpio_out_high_test(int fd, int verbose)
{
	int	errs	= 0;
	s32	state;
	int	i;

	gsc_label("Rx Enabled GPIO Output High");

	for (i = 0; i < 10; i++)
	{
		errs	+= hpdi32_initialize		(fd, -1, 0);
		errs	+= hpdi32_rx_enable			(fd, -1, 0, HPDI32_RX_ENABLE_NO, NULL);
		errs	+= hpdi32_cable_cmd_mode_6	(fd, -1, 0, HPDI32_CABLE_CMD_MODE_OUT_HI, NULL);
		errs	+= hpdi32_cable_cmd_state_6	(fd, -1, 0, &state);

		if (errs)
		{
			printf("FAIL <---  (Device configuration.)\n");
			break;
		}

		if (state != HPDI32_CABLE_CMD_STATE_1)
		{
			printf("FAIL <---  (State not reported as high.)\n");
			errs++;
			break;
		}
	}

	if (errs == 0)
		printf("PASS  (State is high even though receiver is disabled.)\n");

	return(errs);
}



//*****************************************************************************
static int _gpio_output_test(int fd, int verbose)
{
	int	errs	= 0;
	s32	high;
	int	i;
	s32	low;

	gsc_label("GPIO Output");

	for (i = 0; i < 10; i++)
	{
		errs	+= hpdi32_cable_cmd_mode_6(fd, -1, 0, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
		errs	+= hpdi32_cable_cmd_state_6(fd, -1, 0, &low);
		errs	+= hpdi32_cable_cmd_mode_6(fd, -1, 0, HPDI32_CABLE_CMD_MODE_OUT_HI, NULL);
		errs	+= hpdi32_cable_cmd_state_6(fd, -1, 0, &high);

		if (errs)
		{
			printf("FAIL <---  (Cable Command 6 Mode/State request.)\n");
			errs	= 1;
			break;
		}

		if (low != HPDI32_CABLE_CMD_STATE_0)
		{
			printf("FAIL <---  (Cable Command 6 state not reported as low.)\n");
			errs++;
			break;
		}

		if (high != HPDI32_CABLE_CMD_STATE_1)
		{
			printf("FAIL <---  (Cable Command 6 state not reported as high.)\n");
			errs++;
			break;
		}
	}

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
int cable_command_6_test(int fd, int verbose)
{
	int	errs		= 0;
	s32	gpio_0_5	= 0;
	s32	gpio_6		= 0;

	gsc_label("Cable Command 6");

	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_GPIO_0_5, &gpio_0_5);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_GPIO_6, &gpio_6);

	if (errs)
	{
		printf("FAIL <---  (GPIO query failures.)\n");
	}
	else if (gpio_6 == 0)
	{
		printf("SKIPPED  (GPIO 6 not supported.)\n");
	}
	else if (gpio_0_5 == 0)
	{
		printf("SKIPPED  (GPIO 0-5 not supported.)\n");
	}
	else
	{
		printf("\n");

		gsc_label_level_inc();

		errs	+= _rx_enabled_no_test(fd, verbose);
		errs	+= _rx_enabled_yes_test(fd, verbose);
		errs	+= _rx_enabled_gpio_out_low_test(fd, verbose);
		errs	+= _rx_enabled_gpio_out_high_test(fd, verbose);
		errs	+= _gpio_output_test(fd, verbose);

		gsc_label_level_dec();
	}

	return(errs);
}


