// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/clock.c $
// $Rev: 53397 $
// $Date: 2023-07-12 11:59:33 -0500 (Wed, 12 Jul 2023) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _write_data(int fd, int verbose, void* src, size_t bytes)
{
	int		errs;
	size_t	sent;

	memset(src, 0, bytes);
	errs	= hpdi32_write_util(fd, -1, verbose, src, bytes, &sent);

	if (errs)
	{
	}
	else if (sent != bytes)
	{
		if (verbose)
		{
			gsc_label("Write Failure");
			printf(	"FAIL <---  (Wrote %ld bytes instead of %ld.)\n",
					(long) sent,
					(long) bytes);
		}

		errs++;
	}

	return(errs);
}



//*****************************************************************************
static int _tx_clock_off_test(int fd, int verbose)
{
	int				errs	= 0;
	os_time_ns_t	now;
	int				ret;
	u32				rfwr	= 0;
	os_time_ns_t	start;

	gsc_label("Tx Clock Off");

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize		(fd, -1, verbose);
	errs	+= hpdi32_cable_cmd_mode_0	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_cable_cmd_mode_1	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_cable_cmd_mode_2	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
	errs	+= hpdi32_tx_enable			(fd, -1, verbose, HPDI32_TX_ENABLE_NO, NULL);
	errs	+= hpdi32_rx_enable			(fd, -1, verbose, HPDI32_RX_ENABLE_YES, NULL);

	if (verbose)
		gsc_label("Monitoring");

	os_time_get_ns(&start);

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (verbose)
				printf("PASS  (No Rx Data received within one second.)\n");

			break;
		}

		ret	= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_RFWR, &rfwr);

		if (ret)
		{
			if (verbose)
				printf("FAIL <---  (Reading RFWR.)\n");

			errs++;
			break;
		}

		if (rfwr)
		{
			if (verbose)
			{
				printf(	"FAIL <---  (Rx Data received within one second. RFWR %ld.)\n",
						(long) rfwr);
			}

			errs++;
			break;
		}
	}

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
static int _tx_clock_on_test(int fd, int verbose)
{
	int				errs	= 0;
	u32				frr;
	s32				model;
	os_time_ns_t	now;
	int				ret;
	u32				rfwr;
	os_time_ns_t	start;

	gsc_label("Tx Clock On");

	for (;;)		// A convenience loop.
	{
		errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_MODEL, &model);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (Model query failed.)\n");
			else
				printf("FAIL <---\n");

			break;
		}

		if ((model == HPDI32_QUERY_MODEL_HPDI32)	||
			(model == HPDI32_QUERY_MODEL_HPDI32A)	||
			(model == HPDI32_QUERY_MODEL_HPDI32AL))
		{
			printf("SKIPPED  (Not supported on this device.)\n");
			break;
		}

		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_FRR, &frr);

		if (errs)
		{
			if (verbose)
				printf("FAIL <---  (FRR read failed.)\n");
			else
				printf("FAIL <---\n");

			break;
		}

		if ((model == HPDI32_QUERY_MODEL_HPDI32ALT) && ((frr & 0xFF) < 0x05))
		{
			printf("SKIPPED  (Not supported on this device.)\n");
			break;
		}

		if (verbose)
			printf("\n");

		gsc_label_level_inc();

		errs	+= hpdi32_initialize		(fd, -1, verbose);
		errs	+= hpdi32_cable_cmd_mode_0	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
		errs	+= hpdi32_cable_cmd_mode_1	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
		errs	+= hpdi32_cable_cmd_mode_2	(fd, -1, verbose, HPDI32_CABLE_CMD_MODE_OUT_LO, NULL);
		errs	+= hpdi32_tx_enable			(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
		errs	+= hpdi32_rx_enable			(fd, -1, verbose, HPDI32_RX_ENABLE_YES, NULL);

		if (verbose)
			gsc_label("Monitoring");

		os_time_get_ns(&start);

		for (;;)	// monitor loop
		{
			os_time_get_ns(&now);

			if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
			{
				if (verbose)
					printf("FAIL <---  (No Rx Data received within one second.)\n");

				errs++;
				break;
			}

			ret	= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_RFWR, &rfwr);

			if (ret)
			{
				if (verbose)
					printf("FAIL <---  (Reading RFWR.)\n");

				errs++;
				break;
			}

			if (rfwr)
			{
				if (verbose)
					printf("PASS  (Rx Data received within one second.)\n");

				break;
			}
		}

		gsc_label_level_dec();

		if (verbose == 0)
			printf("%s\n", errs ? "FAIL <---" : "PASS");

		break;
	}

	return(errs);
}



//*****************************************************************************
static int _tx_clock_transmit_test(int fd, int verbose)
{
	u32				buf[1024];
	int				err		= 0;
	int				errs	= 0;
	os_time_ns_t	now;
	os_time_ns_t	start;
	u32				tfwr	= 0;

	gsc_label("Tx Clock Transmit");

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize		(fd, -1, verbose);
	errs	+= hpdi32_tx_enable			(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
	errs	+= hpdi32_tx_clock_divider	(fd, -1, verbose, 0xFFF, NULL);
	errs	+= _write_data				(fd, verbose, buf, sizeof(buf));
	errs	+= hpdi32_tx_flow_control	(fd, -1, verbose, HPDI32_TX_FLOW_CONTROL_START, NULL);

	if (verbose)
		gsc_label("Monitoring");

	os_time_get_ns(&start);

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (verbose)
			{
				printf(	"FAIL <---  (Tx FIFO did not empty within one second. TFWR %ld.)\n",
						(long) tfwr);
			}

			errs++;
			break;
		}

		err	= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_TFWR, &tfwr);

		if (err)
		{
			if (verbose)
				printf("FAIL <---  (Reading TFWR.)\n");

			errs++;
			break;
		}

		if (tfwr == 0)
		{
			if (verbose)
				printf("PASS  (Tx FIFO emptied within one second.)\n");

			break;
		}
	}

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
static int _rx_clock_receive_test(int fd, int verbose)
{
	u32				buf[1024];
	int				errs	= 0;
	os_time_ns_t	now;
	int				ret;
	u32				rfwr	= 0;
	os_time_ns_t	start;

	gsc_label("Rx Clock Receive");

	if (verbose)
		printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize		(fd, -1, verbose);
	errs	+= hpdi32_tx_enable			(fd, -1, verbose, HPDI32_TX_ENABLE_YES, NULL);
	errs	+= hpdi32_tx_clock_divider	(fd, -1, verbose, 0xFFF, NULL);
	errs	+= _write_data				(fd, verbose, buf, sizeof(buf));
	errs	+= hpdi32_rx_enable			(fd, -1, verbose, HPDI32_RX_ENABLE_YES, NULL);
	errs	+= hpdi32_reg_read			(fd, -1, verbose, HPDI32_GSC_RFWR, &rfwr);

	if (rfwr)
	{
		if (verbose)
		{
			gsc_label("RFWR Read Failure");
			printf(	"FAIL <---  (Rx FIFO is not initially empty. RFWR %ld.)\n",
					(long) rfwr);
		}

		errs++;
	}

	errs	+= hpdi32_tx_flow_control(fd, -1, verbose, HPDI32_TX_FLOW_CONTROL_START, NULL);

	os_time_get_ns(&start);

	if (verbose)
		gsc_label("Monitoring");

	for (;;)	// monitor loop
	{
		os_time_get_ns(&now);

		if ((now.tv_sec > start.tv_sec) && (now.tv_nsec > start.tv_nsec))
		{
			if (verbose)
				printf("FAIL <---  (Rx FIFO did not receive data within one second.)\n");

			errs++;
			break;
		}

		ret	= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_RFWR, &rfwr);

		if (ret)
		{
			if (verbose)
				printf("FAIL <---  (Reading RFWR.)\n");

			errs++;
			break;
		}

		if (rfwr)
		{
			if (verbose)
				printf("PASS  (Rx FIFO received data within one second.)\n");

			break;
		}
	}

	gsc_label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



//*****************************************************************************
int clock_test(int fd, int verbose)
{
	int	errs		= 0;
	s32	gpio_0_5	= 0;
	s32	gpio_6		= 0;

	gsc_label("Clock Tests");

	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_GPIO_0_5, &gpio_0_5);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_GPIO_6, &gpio_6);

	if (errs)
	{
		printf("FAIL <---  (GPIO query failures.)\n");
	}
	else if (gpio_6 == 0)
	{
		printf("SKIPPED  (GPIO 6 not supported.)\n");
	}
	else if (gpio_0_5 == 0)
	{
		printf("SKIPPED  (GPIO 0-5 not supported.)\n");
	}
	else
	{
		printf("\n");

		gsc_label_level_inc();

		errs	+= _tx_clock_off_test(fd, verbose);
		errs	+= _tx_clock_on_test(fd, verbose);
		errs	+= _tx_clock_transmit_test(fd, verbose);
		errs	+= _rx_clock_receive_test(fd, verbose);

		gsc_label_level_dec();
	}

	return(errs);
}


