// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/reg_read.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	int	errs	= 0;

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
	} list[]	=
	{
		// reg					value
		{ HPDI32_GSC_FRR,		0		},
		{ HPDI32_GSC_BCR,		0		},
		{ HPDI32_GSC_BSR,		0		},
		{ HPDI32_GSC_TAR,		0		},
		{ HPDI32_GSC_RAR,		0		},
		{ HPDI32_GSC_FSR,		0		},
		{ HPDI32_GSC_FDR,		0		},
		{ HPDI32_GSC_TSVLCR,	0		},
		{ HPDI32_GSC_TLVLCR,	0		},
		{ HPDI32_GSC_TLILCR,	0		},
		{ HPDI32_GSC_RSCR,		0		},
		{ HPDI32_GSC_RLCR,		0		},
		{ HPDI32_GSC_ICR,		0		},
		{ HPDI32_GSC_ISR,		0		},
		{ HPDI32_GSC_TCDR,		0		},
		{ HPDI32_GSC_TFSR,		0		},
		{ HPDI32_GSC_RFSR,		0		},
		{ HPDI32_GSC_TFWR,		0		},
		{ HPDI32_GSC_RFWR,		0		},
		{ HPDI32_GSC_IELR,		0		},
		{ HPDI32_GSC_IHLR,		0		},

		{ GSC_PCI_9080_VIDR,	0x10B5	},
		{ GSC_PCI_9080_SVID,	0x10B5	},

		{ GSC_PLX_9080_VIDR,	0x10B5	},
	};

	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;

	errs	+= hpdi32_initialize(fd, -1, 0);

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= 0xDEADBEEF;
		parm.mask	= 0xBEEFDEAD;
		ret			= hpdi32_ioctl(fd, HPDI32_IOCTL_REG_READ, (void*) &parm);

		if (ret < 0)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, hpdi32_ioctl() failure, error %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if (parm.value == 0xDEADBEEF)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, value not changed)\n", __LINE__, i);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		if ((list[i].value) && (parm.value != list[i].value))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	reg_read_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_REG_READ.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_read_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_REG_READ");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


