// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/reg_write.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	int	errs	= 0;

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
		u32	mask;
	} list[]	=
	{
		// reg					value		mask
		{ HPDI32_GSC_TAR,		0xAAAA5555,	0xFFFFFFFF	},
		{ HPDI32_GSC_TAR,		0x000F000F,	0xFFFFFFFF	},

		{ HPDI32_GSC_TLVLCR,	0x5555AAAA,	0xFFFFFFFF	},
		{ HPDI32_GSC_TLVLCR,	0x00000020,	0xFFFFFFFF	},

		{ HPDI32_GSC_RAR,		0xAA5555AA,	0xFFFFFFFF	},
		{ HPDI32_GSC_RAR,		0x000F000F,	0xFFFFFFFF	},

		{ HPDI32_GSC_TSVLCR,	0x55AAAA55,	0xFFFFFFFF	},
		{ HPDI32_GSC_TSVLCR,	0x00000020,	0xFFFFFFFF	},
	};

	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;

	errs	+= hpdi32_initialize(fd, -1, 0);

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= list[i].value;
		parm.mask	= 0xBEEFDEAD;
		ret			= hpdi32_ioctl(fd, HPDI32_IOCTL_REG_WRITE, (void*) &parm);

		if (ret < 0)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, hpdi32_ioctl() failure, error %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		// Now read the register and verify the bits of interest.
		ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_REG_READ, (void*) &parm);

		if (ret < 0)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, hpdi32_ioctl() failure, error %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if ((list[i].value & list[i].mask) != (parm.value & list[i].mask))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, mask 0x%lX, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) list[i].mask,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	reg_write_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_REG_WRITE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_write_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_REG_WRITE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


