// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/rx_enable.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_NO,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x20,
			/* value	*/	0x00
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x20,
			/* value	*/	0x20
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_reg_get_list(fd, list);
	errs	+= service_ioctl_set_reg_list(fd, list);

	errs	+= service_ioctl_reg_get_list(fd, list);
	errs	+= service_ioctl_set_reg_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Verify the RX ENABLE option.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that the Rx FIFO is NOT empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x1000,
			/* value	*/	0x1000
		},

		// Verify the RX DISABLE option.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that the Rx FIFO IS empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x1000,
			/* value	*/	0x0000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_list(fd, list);
	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	rx_enable_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_RX_ENABLE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_enable_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_RX_ENABLE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


