// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/rx_fifo_af.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_AF,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_RAR,
			/* mask		*/	0xFFFF0000,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;
	int	i;

	for (i = 0; i <= 15; i++)
	{
		// Walking one.
		list[0].arg		= 1L << i;
		list[0].value	= list[0].arg << 16;

		errs	+= hpdi32_initialize(fd, -1, 0);

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		errs	+= hpdi32_initialize(fd, -1, 0);

		// Walking zero.
		list[0].arg		= 0xFFFF ^ (1L << i);
		list[0].value	= list[0].arg << 16;

		errs	+= hpdi32_initialize(fd, -1, 0);

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		errs	+= hpdi32_initialize(fd, -1, 0);
	}

	return(errs);
}



//*****************************************************************************
static int _rx_fifo_fill(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x100,
			/* value	*/	0x100
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (bsr & 0x8000)
		{
			// Rx FIFO is not FULL
			errs	+= service_ioctl_set_get_list(fd, list);
			continue;
		}

		if (bsr & 0x100)
		{
			// Tx FIFO is not EMPTY
			os_sleep_ms(1);
			continue;
		}
		else
		{
			// Tx FIFO is EMPTY
			break;
		}
	}

	if ((errs == 0) && (bsr & 0x8000))
	{
		errs++;
		printf("FAIL <---  (Rx FIFO did not fill)\n");
	}

	if ((errs == 0) && (bsr & 0x100))
	{
		errs++;
		printf("FAIL <---  (Tx FIFO did not empty)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _function_test_sub(int fd, int qty)
{
	const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_AF,
			/* arg		*/	qty,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x4000,
			/* value	*/	0x4000
		},

		{ SERVICE_END_LIST }
	};

	const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	0,
			/* arg		*/	qty,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x4000,
			/* value	*/	0x0000
		},
		{
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x4000,
			/* value	*/	0x4000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	// Setup.
	errs	+= service_ioctl_set_get_list(fd, list_1);
	errs	+= _rx_fifo_fill(fd);

	// Test.
	errs	+= service_ioctl_set_get_list(fd, list_2);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;

	errs	+= _function_test_sub(fd, 1024);
	errs	+= _function_test_sub(fd, 100);
	errs	+= _function_test_sub(fd, 50);
	errs	+= _function_test_sub(fd, 1);
	return(errs);
}



/******************************************************************************
*
*	Function:	rx_fifo_af_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_RX_FIFO_AF.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_fifo_af_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_RX_FIFO_AF");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


