// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/rx_fifo_overrun.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	u32	bsr;
	int	errs	= 0;
	int	ret;
	s32	status;

	errs	+= hpdi32_initialize(fd, -1, 0);
	status	= HPDI32_FIFO_ERROR_TEST;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_OVERRUN, &status);
	errs	+= ret ? 1 : 0;
	errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

	if (errs)
	{
	}
	else if (status != HPDI32_FIFO_ERROR_NO)
	{
		errs++;
		printf(	"ERROR <---  (Got status %ld, expected %ld.)\n",
				(long) status,
				(long) HPDI32_FIFO_ERROR_NO);
	}
	else if (bsr & 0x800000)
	{
		errs++;
		printf(	"ERROR <---  "
				"(Rx FIFO should not be overrun; got 0x%lX, expect %lX.)\n",
				(long) bsr & 0x800000,
				(long) 0);
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _rx_fifo_fill(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x100,
			/* value	*/	0x100
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (bsr & 0x8000)
		{
			// Rx FIFO is not FULL
			errs	+= service_ioctl_set_get_list(fd, list);
			continue;
		}

		if (bsr & 0x100)
		{
			// Tx FIFO is not EMPTY
			os_sleep_ms(1);
			continue;
		}
		else
		{
			// Tx FIFO is EMPTY
			break;
		}
	}

	if ((errs == 0) && (bsr & 0x8000))
	{
		errs++;
		printf("FAIL <---  (Rx FIFO did not fill)\n");
	}

	if ((errs == 0) && (bsr & 0x100))
	{
		errs++;
		printf("FAIL <---  (Tx FIFO did not empty)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x100,
			/* value	*/	0x100
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	val;

	errs	+= hpdi32_initialize(fd, -1, 0);

	// Fill the Rx FIFO.
	errs	+= _rx_fifo_fill(fd);

	// Overrun the Rx FIFO.
	errs	+= service_ioctl_set_get_list(fd, list);

	// Verify that the Rx FIFO is overrun.
	val		= HPDI32_FIFO_ERROR_TEST;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_OVERRUN, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != HPDI32_FIFO_ERROR_YES))
	{
		errs++;
		printf(	"FAIL <---  (%d. Rx FIFO not overrun)\n", __LINE__);
	}

	// Clear the Rx FIFO Overrun.
	val		= HPDI32_FIFO_ERROR_CLEAR;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_OVERRUN, &val);
	errs	+= ret ? 1 : 0;

	// Verify that the Rx FIFO is NOT overrun.
	val		= HPDI32_FIFO_ERROR_TEST;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_OVERRUN, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != HPDI32_FIFO_ERROR_NO))
	{
		errs++;
		printf(	"FAIL <---  (%d. Rx FIFO is overrun)\n", __LINE__);
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	rx_fifo_overrun_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_RX_FIFO_OVERRUN.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_fifo_overrun_test(int fd)
{
	int	errs;
	s32	get		= 0;

	gsc_label("HPDI32_IOCTL_RX_FIFO_OVERRUN");
	errs	= hpdi32_query(fd, -1, 0, HPDI32_QUERY_OVER_UNDER_RUN, &get);

	if (errs)
	{
	}
	else if (get == 0)
	{
		printf("SKIPPED  (Not supported on this device.)\n");
	}
	else
	{
		errs	+= _service_test(fd);
		errs	+= _function_test(fd);

		if (errs == 0)
			printf("PASS\n");
	}

	return(errs);
}


