// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/rx_fifo_reset.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x4,
			/* value	*/	0x0
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _rx_fifo_fill(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (bsr & 0x8000)
		{
			// Rx FIFO is not FULL
			errs	+= service_ioctl_set_list(fd, list);
			continue;
		}

		if (bsr & 0x100)
		{
			// Tx FIFO is not EMPTY
			os_sleep_ms(1);
			continue;
		}
		else
		{
			// Tx FIFO is EMPTY
			break;
		}
	}

	if ((errs == 0) && (bsr & 0x8000))
	{
		errs++;
		printf("FAIL <---  (Rx FIFO did not fill)\n");
	}

	if ((errs == 0) && (bsr & 0x100))
	{
		errs++;
		printf("FAIL <---  (Tx FIFO did not empty)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _ae_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Verify that the FIFO is AE when AE=1 and it contains one value.

		{				//	Set AE to 1.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_RAR,
			/* mask		*/	0,
			/* value	*/	0x00000001
		},
		{				//	Rx FIFO Reset
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Put one value in the Rx FIFO.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{				//	Verify that the FIFO is AE.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x2000,
			/* value	*/	0x0000
		},

		// Verify that the FIFO is not AE when AE=1 and it contains two values.

		// Put one more value in the Rx FIFO.

		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{				//	Verify that the FIFO is not AE.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x2000,
			/* value	*/	0x2000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _af_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{				//	Set AF to 1.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_RAR,
			/* mask		*/	0,
			/* value	*/	0x00010000
		},
		{				//	Rx FIFO Reset
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_STOP,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that the FIFO is AF.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x4000,
			/* value	*/	0x0000
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Read two value from the Rx FIFO.
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	0,
			/* arg		*/	2,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that the FIFO is not AF.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x4000,
			/* value	*/	0x4000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_list(fd, list_1);
	errs	+= _rx_fifo_fill(fd);
	errs	+= service_ioctl_set_list(fd, list_2);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _content_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Put data into the Rx FIFO.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Perform the tests.

		{				//	Verify that the FIFO is not empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x1000,
			/* value	*/	0x1000
		},
		{				//	Rx FIFO Reset
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that the FIFO is empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x1000,
			/* value	*/	0x0000
		},

		// Put data into the Rx FIFO.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Perform the tests.

		{				//	Verify that the FIFO is not empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x1000,
			/* value	*/	0x1000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _overrun_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{				//	Verify that there is an overrun.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x800000,
			/* value	*/	0x800000
		},
		{				//	Rx FIFO Reset
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that there is NOT an overrun.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x800000,
			/* value	*/	0x000000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	// Fill the Rx FIFO.
	errs	+= _rx_fifo_fill(fd);

	// Overrun the Rx FIFO.
	errs	+= service_ioctl_set_list(fd, list_1);

	// Perform validation.
	errs	+= service_ioctl_set_list(fd, list_2);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _underrun_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Induce an underrun.

		{
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Perform the tests.

		{				//	Verify that there is an underrun.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x400000,
			/* value	*/	0x400000
		},
		{				//	Rx FIFO Reset
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that there is NOT an overrun.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x400000,
			/* value	*/	0x000000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;
	s32	get		= 0;

	errs	+= _ae_test(fd);
	errs	+= _af_test(fd);
	errs	+= _content_test(fd);

	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_OVER_UNDER_RUN, &get);

	if (get)
	{
		errs	+= _overrun_test(fd);
		errs	+= _underrun_test(fd);
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	rx_fifo_reset_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_RX_FIFO_RESET.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_fifo_reset_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_RX_FIFO_RESET");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


