// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/rx_fifo_status.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	u32	bsr;
	int	errs	= 0;
	int	ret;
	s32	status;

	errs	+= hpdi32_initialize(fd, -1, 0);
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_STATUS, &status);
	errs	+= ret ? 1 : 0;
	errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

	if (errs)
	{
	}
	else if (status != HPDI32_FIFO_STATUS_EMPTY)
	{
		errs++;
		printf(	"ERROR <---  (Got status %ld, expected %ld.)\n",
				(long) status,
				(long) HPDI32_FIFO_STATUS_EMPTY);
	}
	else if ((bsr & 0xF000) != 0xC000)
	{
		errs++;
		printf(	"ERROR <---  (Rx FIFO is not empty; got 0x%lX, expect %lX.)\n",
				(long) bsr & 0xF000,
				(long) 0xC000);
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _rx_fifo_fill(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (bsr & 0x8000)
		{
			// Rx FIFO is not FULL
			errs	+= service_ioctl_set_list(fd, list);
			continue;
		}

		if (bsr & 0x100)
		{
			// Tx FIFO is not EMPTY
			os_sleep_ms(1);
			continue;
		}
		else
		{
			// Tx FIFO is EMPTY
			break;
		}
	}

	if ((errs == 0) && (bsr & 0x8000))
	{
		errs++;
		printf("FAIL <---  (Rx FIFO did not fill)\n");
	}

	if ((errs == 0) && (bsr & 0x100))
	{
		errs++;
		printf("FAIL <---  (Tx FIFO did not empty)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _status_test(int fd, s32 state, int status)
{
	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0xF000,
			/* value	*/	state
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	s32	fifo;
	int	ret;

	errs	+= service_ioctl_set_list(fd, list);
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_STATUS, &fifo);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (fifo != status))
	{
		errs++;
		printf(	"FAIL <---  (%d. status: expected %ld, got %ld)\n",
				__LINE__,
				(long) status,
				(long) fifo);
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	-1,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	u32	rar;

	errs	+= hpdi32_initialize(fd, -1, 0);
	errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_RAR, &rar);

	// Verify the Empty state.
	errs		+= _status_test(fd, 0xC000, HPDI32_FIFO_STATUS_EMPTY);

	// Verify the Almost Empty state.
	list[0].arg	= 1;
	errs		+= service_ioctl_set_list(fd, list);
	errs		+= _status_test(fd, 0xD000, HPDI32_FIFO_STATUS_ALMOST_EMPTY);

	list[0].arg	= (rar & 0xFFFF) - 1;
	errs		+= service_ioctl_set_list(fd, list);
	errs		+= _status_test(fd, 0xD000, HPDI32_FIFO_STATUS_ALMOST_EMPTY);

	// Verify the low Medium state.
	list[0].arg	= 1;
	errs		+= service_ioctl_set_list(fd, list);
	errs		+= _status_test(fd, 0xF000, HPDI32_FIFO_STATUS_MEDIUM);

	// Verify the Full state.
	errs		+= _rx_fifo_fill(fd);
	errs		+= _status_test(fd, 0x3000, HPDI32_FIFO_STATUS_FULL);

	// Verify the Almost Full state.
	list[0].service	= SERVICE_REG_READ;
	list[0].arg	= 1;
	errs		+= service_ioctl_set_list(fd, list);
	errs		+= _status_test(fd, 0xB000, HPDI32_FIFO_STATUS_ALMOST_FULL);

	list[0].arg	= (rar >> 16) - 1;
	errs		+= service_ioctl_set_list(fd, list);
	errs		+= _status_test(fd, 0xB000, HPDI32_FIFO_STATUS_ALMOST_FULL);

	// Verify the high Medium state.
	list[0].arg	= 1;
	errs		+= service_ioctl_set_list(fd, list);
	errs		+= _status_test(fd, 0xF000, HPDI32_FIFO_STATUS_MEDIUM);

	errs	+= hpdi32_initialize(fd, -1, 0);

	return(errs);
}



/******************************************************************************
*
*	Function:	rx_fifo_status_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_RX_FIFO_STATUS.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_fifo_status_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_RX_FIFO_STATUS");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


