// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/rx_fifo_underrun.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	u32	bsr;
	int	errs	= 0;
	int	ret;
	s32	status;

	errs	+= hpdi32_initialize(fd, -1, 0);
	status	= HPDI32_FIFO_ERROR_TEST;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_UNDERRUN, &status);
	errs	+= ret ? 1 : 0;
	errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

	if (errs)
	{
	}
	else if (status != HPDI32_FIFO_ERROR_NO)
	{
		errs++;
		printf(	"ERROR <---  (Got status %ld, expected %ld.)\n",
				(long) status,
				(long) HPDI32_FIFO_ERROR_NO);
	}
	else if (bsr & 0x400000)
	{
		errs++;
		printf(	"ERROR <---  "
				"(Rx FIFO should not be underrun; got 0x%lX, expect %lX.)\n",
				(long) bsr & 0x400000,
				(long) 0);
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	val;

	errs	+= hpdi32_initialize(fd, -1, 0);

	// Underrun the Rx FIFO.
	errs	+= service_ioctl_set_get_list(fd, list);

	// Verify that the Rx FIFO is Underrun.
	val		= HPDI32_FIFO_ERROR_TEST;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_UNDERRUN, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != HPDI32_FIFO_ERROR_YES))
	{
		errs++;
		printf(	"FAIL <---  (%d. Rx FIFO not underrun)\n", __LINE__);
	}

	// Clear the Rx FIFO Underrun.
	val		= HPDI32_FIFO_ERROR_CLEAR;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_UNDERRUN, &val);
	errs	+= ret ? 1 : 0;

	// Verify that the Rx FIFO is NOT Underrun.
	val		= HPDI32_FIFO_ERROR_TEST;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_UNDERRUN, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != HPDI32_FIFO_ERROR_NO))
	{
		errs++;
		printf(	"FAIL <---  (%d. Rx FIFO is underrun)\n", __LINE__);
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	rx_fifo_underrun_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_RX_FIFO_UNDERRUN.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_fifo_underrun_test(int fd)
{
	int	errs;
	s32	get		= 0;

	gsc_label("HPDI32_IOCTL_RX_FIFO_UNDERRUN");
	errs	= hpdi32_query(fd, -1, 0, HPDI32_QUERY_OVER_UNDER_RUN, &get);

	if (errs)
	{
	}
	else if (get == 0)
	{
		printf("SKIPPED  (Not supported on this device.)\n");
	}
	else
	{
		errs	+= _service_test(fd);
		errs	+= _function_test(fd);

		if (errs == 0)
			printf("PASS\n");
	}

	return(errs);
}


