// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/rx_io_overrun.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_OVERRUN,
			/* arg		*/	HPDI32_IO_ERROR_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_OVERRUN,
			/* arg		*/	HPDI32_IO_ERROR_CHECK,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_get_list(fd, list);
	errs	+= service_ioctl_set_get_list(fd, list);
	errs	+= hpdi32_initialize(fd, -1, 0);

	return(errs);
}



//*****************************************************************************
static int _rx_fifo_fill(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x100,
			/* value	*/	0x100
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (bsr & 0x8000)
		{
			// Rx FIFO is not FULL
			errs	+= service_ioctl_set_get_list(fd, list);
			continue;
		}

		if (bsr & 0x100)
		{
			// Tx FIFO is not EMPTY
			os_sleep_ms(1);
			continue;
		}
		else
		{
			// Tx FIFO is EMPTY
			break;
		}
	}

	if ((errs == 0) && (bsr & 0x8000))
	{
		errs++;
		printf("FAIL <---  (Rx FIFO did not fill)\n");
	}

	if ((errs == 0) && (bsr & 0x100))
	{
		errs++;
		printf("FAIL <---  (Tx FIFO did not empty)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	ignore[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_OVERRUN,
			/* arg		*/	HPDI32_IO_ERROR_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	check[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_OVERRUN,
			/* arg		*/	HPDI32_IO_ERROR_CHECK,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	clear[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_OVERRUN,
			/* arg		*/	HPDI32_FIFO_ERROR_CLEAR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	char	buf[4];
	int		errs	= 0;
	long	ret;

	errs	+= hpdi32_initialize(fd, -1, 0);

	// Fill the Rx FIFO.
	errs	+= _rx_fifo_fill(fd);

	// Overrun the Rx FIFO.
	errs	+= service_ioctl_set_list(fd, list);

	// Ignore overruns
	errs	+= service_ioctl_set_list(fd, ignore);
	ret		= hpdi32_read(fd, buf, 4);

	if ((errs == 0) && (ret != 4))
	{
		errs++;
		printf("FAIL <---  (%d. overrun was not ignored)\n", __LINE__);
	}

	// Check overruns
	errs	+= service_ioctl_set_list(fd, check);
	ret		= hpdi32_read(fd, buf, 4);

	if ((errs == 0) && (ret >= 0))
	{
		errs++;
		printf("FAIL <---  (%d. overrun was ignored)\n", __LINE__);
	}

	// Clear the overrun and check again.
	errs	+= service_ioctl_set_list(fd, clear);
	ret		= hpdi32_read(fd, buf, 4);

	if ((errs == 0) && (ret != 4))
	{
		errs++;
		printf("FAIL <---  (%d. overrun was observed)\n", __LINE__);
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	rx_io_overrun_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_RX_IO_OVERRUN.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_io_overrun_test(int fd)
{
	int	errs;
	s32	get		= 0;

	gsc_label("HPDI32_IOCTL_RX_IO_OVERRUN");
	errs	= hpdi32_query(fd, -1, 0, HPDI32_QUERY_OVER_UNDER_RUN, &get);

	if (errs)
	{
	}
	else if (get == 0)
	{
		printf("SKIPPED  (Not supported on this device.)\n");
	}
	else
	{
		errs	+= _service_test(fd);
		errs	+= _function_test(fd);

		if (errs == 0)
			printf("PASS\n");
	}

	return(errs);
}


