// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/tx_auto_start.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_AUTO_START,
			/* arg		*/	HPDI32_AUTO_START_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_AUTO_START,
			/* arg		*/	HPDI32_AUTO_START_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_get_list(fd, list);
	errs	+= service_ioctl_set_get_list(fd, list);
	errs	+= hpdi32_initialize(fd, -1, 0);

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	disable[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_STOP,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_AUTO_START,
			/* arg		*/	HPDI32_AUTO_START_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	check_not_empty[]	=
	{
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x100,
			/* value	*/	0x100
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	enable[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_STOP,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_AUTO_START,
			/* arg		*/	HPDI32_AUTO_START_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	check_empty[]	=
	{
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x100,
			/* value	*/	0x000
		},

		{ SERVICE_END_LIST }
	};

	char	buf[32];
	int		errs	= 0;
	long	ret;

	// Verify that the DISABLE option works.
	memset(buf, 0, sizeof(buf));
	errs	+= hpdi32_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_list(fd, disable);

	if (errs == 0)
	{
		ret	= hpdi32_write(fd, buf, 4);

		if (ret != 4)
		{
			errs++;
			printf(	"FAIL <---  (%d. sent %ld, expected %ld)\n",
					__LINE__,
					(long) ret,
					(long) 1);
		}
	}

	if (errs == 0)
		errs	+= service_ioctl_set_list(fd, check_not_empty);

	// Verify that the ENABLE option works.
	errs	+= hpdi32_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_list(fd, enable);

	if (errs == 0)
	{
		ret	= hpdi32_write(fd, buf, 4);

		if (ret != 4)
		{
			errs++;
			printf(	"FAIL <---  (%d. sent %ld, expected %ld)\n",
					__LINE__,
					(long) ret,
					(long) 1);
		}
	}

	if (errs == 0)
		errs	+= service_ioctl_set_list(fd, check_empty);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	tx_auto_start_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_TX_AUTO_START.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int tx_auto_start_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_TX_AUTO_START");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


