// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/tx_fifo_af.c $
// $Rev: 52308 $
// $Date: 2022-12-20 15:44:14 -0600 (Tue, 20 Dec 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FIFO_AF,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_TAR,
			/* mask		*/	0xFFFF0000,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;
	int	i;

	for (i = 0; i <= 15; i++)
	{
		// Walking one.
		list[0].arg		= 1L << i;
		list[0].value	= list[0].arg << 16;

		errs	+= hpdi32_initialize(fd, -1, 0);

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		errs	+= hpdi32_initialize(fd, -1, 0);

		// Walking zero.
		list[0].arg		= 0xFFFF ^ (1L << i);
		list[0].value	= list[0].arg << 16;

		errs	+= hpdi32_initialize(fd, -1, 0);

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		errs	+= hpdi32_initialize(fd, -1, 0);
	}

	return(errs);
}



//*****************************************************************************
static int _compute_fifo_size(int fd, s32* size)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	size[0]	= 0;

	for (i = 0; i < 1024; i++)
	{
		errs	+= service_ioctl_set_get_list(fd, list);
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);
		size[0]	+= 1024;

		if (bsr & 0x800)
		{
			// The Tx FIFO is not FULL
			continue;
		}
		else
		{
			// The Tx FIFO is FULL.
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _function_test_sub(int fd, int ae, int qty, u32 value)
{
	const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FIFO_AF,
			/* arg		*/	ae,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x200,
			/* value	*/	0x200
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	qty,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x400,
			/* value	*/	value
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_get_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;
	s32	size;

	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_FIFO_SIZE_TX, &size);

	if ((errs == 0) && (size <= 0))
		errs	+= _compute_fifo_size(fd, &size);

	if ((errs == 0) && (size > 0))
	{
		errs	+= _function_test_sub(fd, 1024, size - 1024, 0x000);
		errs	+= _function_test_sub(fd, 1024, size - 1025, 0x400);

		errs	+= _function_test_sub(fd,  100, size -  100, 0x000);
		errs	+= _function_test_sub(fd,  100, size -  101, 0x400);

		errs	+= _function_test_sub(fd,	50, size -	 50, 0x000);
		errs	+= _function_test_sub(fd,	50, size -	 51, 0x400);

		errs	+= _function_test_sub(fd,	 1, size -	  1, 0x000);
		errs	+= _function_test_sub(fd,	 1, size -	  2, 0x400);
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	tx_fifo_af_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_TX_FIFO_AF.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int tx_fifo_af_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_TX_FIFO_AF");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


