// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/tx_fifo_overrun.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	u32	bsr;
	int	errs	= 0;
	int	ret;
	s32	status;

	errs	+= hpdi32_initialize(fd, -1, 0);
	status	= HPDI32_FIFO_ERROR_TEST;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_FIFO_OVERRUN, &status);
	errs	+= ret ? 1 : 0;
	errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

	if (errs)
	{
	}
	else if (status != HPDI32_FIFO_ERROR_NO)
	{
		errs++;
		printf(	"ERROR <---  (Got status %ld, expected %ld.)\n",
				(long) status,
				(long) HPDI32_FIFO_ERROR_NO);
	}
	else if (bsr & 0x200000)
	{
		errs++;
		printf(	"ERROR <---  "
				"(Tx FIFO should not be overrun; got 0x%lX, expect %lX.)\n",
				(long) bsr & 0x200000,
				(long) 0);
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _tx_fifo_fill(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= service_ioctl_set_list(fd, list);
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (bsr & 0x800)
		{
			// The Tx FIFO is not FULL
			continue;
		}
		else
		{
			// The Tx FIFO is FULL.
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	val;

	errs	+= hpdi32_initialize(fd, -1, 0);

	// Fill the Tx FIFO.
	errs	+= _tx_fifo_fill(fd);

	// Overrun the Tx FIFO.
	errs	+= service_ioctl_set_get_list(fd, list);

	// Verify that the Tx FIFO is overrun.
	val		= HPDI32_FIFO_ERROR_TEST;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_FIFO_OVERRUN, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != HPDI32_FIFO_ERROR_YES))
	{
		errs++;
		printf(	"FAIL <---  (%d. Tx FIFO not overrun)\n", __LINE__);
	}

	// Clear the Tx FIFO Overrun.
	val		= HPDI32_FIFO_ERROR_CLEAR;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_FIFO_OVERRUN, &val);
	errs	+= ret ? 1 : 0;

	// Verify that the Tx FIFO is NOT overrun.
	val		= HPDI32_FIFO_ERROR_TEST;
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_FIFO_OVERRUN, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != HPDI32_FIFO_ERROR_NO))
	{
		errs++;
		printf(	"FAIL <---  (%d. Tx FIFO is overrun)\n", __LINE__);
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	tx_fifo_overrun_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_TX_FIFO_OVERRUN.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int tx_fifo_overrun_test(int fd)
{
	int	errs;
	s32	get		= 0;


	gsc_label("HPDI32_IOCTL_TX_FIFO_OVERRUN");
	errs	= hpdi32_query(fd, -1, 0, HPDI32_QUERY_OVER_UNDER_RUN, &get);

	if (errs)
	{
	}
	else if (get == 0)
	{
		printf("SKIPPED  (Not supported on this device.)\n");
	}
	else
	{
		errs	+= _service_test(fd);
		errs	+= _function_test(fd);

		if (errs == 0)
			printf("PASS\n");
	}

	return(errs);
}


