// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/tx_fifo_reset.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x2,
			/* value	*/	0x0
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _compute_fifo_size(int fd, s32* size)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	size[0]	= 0;

	for (i = 0; i < 1024; i++)
	{
		errs	+= service_ioctl_set_get_list(fd, list);
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);
		size[0]	+= 1024;

		if (bsr & 0x800)
		{
			// The Tx FIFO is not FULL
			continue;
		}
		else
		{
			// The Tx FIFO is FULL.
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _ae_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Set AE=1.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Set AE to 1.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_TAR,
			/* mask		*/	0,
			/* value	*/	0x00000001
		},
		{				//	Tx FIFO Reset
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Verify that the FIFO is AE when AE=1 and it contains one value.

		{				//	Put one value in the FIFO.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that the FIFO is AE.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x200,
			/* value	*/	0x000
		},

		// Verify that the FIFO is not AE when AE=1 and it contains two values.

		{				//	Put another value in the FIFO.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	10000,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that the FIFO is not AE.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x200,
			/* value	*/	0x200
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _af_test(int fd, s32 qty)
{
	const service_data_t	list[]	=
	{
		// Set AF=1.

		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_TAR,
			/* mask		*/	0,
			/* value	*/	0x00010000
		},
		{				//	Tx FIFO Reset
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Verify that the FIFO is not AF when AF=1 and it has 2 empy places.

		{				//	Put SIZE-2 values in the FIFO.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	qty - 10000,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that the FIFO is not AF.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x400,
			/* value	*/	0x400
		},

		// Verify that the FIFO is AF when AF=1 and it has 1 empy places.

		{				//	Put another value in the FIFO.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	9999,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that the FIFO is AF.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x400,
			/* value	*/	0x000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _content_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Put data into the Tx FIFO.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Perform the tests.

		{				//	Verify that the FIFO is not empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x100,
			/* value	*/	0x100
		},
		{				//	Tx FIFO Reset
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that the FIFO is empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x100,
			/* value	*/	0x000
		},

		// Put data into the Tx FIFO.

		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Perform the tests.

		{				//	Verify that the FIFO is not empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x100,
			/* value	*/	0x100
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _overrun_test(int fd, s32 qty)
{
	const service_data_t	list[]	=
	{
		//	Overrun the FIFO.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	qty + 10,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Perform tests.

		{				//	Verify that there is an overrun.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x200000,
			/* value	*/	0x200000
		},
		{				//	Tx FIFO Reset
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Verify that there is NOT an overrun.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x200000,
			/* value	*/	0x000000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_list(fd, list);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;
	s32	get		= 0;
	s32	size;

	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_FIFO_SIZE_TX, &size);

	if ((errs == 0) && (size <= 0))
		errs	+= _compute_fifo_size(fd, &size);

	if ((errs == 0) && (size > 0))
	{
		errs	+= _ae_test(fd);
		errs	+= _af_test(fd, size);
		errs	+= _content_test(fd);

		errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_OVER_UNDER_RUN, &get);

		if (get)
			errs	+= _overrun_test(fd, size);
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	tx_fifo_reset_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_TX_FIFO_RESET.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int tx_fifo_reset_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_TX_FIFO_RESET");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


