// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/tx_flow_control.c $
// $Rev: 51590 $
// $Date: 2022-10-05 07:04:06 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_AUTO_STOP,
			/* arg		*/	HPDI32_AUTO_STOP_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_STOP,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x100,
			/* value	*/	0x000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x100,
			/* value	*/	0x100
		},

		{ SERVICE_END_LIST }
	};

	s32	as;
	int	errs;

	errs	= hpdi32_query(fd, -1, 0, HPDI32_QUERY_TX_AUTO_STOP, &as);

	if (as)
	{
		errs	+= hpdi32_initialize(fd, -1, 0);

		errs	+= service_ioctl_set_get_list(fd, list_1);
		errs	+= service_ioctl_reg_get_list(fd, list_2);
		errs	+= service_ioctl_set_reg_list(fd, list_2);

		errs	+= service_ioctl_set_get_list(fd, list_1);
		errs	+= service_ioctl_reg_get_list(fd, list_2);
		errs	+= service_ioctl_set_reg_list(fd, list_2);

		errs	+= hpdi32_initialize(fd, -1, 0);
	}

	return(errs);
}



//*****************************************************************************
static int _tx_fifo_fill(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (bsr & 0x800)
		{
			// Tx FIFO is not FULL
			errs	+= service_ioctl_set_get_list(fd, list);
			continue;
		}

		// Tx FIFO is FULL
		break;
	}

	if ((errs == 0) && (bsr & 0x800))
	{
		errs++;
		printf("FAIL <---  (Tx FIFO did not fill)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	stop[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_STOP,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x100,
			/* value	*/	0x000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	start[]	=
	{
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x800,
			/* value	*/	0x000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BSR,
			/* mask		*/	0x100,
			/* value	*/	0x000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hpdi32_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_list(fd, stop);
	errs	+= _tx_fifo_fill(fd);
	errs	+= service_ioctl_set_list(fd, start);

	errs	+= hpdi32_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_list(fd, stop);
	errs	+= _tx_fifo_fill(fd);
	errs	+= service_ioctl_set_list(fd, start);

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	tx_flow_control_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_TX_FLOW_CONTROL.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int tx_flow_control_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_TX_FLOW_CONTROL");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


