// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/tx_io_abort.c $
// $Rev: 52308 $
// $Date: 2022-12-20 15:44:14 -0600 (Tue, 20 Dec 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	os_sem_t	sem_enter;
static	os_sem_t	sem_exit;



//*****************************************************************************
static int _service_test(void)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _tx_fifo_fill(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_STOP,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	errs	+= service_ioctl_set_list(fd, list_1);

	for (i = 0; i < 1024; i++)
	{
		errs	+= service_ioctl_set_list(fd, list_2);
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (bsr & 0x800)
		{
			// The Tx FIFO is not FULL
			continue;
		}
		else
		{
			// The Tx FIFO is FULL.
			errs	+= service_ioctl_set_list(fd, list_2);
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _abort_none_test(int fd)
{
	int	errs	= 0;
	s32	get;
	int	ret;

	errs	+= hpdi32_initialize(fd, -1, 0);
	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_IO_ABORT, &get);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (get))
	{
		errs++;
		printf(	"FAIL <---  (%d. got %ld, expected %ld)\n",
				__LINE__,
				(long) get,
				(long) 0);
	}

	errs	+= hpdi32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _tx_thread(void* arg)
{
	char	buffer[1024];
	int		fd		= (VP2I) arg;

	memset(buffer, 0, sizeof(buffer));
	os_sem_unlock(&sem_enter);
	hpdi32_write(fd, buffer, sizeof(buffer));
	os_sem_unlock(&sem_exit);
	return(0);
}



//*****************************************************************************
static int _abort_one_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_AUTO_START,
			/* arg		*/	HPDI32_AUTO_START_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_IO_OVERRUN,
			/* arg		*/	HPDI32_IO_ERROR_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_IO_TIMEOUT,
			/* arg		*/	15,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	s32			get;
	int			i;
	int			ret;
	os_thread_t	thread;

	errs	+= hpdi32_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_list(fd, list);
	errs	+= _tx_fifo_fill(fd);

	os_sem_create(&sem_enter);
	os_sem_create(&sem_exit);
	memset(&thread, 0, sizeof(thread));
	errs	+= os_thread_create(
				&thread,
				"Tx I/O Abort",
				_tx_thread,
				(I2VP) fd);

	// Wait for the thread to execute.
	os_sem_lock(&sem_enter);

	if (errs == 0)
	{
		for (i = 0; i < 20; i++)
		{
			ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_IO_ABORT, &get);
			errs	+= ret ? 1 : 0;

			if ((errs) || (get))
				break;

			os_sleep_ms(1000);
		}

		if (errs)
		{
		}
		else if (get == 0)
		{
			errs++;
			printf("FAIL <---  (%d. abort not detected)\n", __LINE__);
		}
	}

	// Wait for the thread to finish.
	os_sem_lock(&sem_exit);

	// Finish up.
	errs	+= os_thread_destroy(&thread);
	errs	+= hpdi32_initialize(fd, -1, 0);
	os_sem_destroy(&sem_enter);
	os_sem_destroy(&sem_exit);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int errs	= 0;

	errs	+= _abort_none_test(fd);
	errs	+= _abort_one_test(fd);
	return(errs);
}



/******************************************************************************
*
*	Function:	tx_io_abort_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_TX_IO_ABORT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int tx_io_abort_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_TX_IO_ABORT");
	errs	+= _service_test();
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


