// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/sbtest/wait_event.c $
// $Rev: 52308 $
// $Date: 2022-12-20 15:44:14 -0600 (Tue, 20 Dec 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1_SEC		(1000L)
#define	_15_SEC		(15L * _1_SEC)
#define	_30_SEC		(30L * _1_SEC)



// variables ******************************************************************

static	int			rx_stat;
static	os_sem_t	sem_enter;
static	os_sem_t	sem_exit;
static	int			tx_stat;
static	gsc_wait_t	wait_dst;
static	int			wait_fd;



//*****************************************************************************
static int _service_test(void)
{
	// There are no settings bits to test here.
	return(0);
}



//*****************************************************************************
static int _rx_fifo_fill(int fd, u32 bsr_mask, u32 bsr_value)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				//	Tx Start
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HPDI32_GSC_BCR,
			/* mask		*/	0x100,
			/* value	*/	0x100
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if ((bsr & bsr_mask) != bsr_value)
		{
			// Rx FIFO is not full enough.
			errs	+= service_ioctl_set_get_list(fd, list);
			continue;
		}

		if (bsr & 0x100)
		{
			// Tx FIFO is not EMPTY
			os_sleep_ms(1);
			continue;
		}
		else
		{
			// Tx FIFO is EMPTY
			break;
		}
	}

	if ((errs == 0) && ((bsr & bsr_mask) != bsr_value))
	{
		errs++;
		printf("FAIL <---  (Rx FIFO did not fill enough.)\n");
	}

	if ((errs == 0) && (bsr & 0x100))
	{
		errs++;
		printf("FAIL <---  (Tx FIFO did not empty.)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _tx_fifo_fill(int fd, u32 bsr_mask, u32 bsr_value)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	bsr;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= service_ioctl_set_list(fd, list);
		errs	+= hpdi32_reg_read(fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if ((bsr & bsr_mask) == bsr_value)
			break;
	}

	return(errs);
}



//*****************************************************************************
static int _wait_thread(void* arg)
{
	int	ret;

	os_sem_unlock(&sem_enter);
	ret	= hpdi32_ioctl(wait_fd, HPDI32_IOCTL_WAIT_EVENT, &wait_dst);

	if (ret < 0)
		wait_dst.flags	= 0xFFFFFFFF;

	os_sem_unlock(&sem_exit);
	return(0);
}



//*****************************************************************************
static int _wait_main_dma(int fd)
{
	static const service_data_t	list[]	=
	{
		{					// Select DMA mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_BMDMA,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_OVERRUN,
			/* arg		*/	HPDI32_IO_ERROR_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024];
	int	errs	= 0;
	s32	ret;

	memset(buf, 0, sizeof(buf));

	errs	+= service_ioctl_set_list(fd, list);
	ret		= hpdi32_write(fd, buf, sizeof(buf));

	if (ret != sizeof(buf))
	{
		errs++;
		printf(	"FAIL <---  (%d. write failure: returned %d, expected %d)\n",
				__LINE__,
				ret,
				(int) sizeof(buf));
	}

	return(errs);
}



//*****************************************************************************
static int _wait_main_gsc(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0001,
			/* reg		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* mask		*/	0x0001,
			/* value	*/	0x0001
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int _tx_data(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_fv_start(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0001,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list);
	errs	+= _tx_data(fd);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_fv_end(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0002,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list);
	errs	+= _tx_data(fd);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_cc1_lv_gpio0(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0004,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list);
	errs	+= _tx_data(fd);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_cc2_sv_gpio1(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_CABLE_CMD_MODE_2,
			/* arg		*/	HPDI32_CABLE_CMD_MODE_FC,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_STATUS_VAL_CNT,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0008,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
#if 1
// Disable this code to see if we can get the Tx Status
// interrupt by transmitting data.
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_CABLE_CMD_MODE_2,
			/* arg		*/	HPDI32_CABLE_CMD_MODE_OUT_LO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_CABLE_CMD_MODE_2,
			/* arg		*/	HPDI32_CABLE_CMD_MODE_OUT_HI,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
#endif
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list_1);
	errs	+= _tx_data(fd);
	errs	+= service_ioctl_set_list(fd, list_2);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_cc3_rr_gpio2(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0010,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list_1);
	errs	+= _rx_fifo_fill(fd, 0x8000, 0x0000);
	errs	+= service_ioctl_set_list(fd, list_2);

	return(errs);
}



//*****************************************************************************
static int _wait_gsc_cc4_tr_gpio3(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0020,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_cc5_te_gpio4(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0040,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_cc6_re_gpio5(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0080,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_ENABLE,
			/* arg		*/	HPDI32_RX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_tx_fifo_e(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	HPDI32_GSC_FDR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_ENABLE,
			/* arg		*/	HPDI32_TX_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_tx_fifo_ae(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0200,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list_1);
	errs	+= _tx_fifo_fill(fd, 0x200, 0x200);
	errs	+= service_ioctl_set_list(fd, list_2);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_tx_fifo_af(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0400,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list);
	errs	+= _tx_fifo_fill(fd, 0x400, 0x000);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_tx_fifo_f(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x0800,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list);
	errs	+= _tx_fifo_fill(fd, 0x800, 0x000);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_rx_fifo_e(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x1000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_CONFIG_EDGE,
			/* arg		*/	0x1000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_CONFIG_HIGH,
			/* arg		*/	0x1000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list_1);
	errs	+= _rx_fifo_fill(fd, 0x1000, 0x1000);
	errs	+= service_ioctl_set_list(fd, list_2);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_rx_fifo_ae(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x2000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_CONFIG_EDGE,
			/* arg		*/	0x2000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_CONFIG_HIGH,
			/* arg		*/	0x2000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_FIFO_RESET,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list_1);
	errs	+= _rx_fifo_fill(fd, 0x2000, 0x2000);
	errs	+= service_ioctl_set_list(fd, list_2);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_rx_fifo_af(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x4000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_CONFIG_EDGE,
			/* arg		*/	0x4000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_CONFIG_HIGH,
			/* arg		*/	0x4000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list);
	errs	+= _rx_fifo_fill(fd, 0x4000, 0x0000);
	return(errs);
}



//*****************************************************************************
static int _wait_gsc_rx_fifo_f(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_ENABLE,
			/* arg		*/	0x8000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_CONFIG_EDGE,
			/* arg		*/	0x8000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_IRQ_CONFIG_HIGH,
			/* arg		*/	0x8000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_list(fd, list);
	errs	+= _rx_fifo_fill(fd, 0x8000, 0x0000);
	return(errs);
}



//*****************************************************************************
static int _wait_io_tx_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[100];
	int	errs	= 0;
	int	ret;

	memset(buf, 0, sizeof(buf));
	errs	+= service_ioctl_set_list(fd, list);
	ret		= hpdi32_write(fd, buf, sizeof(buf));

	if (ret != sizeof(buf))
	{
		errs++;
		printf(	"FAIL <---  (%d. write failure: returned %d, expected %d)\n",
				__LINE__,
				ret,
				(int) sizeof(buf));
	}

	return(errs);
}



//*****************************************************************************
static int _wait_io_tx_error(int fd)
{
	hpdi32_write(fd, NULL, 3);
	return(0);
}



//*****************************************************************************
static int _wait_io_tx_timeout(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_IO_OVERRUN,
			/* arg		*/	HPDI32_IO_ERROR_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_AUTO_START,
			/* arg		*/	HPDI32_AUTO_START_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32		buf[32];
	int		errs	= 0;
	long	ret;

	memset(buf, 0, sizeof(buf));
	errs	+= service_ioctl_set_list(fd, list);
	errs	+= _tx_fifo_fill(fd, 0x0800, 0x0000);
	ret		= hpdi32_write(fd, buf, sizeof(buf));

	if (ret != 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. hpdi32_write(): got %ld, expected %ld\n",
				__LINE__,
				(long) ret,
				(long) 0);
	}

	return(errs);
}



//*****************************************************************************
static int _write_thread(void* arg)
{
	u32		buf[1024];
	int		fd		= (VP2I) arg;
	long	ret;

	memset(buf, 0, sizeof(buf));
	ret	= hpdi32_write(fd, buf, sizeof(buf));

	if (ret == 0)
	{
		tx_stat	= 1;
	}
	else
	{
		tx_stat	= 2;
		printf("FAIL <---  (%d. write returned %ld, expected 0)\n",
				__LINE__,
				(long) ret);
	}

	return(0);
}



//*****************************************************************************
static int	_wait_io_tx_abort(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_AUTO_START,
			/* arg		*/	HPDI32_AUTO_START_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_IO_OVERRUN,
			/* arg		*/	HPDI32_IO_ERROR_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_IO_TIMEOUT,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_TX_FLOW_CONTROL,
			/* arg		*/	HPDI32_TX_FLOW_CONTROL_STOP,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	int			i;
	int			ret;
	s32			set;
	os_thread_t	thread;

	errs	+= service_ioctl_set_reg_list(fd, list_1);
	errs	+= _tx_fifo_fill(fd, 0x800, 0x000);
	errs	+= service_ioctl_set_reg_list(fd, list_2);

	// Now start the write thread so we can do the abort.
	memset(&thread, 0, sizeof(thread));

	if (errs == 0)
	{
		errs	+= os_thread_create(	&thread,
										"Tx",
										_write_thread,
										(I2VP) fd);
	}

	for (i = 0; (errs == 0) && (i < 100); i++)
	{
		set		= 0;
		ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_IO_ABORT, &set);
		errs	+= ret ? 1 : 0;

		if (errs)
			break;

		if (set)
			break;

		os_sleep_ms(100);
	}

	// Wait for the thread to report its status.

	for (i = 0; i < 100; i++)
	{
		if (errs)
			break;

		if (tx_stat == 0)
		{
			os_sleep_ms(100);
			continue;
		}

		if (tx_stat == 1)
			break;

		errs++;
		break;
	}

	errs	+= os_thread_destroy(&thread);
	return(errs);
}



//*****************************************************************************
static int _wait_io_rx_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1];
	int	errs	= 0;
	int	ret;

	errs	+= service_ioctl_set_list(fd, list);
	errs	+= _rx_fifo_fill(fd, 0x1000, 0x1000);
	ret		= hpdi32_read(fd, buf, sizeof(buf));

	if (ret != sizeof(buf))
	{
		errs++;
		printf(	"FAIL <---  (%d. read failure: returned %d, expected %d)\n",
				__LINE__,
				ret,
				(int) sizeof(buf));
	}

	return(errs);
}



//*****************************************************************************
static int _wait_io_rx_error(int fd)
{
	hpdi32_read(fd, NULL, 3);
	return(0);
}



//*****************************************************************************
static int _wait_io_rx_timeout(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_OVERRUN,
			/* arg		*/	HPDI32_IO_ERROR_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_UNDERRUN,
			/* arg		*/	HPDI32_IO_ERROR_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_AUTO_START,
			/* arg		*/	HPDI32_AUTO_START_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32		buf[32];
	int		errs;
	long	ret;

	errs	= service_ioctl_set_list(fd, list);
	ret		= hpdi32_read(fd, buf, sizeof(buf));

	if (ret != 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. hpdi32_read(): got %ld, expected 0\n",
				__LINE__,
				(long) ret);
	}

	return(errs);
}



//*****************************************************************************
static int _read_thread(void* arg)
{
	u32		buf[1024];
	int		fd		= (VP2I) arg;
	long	ret;

	ret	= hpdi32_read(fd, buf, sizeof(buf));

	if (ret == 0)
	{
		rx_stat	= 1;
	}
	else
	{
		rx_stat	= 2;
		printf("FAIL <---  (%d. read returned %ld, expected 0)\n",
				__LINE__,
				(long) ret);
	}

	return(0);
}



//*****************************************************************************
static int	_wait_io_rx_abort(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_OVERRUN,
			/* arg		*/	HPDI32_IO_ERROR_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_UNDERRUN,
			/* arg		*/	HPDI32_IO_ERROR_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HPDI32_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs;
	int			i;
	int			ret;
	s32			set;
	os_thread_t	thread;

	errs	= service_ioctl_set_reg_list(fd, list);

	// Now start the read thread so we can do the abort.
	memset(&thread, 0, sizeof(thread));

	if (errs == 0)
	{
		errs	= os_thread_create(	&thread,
									"Rx",
									_read_thread,
									(I2VP) fd);
	}

	for (i = 0; (errs == 0) && (i < 100); i++)
	{
		set		= 0;
		ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_IO_ABORT, &set);
		errs	+= ret ? 1 : 0;

		if (errs)
			break;

		if (set)
			break;

		os_sleep_ms(100);
	}

	// Wait for the thread to report its status.

	for (i = 0; i < 100; i++)
	{
		if (errs)
			break;

		if (rx_stat == 0)
		{
			os_sleep_ms(100);
			continue;
		}

		if (rx_stat == 1)
			break;

		errs++;
		break;
	}

	errs	+= os_thread_destroy(&thread);
	return(errs);
}



//*****************************************************************************
static int	_wait_timeout(int fd)
{
	os_sleep_ms(2000);
	return(0);
}



//*****************************************************************************
static int	_wait_cancel(int fd)
{
	int			errs;
	int			ret;
	gsc_wait_t	wait;

	wait.flags			= 0;
	wait.main			= 0xFFFFFFFF;
	wait.gsc			= 0xFFFFFFFF;
	wait.alt			= 0xFFFFFFFF;
	wait.io				= 0xFFFFFFFF;
	wait.timeout_ms		= 0;
	wait.count			= 0;
	ret		= hpdi32_ioctl(wait_fd, HPDI32_IOCTL_WAIT_CANCEL, &wait);
	errs	= ret ? 1 : 0;
	return(errs);
}



// variables ******************************************************************

static const struct
{
	const char*	name;
	int			(*function)(int fd);
	gsc_wait_t	wait;
} wait_src[]	=
{
	// name							function					flags			main							...			timeout_ms
	{ "DMA0/DMA1",					_wait_main_dma,				{ 0,			GSC_WAIT_MAIN_DMA0 |
																				GSC_WAIT_MAIN_DMA1,				0, 0, 0,	_15_SEC	}	},
	{ "GSC",						_wait_main_gsc,				{ 0,			GSC_WAIT_MAIN_GSC,				0, 0, 0,	_15_SEC	}	},

	// name							function					flags ...		gsc								...			timeout_ms
	{ "Frame Valid Start",			_wait_gsc_fv_start,			{ 0, 0,			HPDI32_WAIT_GSC_CC0_FV_S_GPIO6,	0, 0,		_15_SEC	}	},
	{ "Frame Valid End",			_wait_gsc_fv_end,			{ 0, 0,			HPDI32_WAIT_GSC_CC0_FV_E_GPIO6,	0, 0,		_15_SEC	}	},
	{ "Cable Command 1 (LV/GPIO0)",	_wait_gsc_cc1_lv_gpio0,		{ 0, 0,			HPDI32_WAIT_GSC_CC1_LV_GPIO0,	0, 0,		_15_SEC	}	},
	{ "Cable Command 2 (SV/GPIO1)",	_wait_gsc_cc2_sv_gpio1,		{ 0, 0,			HPDI32_WAIT_GSC_CC2_SV_GPIO1,	0, 0,		_15_SEC	}	},
	{ "Cable Command 3 (RR/GPIO2)",	_wait_gsc_cc3_rr_gpio2,		{ 0, 0,			HPDI32_WAIT_GSC_CC3_RR_GPIO2,	0, 0,		_15_SEC	}	},
	{ "Cable Command 4 (TR/GPIO3)",	_wait_gsc_cc4_tr_gpio3,		{ 0, 0,			HPDI32_WAIT_GSC_CC4_TR_GPIO3,	0, 0,		_15_SEC	}	},
	{ "Cable Command 5 (TE/GPIO4)",	_wait_gsc_cc5_te_gpio4,		{ 0, 0,			HPDI32_WAIT_GSC_CC5_TE_GPIO4,	0, 0,		_15_SEC	}	},
	{ "Cable Command 6 (RE/GPIO5)",	_wait_gsc_cc6_re_gpio5,		{ 0, 0,			HPDI32_WAIT_GSC_CC6_RE_GPIO5,	0, 0,		_15_SEC	}	},
	{ "Tx FIFO Empty",				_wait_gsc_tx_fifo_e,		{ 0, 0,			HPDI32_WAIT_GSC_TX_FIFO_EMPTY,	0, 0,		_15_SEC	}	},
	{ "Tx FIFO Almost Empty",		_wait_gsc_tx_fifo_ae,		{ 0, 0,			HPDI32_WAIT_GSC_TX_FIFO_AE,		0, 0,		_15_SEC	}	},
	{ "Tx FIFO Almost Full",		_wait_gsc_tx_fifo_af,		{ 0, 0,			HPDI32_WAIT_GSC_TX_FIFO_AF,		0, 0,		_15_SEC	}	},
	{ "Tx FIFO Full",				_wait_gsc_tx_fifo_f,		{ 0, 0,			HPDI32_WAIT_GSC_TX_FIFO_FULL,	0, 0,		_15_SEC	}	},
	{ "Rx FIFO Empty",				_wait_gsc_rx_fifo_e,		{ 0, 0,			HPDI32_WAIT_GSC_RX_FIFO_EMPTY,	0, 0,		_15_SEC	}	},
	{ "Rx FIFO Almost Empty",		_wait_gsc_rx_fifo_ae,		{ 0, 0,			HPDI32_WAIT_GSC_RX_FIFO_AE,		0, 0,		_15_SEC	}	},
	{ "Rx FIFO Almost Full",		_wait_gsc_rx_fifo_af,		{ 0, 0,			HPDI32_WAIT_GSC_RX_FIFO_AF,		0, 0,		_15_SEC	}	},
	{ "Rx FIFO Full",				_wait_gsc_rx_fifo_f,		{ 0, 0,			HPDI32_WAIT_GSC_RX_FIFO_FULL,	0, 0,		_15_SEC	}	},

	// name							function					flags ...		io											timeout_ms
	{ "Rx Done",					_wait_io_rx_done,			{ 0, 0, 0, 0,	HPDI32_WAIT_IO_RX_DONE,						_15_SEC	}	},
	{ "Rx Error",					_wait_io_rx_error,			{ 0, 0, 0, 0,	HPDI32_WAIT_IO_RX_ERROR,					_15_SEC	}	},
	{ "Rx Timeout",					_wait_io_rx_timeout,		{ 0, 0, 0, 0,	HPDI32_WAIT_IO_RX_TIMEOUT,					_15_SEC	}	},
	{ "Rx Abort",					_wait_io_rx_abort,			{ 0, 0, 0, 0,	HPDI32_WAIT_IO_RX_ABORT,					_15_SEC	}	},
	{ "Tx Done",					_wait_io_tx_done,			{ 0, 0, 0, 0,	HPDI32_WAIT_IO_TX_DONE,						_15_SEC	}	},
	{ "Tx Error",					_wait_io_tx_error,			{ 0, 0, 0, 0,	HPDI32_WAIT_IO_TX_ERROR,					_15_SEC	}	},
	{ "Tx Timeout",					_wait_io_tx_timeout,		{ 0, 0, 0, 0,	HPDI32_WAIT_IO_TX_TIMEOUT,					_15_SEC	}	},
	{ "Tx Abort",					_wait_io_tx_abort,			{ 0, 0, 0, 0,	HPDI32_WAIT_IO_TX_ABORT,				_15_SEC	}	},

	// name							function					flags ...		io											timeout_ms
	{ "Wait Timeout",				_wait_timeout,				{ 0, 0, 0, 0,	HPDI32_WAIT_IO_RX_DONE,						_1_SEC	}	},
	{ "Wait Cancel",				_wait_cancel,				{ 0, 0, 0, 0,	HPDI32_WAIT_IO_RX_DONE,						_30_SEC	}	},

	// terminate list
	{ NULL, NULL, { 1 } }
};



//*****************************************************************************
static int _process_thread(int fd, int index)
{
	char		buf[64];
	int			errs;
	int			i;
	int			ret;
	os_thread_t	thread;
	gsc_wait_t	wait;

	errs	= hpdi32_initialize(fd, -1, 0);

	for (;;)	// A convenience loop.
	{
		os_sem_create_qty(&sem_enter, 1, 0);
		os_sem_create_qty(&sem_exit, 1, 0);
		memset(&thread, 0, sizeof(thread));
		sprintf(buf, "wait event %d", index);
		wait_dst	= wait_src[index].wait;
		errs		+= os_thread_create(&thread, buf, _wait_thread, NULL);

		// Wait for the thread to become waiting.
		os_sem_lock(&sem_enter);

		if (errs == 0)
		{
			wait	= wait_src[index].wait;

			for (i = 0; i < 100; i++)
			{
				ret	= hpdi32_ioctl(wait_fd, HPDI32_IOCTL_WAIT_STATUS, &wait);

				if (ret < 0)
				{
					printf(	"FAIL <---  (%d. status request %d: error %d)\n",
							__LINE__,
							index,
							ret);
					errs++;
					break;
				}

				if (wait.count == 0)
				{
					os_sleep_ms(100);
					continue;
				}

				if (wait.count == 1)
					break;

				errs++;
				printf(	"FAIL <---  (%d. invalid wait count %ld)\n",
						__LINE__,
						(long) wait.count);
				break;
			}
		}

		// Initiate the respective event.
		errs	+= wait_src[index].function(wait_fd);

		if (errs)
			break;		// There was an error.

		// Wait for the resumed thread to run.
		os_sem_lock(&sem_exit);

		for (i = 0;; i++)
		{
			if (wait_dst.flags)
				break;

			if (i >= 100)
			{
				printf(	"FAIL <---  (%d. thread failed to resume)\n",
						__LINE__);
				errs++;
				break;
			}

			os_sleep_ms(100);
		}

		// Verify that the wait ended as expected.

		if (wait_src[index].wait.flags == 0xFFFFFFFF)
			break;		// There was an error.

		if (errs)
			break;		// There was an error.

		if (wait_src[index].wait.timeout_ms == _1_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_TIMEOUT)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_TIMEOUT,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		if (wait_src[index].wait.timeout_ms == _15_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_DONE)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_DONE,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		if (wait_src[index].wait.timeout_ms == _30_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_CANCEL)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_TIMEOUT,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		printf("FAIL <---  (%d. INTERNAL ERROR)\n", __LINE__);
		errs++;
		break;
	}

	// Force termination, just in case of an error.
	wait.flags			= 0;
	wait.main			= 0xFFFFFFFF;
	wait.gsc			= 0xFFFFFFFF;
	wait.alt			= 0xFFFFFFFF;
	wait.io				= 0xFFFFFFFF;
	wait.timeout_ms		= 0;
	wait.count			= 0;
	ret		= hpdi32_ioctl(wait_fd, HPDI32_IOCTL_WAIT_CANCEL, &wait);
	errs	+= ret ? 1 : 0;
	errs	+= os_thread_destroy(&thread);
	errs	+= hpdi32_initialize(fd, -1, 0);

	os_sem_destroy(&sem_enter);
	os_sem_destroy(&sem_exit);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;
	int	i;

	gsc_label_level_inc();

	// Create threads awaiting each of the source crtiteria
	// and verify that it resumes when the criteria is met.

	for (i = 0; wait_src[i].wait.flags == 0; i++)
	{
		gsc_label(wait_src[i].name);
		wait_fd	= fd;
		memset(&wait_dst, 0, sizeof(wait_dst));
		errs	+= _process_thread(fd, i);
	}

	gsc_label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	wait_event_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HPDI32_IOCTL_WAIT_EVENT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int wait_event_test(int fd)
{
	int	errs	= 0;

	gsc_label("HPDI32_IOCTL_WAIT_EVENT");
	errs	+= _service_test();

	if (errs == 0)
		printf("PASS\n");

	errs	+= _function_test(fd);

	return(errs);
}


