// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/signals/tx.c $
// $Rev: 52307 $
// $Date: 2022-12-20 15:43:52 -0600 (Tue, 20 Dec 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _config(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Configuration");
	printf("\n");

	gsc_label_level_inc();

	errs	+= hpdi32_initialize			(args->fd, -1, 1);
	errs	+= hpdi32_cable_cmd_mode_0		(args->fd, -1, 1, HPDI32_CABLE_CMD_MODE_FC, NULL);
	errs	+= hpdi32_cable_cmd_mode_1		(args->fd, -1, 1, HPDI32_CABLE_CMD_MODE_FC, NULL);
	errs	+= hpdi32_cable_cmd_mode_2		(args->fd, -1, 1, HPDI32_CABLE_CMD_MODE_FC, NULL);
	errs	+= hpdi32_cable_cmd_mode_3		(args->fd, -1, 1, HPDI32_CABLE_CMD_MODE_FC, NULL);
	errs	+= hpdi32_cable_cmd_mode_4		(args->fd, -1, 1, HPDI32_CABLE_CMD_MODE_FC, NULL);
	errs	+= hpdi32_cable_cmd_mode_5		(args->fd, -1, 1, HPDI32_CABLE_CMD_MODE_FC, NULL);
	errs	+= hpdi32_cable_cmd_mode_6		(args->fd, -1, 1, HPDI32_CABLE_CMD_MODE_FC, NULL);
	errs	+= hpdi32_rx_enable				(args->fd, -1, 1, HPDI32_RX_ENABLE_NO, NULL);
	errs	+= hpdi32_tristate_te_re		(args->fd, -1, 1, HPDI32_TRISTATE_TE_RE_YES, NULL);
	errs	+= hpdi32_tx_auto_start			(args->fd, -1, 1, HPDI32_AUTO_START_NO, NULL);
	errs	+= hpdi32_tx_clock_divider		(args->fd, -1, 1, args->divider, NULL);
	errs	+= hpdi32_tx_io_data_size		(args->fd, -1, 1, HPDI32_IO_DATA_SIZE_32_BITS, NULL);
	errs	+= hpdi32_tx_io_mode			(args->fd, -1, 1, GSC_IO_MODE_PIO, NULL);
	errs	+= hpdi32_tx_io_overrun			(args->fd, -1, 1, HPDI32_IO_ERROR_IGNORE, NULL);
	errs	+= hpdi32_tx_io_timeout			(args->fd, -1, 1, 1, NULL);
	errs	+= hpdi32_tx_line_val_off_cnt	(args->fd, -1, 1, 10, NULL);
	errs	+= hpdi32_tx_line_val_on_cnt	(args->fd, -1, 1, 10, NULL);
	errs	+= hpdi32_tx_status_val_cnt		(args->fd, -1, 1, 1, NULL);
	errs	+= hpdi32_tx_status_val_mir		(args->fd, -1, 1, HPDI32_TX_STATUS_VAL_MIR_YES, NULL);
	errs	+= hpdi32_tx_remote_throttle	(args->fd, -1, 1, HPDI32_TX_REMOTE_THROTTLE_NO, NULL);

	gsc_label_level_dec();

	return(errs);
}



//*****************************************************************************
static int _tx_disable(const args_t* args)
{
	s32	arg;
	int	errs	= 0;
	int	ret;

	arg		= HPDI32_TX_ENABLE_NO;
	ret		= hpdi32_ioctl(args->fd, HPDI32_IOCTL_TX_ENABLE, &arg);
	errs	+= ret ? 1 : 0;

	return(errs);
}



//*****************************************************************************
static int _tx_enable(const args_t* args)
{
	s32	arg;
	int	errs	= 0;
	int	ret;

	arg		= HPDI32_TX_ENABLE_YES;
	ret		= hpdi32_ioctl(args->fd, HPDI32_IOCTL_TX_ENABLE, &arg);
	errs	+= ret ? 1 : 0;

	arg		= HPDI32_AUTO_START_YES;
	ret		= hpdi32_ioctl(args->fd, HPDI32_IOCTL_TX_AUTO_START, &arg);
	errs	+= ret ? 1 : 0;

	return(errs);
}



//*****************************************************************************
static int _tx_start(const args_t* args)
{
	s32	arg;
	int	errs	= 0;
	int	ret;

	arg		= HPDI32_TX_FLOW_CONTROL_START;
	ret		= hpdi32_ioctl(args->fd, HPDI32_IOCTL_TX_FLOW_CONTROL, &arg);
	errs	+= ret ? 1 : 0;

	return(errs);
}



//*****************************************************************************
static int _tx_stop(const args_t* args)
{
	s32	arg;
	int	errs	= 0;
	int	ret;

	arg		= HPDI32_TX_FLOW_CONTROL_STOP;
	ret		= hpdi32_ioctl(args->fd, HPDI32_IOCTL_TX_FLOW_CONTROL, &arg);
	errs	+= ret ? 1 : 0;

	return(errs);
}



//*****************************************************************************
static int _wait_for_tx_empty(const args_t* args)
{
	s32	arg;
	int	errs	= 0;
	int	ret;

	for (; errs == 0;)
	{
		arg		= -1;
		ret		= hpdi32_ioctl(args->fd, HPDI32_IOCTL_TX_FIFO_STATUS, &arg);
		errs	+= ret ? 1 : 0;

		if ((errs) || (arg == HPDI32_FIFO_STATUS_EMPTY))
			break;

		os_sleep_ms(1);
	}

	return(errs);
}



//*****************************************************************************
static int _write_data(const args_t* args)
{
	#define	SIZE	21

	s32		buf[SIZE];
	int		errs;
	int		i;
	int		ret;

	memset(buf, 0, sizeof(buf));

	for (i = 0; i < SIZE; i++)
	{
		buf[i]	= 'A' + i;
		buf[i]	= buf[i] | (buf[i] << 8);
		buf[i]	= buf[i] | (buf[i] << 16);
	}

	ret		= hpdi32_write(args->fd, buf, sizeof(buf));
	errs	= (ret == sizeof(buf)) ? 0 : 1;

	return(errs);
}



//*****************************************************************************
static int _tx_data(const args_t* args)
{
	int	block;
	int	cycle;
	int	errs	= 0;

	gsc_label("Transmitting Data");
	errs	+= _tx_enable(args);

	for (cycle = 0; (errs == 0) && (cycle < 3000); cycle++)
	{
		for (block = 0; (errs == 0) && (block < 10); block++)
		{
			errs	+= _write_data(args);
			errs	+= _tx_start(args);
			errs	+= _wait_for_tx_empty(args);
			errs	+= _tx_stop(args);
			os_sleep_ms(1);
		}
	}

	errs	+= _tx_disable(args);
	printf("%s\n", errs ? "FAIL <---" : "Done");

	return(errs);
}



//*****************************************************************************
int tx_signals(const args_t* args)
{
	int		errs	= 0;

	gsc_label("Tx Signals");
	printf("\n");

	gsc_label_level_inc();

	errs	+= _config(args);
	errs	+= _tx_data(args);

	gsc_label_level_dec();

	return(errs);
}


