// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/tx_simple/perform.c $
// $Rev: 52317 $
// $Date: 2022-12-20 15:49:18 -0600 (Tue, 20 Dec 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32	buffer[256L * 1024];	// 1MB of data.



/******************************************************************************
*
*	Function:	_config
*
*	Purpose:
*
*		Configure things in preparation for the test.
*
*	Arguments:
*
*		fd		The handle of the device to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _config(int fd)
{
	s32	arg;
	int	errs	= 0;
	int	ret;

	printf("Configuring:\n");

	// Initialize the device.
	printf("  Initialize: ");
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_INITIALIZE, NULL);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Set the I/O Mode to Demand Mode DMA.
	printf("  Tx I/O Mode: DMDMA: ");
	arg	= GSC_IO_MODE_DMDMA;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_IO_MODE, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Tristate these signals in case we are connected to another HPDI32.
	printf("  Tx/Rx Tristate Mode: Enable: ");
	arg	= HPDI32_TRISTATE_TE_RE_YES;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_TRISTATE_TE_RE, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	// Enable the transmitter.
	printf("  Tx Enable: Yes: ");
	arg	= HPDI32_TX_ENABLE_YES;
	ret	= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_ENABLE, &arg);

	if (ret == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---  (error = %d)\n", ret);
		errs++;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_tx_data
*
*	Purpose:
*
*		Transmit the data.
*
*	Arguments:
*
*		fd		The handle of the device to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _tx_data(int fd)
{
	int			errs	= 0;
	const char*	factor	= "";
	int			i;
	int			ret;
	size_t		send	= sizeof(buffer);
	long long	total	= 0;

	printf("Transmitting Data:\n");

	for (i = 0; i < 256; i++)
	{
		if ((i % 64) == 0)
			printf("\n  ");

		printf(".");
		fflush(stdout);
		ret	= hpdi32_write(fd, buffer, send);

		if (ret > 0)
			total	+= ret;

		if (ret < 0)
		{
			printf("FAIL <---  (error = %d)\n", ret);
			errs++;
			break;
		}

		if (ret != send)
		{
			printf(	" FAIL <--- (Requested %ld bytes but wrote only %ld bytes.)",
					(long) send,
					(long) ret);
			errs++;
			break;
		}
	}

	printf("\n\n");

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "K";
	}

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "M";
	}

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "G";
	}

	printf("Wrote %lld%s Bytes\n", total, factor);
	return(errs);
}



//*****************************************************************************
int perform_tests(int fd)
{
	int		errs		= 0;

	memset(buffer, 0, sizeof(buffer));

	errs	+= _config(fd);

	if (errs == 0)
		errs	+= _tx_data(fd);

	return(errs);
}


