// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/txrate/perform.c $
// $Rev: 53395 $
// $Date: 2023-07-12 11:57:23 -0500 (Wed, 12 Jul 2023) $

// HPDI32: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M				(1L * 1024 * 1024)
#define	_2M				(2L * 1024 * 1024)
#define	_8M				(8L * 1024 * 1024)
#define	STATS_LOG_LIMIT	(10 * 1024)



// data types *****************************************************************

typedef struct
{
	os_time_ns_t	t0;
	os_time_ns_t	t1;
	int				results;
} tx_stats_t;



// variables ******************************************************************

static	u8				_buffer[_8M];
static	tx_stats_t		_stats[STATS_LOG_LIMIT];
static	int				_stats_count;
static	int				_stats_lost;
static	os_time_ns_t	_stats_t0;



//*****************************************************************************
static void _tx_stats_append(os_time_ns_t* t0, os_time_ns_t* t1, int results)
{
	tx_stats_t*	tstp;

	if (_stats_count < STATS_LOG_LIMIT)
	{
		tstp			= _stats + _stats_count;
		tstp->t0		= t0[0];
		tstp->t1		= t1[0];
		tstp->results	= results;
		_stats_count++;
	}
	else
	{
		_stats_lost++;
	}
}



//*****************************************************************************
static void _tx_stats_init(void)
{
	memset(_stats, 0, sizeof(_stats));
	os_time_get_ns(&_stats_t0);
	_stats_count	= 0;
	_stats_lost		= 0;
}




//*****************************************************************************
static void _tx_stats_report(void)
{
	long long	ave;
	long long	delta		= 0;
	int			i;
	long long	sum_ns		= 0;
	long long	total_bytes	= 0;
	int			total_calls	= 0;
	long long	total_ns	= 0;

	printf("\n");
	gsc_label("Timing  Statistics");
	printf("(");
	gsc_label_long_comma(_stats_count);
	printf(" entrie%s recorded, ", (_stats_count == 1) ? "" : "s");
	gsc_label_long_comma(_stats_lost);
	printf(" entrie%s lost)\n", (_stats_count == 1) ? "" : "s");

	printf("\n");
	printf("  No.   Time Stamp      Delta           Results\n");
	printf("  ====  ==============  ==============  ===============\n");

	for (i = 0; i < _stats_count; i++)
	{
		printf("  %4d", i);

		delta	= (1000000000LL * _stats[i].t0.tv_sec + _stats[i].t0.tv_nsec)
				- (1000000000LL * _stats_t0.tv_sec + _stats_t0.tv_nsec);
		printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

		delta	= (1000000000LL * _stats[i].t1.tv_sec + _stats[i].t1.tv_nsec)
				- (1000000000LL * _stats[i].t0.tv_sec + _stats[i].t0.tv_nsec);
		printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

		if (_stats[i].results >= 0)
		{
			total_calls++;
			sum_ns		+= delta;
			total_bytes	+= _stats[i].results;
			total_ns	= (1000000000LL * _stats[i].t1.tv_sec + _stats[i].t1.tv_nsec)
						- (1000000000LL * _stats_t0.tv_sec + _stats_t0.tv_nsec);
		}

		printf("  ");

		if (_stats[i].results >= 0)
		{
			gsc_label_long_comma(_stats[i].results);
			printf(" Byte%s\n", (_stats[i].results == 1) ? "" : "s");
		}
		else
		{
			printf("%d\n", _stats[i].results);
		}
	}

	if (_stats_count)
	{
		printf("Total Data:   ");
		gsc_label_long_comma(total_bytes);
		printf(" Bytes\n");

		printf("Total Time:   ");
		printf("%lld.%09lld", total_ns / 1000000000, total_ns % 1000000000);
		printf(" Seconds\n");

		printf("Average Rate: ");
		total_ns	= (total_ns > 0) ? total_ns : 1;
		ave			= 1000000000LL * total_bytes / total_ns;
		gsc_label_long_comma(ave);
		printf(" Bytes/Second\n");

		printf("Total Calls:  ");
		gsc_label_long_comma(total_calls);
		printf("\n");

		printf("Average Call: ");
		total_calls	= (total_calls > 0) ? total_calls : 1;
		ave			= sum_ns / total_calls;
		printf("%lld.%09lld", ave / 1000000000, ave % 1000000000);
		printf(" Seconds/call\n");
	}

	printf("\n");
}




//*****************************************************************************
static void _format_num(char* buf, long long l)
{
	char	num[32];
	int		i;
	char*	src;

	sprintf(num, "%lld", l);

	for (src = num; src[0]; src++)
	{
		if (src == num)
		{
			buf[0]	= src[0];
			buf[1]	= 0;
			buf++;
			continue;
		}

		i	= (int) strlen(src);

		if ((i % 3) == 0)
		{
			buf[0]	= ',';
			buf++;
		}

		buf[0]	= src[0];
		buf[1]	= 0;
		buf++;
	}
}



//*****************************************************************************
static int _show_num_factor(char* buf, unsigned long long ul)
{
	const char*	factor	= "";
	int			space	= 0;

	if ((ul % 1024) == 0)
	{
		space	= 1;
		factor	= "K";
		ul		/= 1024;
	}

	if ((ul % 1024) == 0)
	{
		factor	= "M";
		ul		/= 1024;
	}

	if ((ul % 1024) == 0)
	{
		factor	= "G";
		ul		/= 1024;
	}

	if ((ul % 1024) == 0)
	{
		factor	= "T";
		ul		/= 1024;
	}

	_format_num(buf, ul);

	if (factor[0])
	{
		strcat(buf, " ");
		strcat(buf, factor);
	}

	return(space);
}



//*****************************************************************************
static int _prefill_fifo(const args_t* args)
{
	u32	bsr;
	int	errs	= 0;
	int	i;
	int	j;
	int	ret;
	s32	v;

	v		= HPDI32_TX_ENABLE_NO;
	ret		= hpdi32_ioctl(args->fd, HPDI32_IOCTL_TX_ENABLE, &v);
	errs	+= ret ? 1 : 0;

	for (i = 0; i < 1024; i++)
	{
		for (j = 0; j < 1024; j++)
			errs	+= hpdi32_reg_write(args->fd, -1, 0, HPDI32_GSC_FDR, 0);

		errs	+= hpdi32_reg_read(args->fd, -1, 0, HPDI32_GSC_BSR, &bsr);

		if (bsr & 0x800)
		{
			// The Tx FIFO is not FULL
			continue;
		}
		else
		{
			// The Tx FIFO is FULL.
			break;
		}
	}

	v		= HPDI32_AUTO_START_YES;
	ret		= hpdi32_ioctl(args->fd, HPDI32_IOCTL_TX_AUTO_START, &v);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _tx_data(const args_t* args, void* buf, size_t size, s32 value)
{
	os_time_us_t	begin;
	char			buffer[256];
	int				errs	= 0;
	os_time_us_t	now		= { 0, 0 };
	int				ret;
	int				space	= 0;
	os_time_ns_t	t0;
	os_time_ns_t	t1;
	os_time_us_t	t_limit;
	long long		total	= 0;

	sprintf(buffer, "Tx Data (");

	if (args->scan & SCAN_ALMOST_EMPTY_SIZE)
	{
		strcat(buffer, "AE ");
		_show_num_factor(buffer + strlen(buffer), value);
		strcat(buffer, ")");
		gsc_label(buffer);
		ret		= hpdi32_ioctl(args->fd, HPDI32_IOCTL_TX_FIFO_AE, &value);
		errs	+= ret ? 1 : 0;
	}
	else if (args->scan & SCAN_ALMOST_FULL_SIZE)
	{
		strcat(buffer, "AF ");
		_show_num_factor(buffer + strlen(buffer), value);
		strcat(buffer, ")");
		gsc_label(buffer);
		ret		= hpdi32_ioctl(args->fd, HPDI32_IOCTL_TX_FIFO_AF, &value);
		errs	+= ret ? 1 : 0;
	}
	else
	{
		space	= _show_num_factor(buffer + strlen(buffer), size);
		strcat(buffer, space ? "B)" : " B)");
		gsc_label(buffer);
	}

	fflush(stdout);

	errs	+= _prefill_fifo(args);

	os_time_get_us(&begin);
	t_limit			= begin;
	t_limit.tv_sec	+= args->seconds;

	for (;;)
	{
		if (args->tx_stats)
			os_time_get_ns(&t0);

		ret	= hpdi32_write(args->fd, buf, size);

		if (args->tx_stats)
		{
			os_time_get_ns(&t1);
			_tx_stats_append(&t0, &t1, ret);
		}

		if (ret < 0)
		{
			errs++;
			printf("FAIL <---  (%d. hpdi32_write()", __LINE__);
			printf(": total %lld", total);
			printf(", expected %ld", (long) size);
			printf(", error %d)\n", ret);
			break;
		}

		if (ret != size)
		{
			errs++;
			printf("FAIL <---  (%d. hpdi32_write()", __LINE__);
			printf(": total %lld", total);
			printf(", sent %d", ret);
			printf(", expected %ld)\n", (long) size);
			break;
		}

		total	+= size;
		os_time_get_us(&now);

		if ((now.tv_sec > t_limit.tv_sec) ||
			((now.tv_sec == t_limit.tv_sec) && (now.tv_usec > t_limit.tv_usec)))
		{
			break;
		}
	}

	if (errs == 0)
	{
		char*		dst;
		long		ms;
		long long	rate;
		long		sec;
		long		us;

		us		= (now.tv_sec - begin.tv_sec) * 1000000L
				+ (now.tv_usec + 1000000L - begin.tv_usec)
				- 1000000L;
		sec		= us / 1000000L;
		rate	= (long long) (1000000.0 * total / us);
		ms		= (us - sec * 1000000L + 500) / 1000;

		sprintf(buffer, "PASS  (");
		dst	= buffer + strlen(buffer);

		// Duration
		_format_num(dst, sec);
		strcat(dst, ".");
		dst	+= strlen(dst);
		sprintf(dst, "%03ld", ms);
		strcat(dst, " sec");
		dst	+= strlen(dst);

		// Total bytes transmitted.
		strcat(dst, ", ");
		dst	+= strlen(dst);
		_show_num_factor(dst, total);
		strcat(dst, "B");
		dst	+= strlen(dst);

		// Data transmission rate.
		strcat(dst, ", ");
		dst	+= strlen(dst);
		_show_num_factor(dst, rate);
		strcat(dst, " B/S");
		dst	+= strlen(dst);

		// End data.
		printf("%s)\n", buffer);
	}

	errs	+= hpdi32_tx_fifo_overrun(args->fd, -1, 1, HPDI32_FIFO_ERROR_TEST, NULL);
	return(errs);
}



//*****************************************************************************
static int _tx_data_scan_almost_size(const args_t* args, size_t size)
{
	int		errs	= 0;
	long	l;

	for (l = 8; l <= 0xFFFF; l++)
		errs	+= _tx_data(args, _buffer, size, l);

	return(errs);
}



//*****************************************************************************
static int _tx_data_scan_write_size(const args_t* args)
{
	int		errs	= 0;
	long	l;

	for (l = 1024; l <= _2M; l += 1024)
		errs	+= _tx_data(args, _buffer, l, 0);

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	size_t	size	= _1M;

	errs	+= hpdi32_config_tx			(args->fd, -1, 1);
	errs	+= hpdi32_tx_io_mode		(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= hpdi32_tx_clock_divider	(args->fd, -1, 1, args->clock_div, NULL);
	_tx_stats_init();

	if (args->scan & SCAN_WRITE_SIZE)
		errs	+= _tx_data_scan_write_size(args);
	else if (args->scan)
		errs	+= _tx_data_scan_almost_size(args, size);
	else
		errs	+= _tx_data(args, _buffer, size, 0);

	if (args->tx_stats)
		_tx_stats_report();

	hpdi32_reg_read(args->fd, -1, 1, HPDI32_GSC_BSR, NULL);

	errs	+= hpdi32_tx_fifo_overrun(args->fd, -1, 1, -1, NULL);
	return(errs);
}



