// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/irq_config_high.c $
// $Rev: 43022 $
// $Date: 2018-06-22 17:36:23 -0500 (Fri, 22 Jun 2018) $

// HPDI32: Sample Application: source file

#include "main.h"



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_type_s32_range_t	_arg_1_set[]	=
{
	// min	max		desc
	{ 0,	0xFFFF,	"Configure these for Level High (or Rising Edge) triggering."	},
	{ -1,	-1,		"Query the current setting."									}
};

static const arg_entry_t		_arg_1	=
{
	/* type	*/	ARG_TYPE_S32_RANGE,
	/* qty	*/	SIZEOF_ARRAY(_arg_1_set),
	/* list	*/	(void*) _arg_1_set
};

static const arg_entry_t*		_arg_list[]	=
{
	&_arg_1
};

const command_entry_t			cmd_irq_config_high	=
{
	/* cmd		*/	"irq_config_high",
	/* desc		*/	"Specify interrupts as Level High/Rising Edge triggerred.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	int	errs	= 0;
	int	ret;
	s32	set		= 0;
	s32	tmp		= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 2);	// cmd = arg

		if (errs == 0)
			errs	= config_arg_validate_s32_range(line_data, 1, &cmd_irq_config_high, &set);

		if (errs == 0)
		{
			tmp		= set;
			ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_IRQ_CONFIG_HIGH, &set);
			errs	+= ret ? 1 : 0;
		}

		if (errs == 0)
			config_display_verbose(fd, line_data, &cmd_irq_config_high, tmp);
	}
	else
	{
		// Print the current value.
		set		= -1;
		ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_IRQ_CONFIG_HIGH, &set);
		errs	+= ret ? 1 : 0;

		if (errs == 0)
			printf("0x%04lX", (long) set);
	}

	return(errs);
}


