// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/list.c $
// $Rev: 51587 $
// $Date: 2022-10-05 07:00:11 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	LIST_NOTHING	0
#define	LIST_CURRENT	1
#define	LIST_KEYS		2
#define	LIST_VALUES		3



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_type_str_s32_t	_arg_1_set[]	=
{
	// name			value			desc
	{ "no",			LIST_NOTHING,	"Do nothing."									},
	{ "current",	LIST_CURRENT,	"List all commands with their current values."	},
	{ "keys",		LIST_KEYS,		"List all commands with descriptions."			},
	{ "values",		LIST_VALUES,	"List all values with descriptions."			}
};

static const arg_entry_t		_arg_1	=
{
	/* type	*/	ARG_TYPE_STR_S32,
	/* qty	*/	SIZEOF_ARRAY(_arg_1_set),
	/* list	*/	(void*) _arg_1_set
};

static const arg_entry_t*		_arg_list[]	=
{
	&_arg_1
};

const command_entry_t			cmd_list	=
{
	/* cmd		*/	"list",
	/* desc		*/	"List commands and their values.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	int	errs	= 0;
	s32	set		= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 2);	// cmd = arg

		if (errs == 0)
			errs	= config_arg_validate_str_s32(line_data, 1, &cmd_list, &set);

		if (errs == 0)
			config_display_verbose(fd, line_data, &cmd_list, set);

		if (errs == 0)
		{
			switch (set)
			{
				default:

					errs	= 1;
					printf("INTERNAL ERROR: line %d, file %s\n", __LINE__, __FILE__);
					break;

				case LIST_NOTHING:

					break;

				case LIST_CURRENT:

					errs	= config_list_key_current(fd);
					break;

				case LIST_KEYS:

					errs	= config_list_key_desc(fd);
					break;

				case LIST_VALUES:

					errs	= config_list_key_values(fd);
					break;
			}
		}
	}
	else
	{
		// Print the current value.
		// No current value to print.
	}

	return(errs);
}


