// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/main.h $
// $Rev: 53390 $
// $Date: 2023-07-12 11:18:22 -0500 (Wed, 12 Jul 2023) $

// HPDI32: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "hpdi32_main.h"



// macros *********************************************************************

#define	_1M						(1024L * 1024L)
#define	_32M					(32L * _1M)

#define	READ_DEST_IGNORE		0	// Do nothing with data read.
#define	READ_DEST_FILE			1	// Save data read to a file.

#define	WRITE_SRC_INC			0	// An incrementing pattern.
#define	WRITE_SRC_FILE			1	// Load the data from a file.

#define	SIZEOF_ARRAY(a)			(sizeof((a)) / sizeof((a)[0]))



// data types *****************************************************************

typedef struct
{
	// Application Settings

	char*	cfg_file;
	s32		continuous;		// > 0 = ignore errors, < 0 = stop on errors
	s32		index;			// device index
	s32		minute_limit;	// for continuous testing
	int		qty;			// Number of devices detected.
	s32		test_limit;		// for continuous testing

	int		fd;				// File descriptor for device to access.

	// Device Settings

} args_t;

typedef struct
{
	const char*	line_buf;
	long		line_no;
	int			qty;			// number of parsed strings
	char*		(text[1024]);	// list of parsed strings
} line_data_t;

typedef enum
{
	ARG_TYPE_STR_S32,
	ARG_TYPE_S32_RANGE,
	ARG_TYPE_U32_RANGE
} arg_type_t;

typedef struct
{
	s32			min;
	s32			max;
	const char*	desc;
} arg_type_s32_range_t;

typedef struct
{
	u32			min;
	u32			max;
	const char*	desc;
} arg_type_u32_range_t;

typedef struct
{
	const char*	name;
	s32			value;
	const char*	desc;
} arg_type_str_s32_t;

typedef struct
{
	arg_type_t	type;
	int			qty;
	void*		list;
} arg_entry_t;

typedef struct
{
	const char*			cmd;
	const char*			desc;
	int					(*function)(int fd, line_data_t* line_data);
	int					arg_qty;
	const arg_entry_t**	arg_list;
} command_entry_t;

typedef struct
{
	int				read_dest;
	char			read_dest_file[1024];
	FILE*			read_dest_ptr;
	s32				rx_io_data_size;

	os_time_us_t	tv;
	s32				tx_io_data_size;

	s32				verbose;

	int				write_src;
	char			write_src_file[1024];
} global_t;



// variables ******************************************************************

extern	const command_entry_t	cmd_cable_cmd_mode_0;
extern	const command_entry_t	cmd_cable_cmd_mode_1;
extern	const command_entry_t	cmd_cable_cmd_mode_2;
extern	const command_entry_t	cmd_cable_cmd_mode_3;
extern	const command_entry_t	cmd_cable_cmd_mode_4;
extern	const command_entry_t	cmd_cable_cmd_mode_5;
extern	const command_entry_t	cmd_cable_cmd_mode_6;
extern	const command_entry_t	cmd_cable_cmd_state_0;
extern	const command_entry_t	cmd_cable_cmd_state_1;
extern	const command_entry_t	cmd_cable_cmd_state_2;
extern	const command_entry_t	cmd_cable_cmd_state_3;
extern	const command_entry_t	cmd_cable_cmd_state_4;
extern	const command_entry_t	cmd_cable_cmd_state_5;
extern	const command_entry_t	cmd_cable_cmd_state_6;

extern	const command_entry_t	cmd_echo;

extern	const command_entry_t	cmd_initialize;
extern	const command_entry_t	cmd_irq_config_edge;
extern	const command_entry_t	cmd_irq_config_high;
extern	const command_entry_t	cmd_irq_enable;

extern	const command_entry_t	cmd_list;

extern	const command_entry_t	cmd_read;
extern	const command_entry_t	cmd_read_dest;
extern	const command_entry_t	cmd_read_dest_file;

extern	const command_entry_t	cmd_reg_list;
extern	const command_entry_t	cmd_reg_mod;
extern	const command_entry_t	cmd_reg_read;
extern	const command_entry_t	cmd_reg_show;
extern	const command_entry_t	cmd_reg_write;

extern	const command_entry_t	cmd_rx_auto_start;
extern	const command_entry_t	cmd_rx_enable;
extern	const command_entry_t	cmd_rx_fifo_ae;
extern	const command_entry_t	cmd_rx_fifo_af;
extern	const command_entry_t	cmd_rx_fifo_overrun;
extern	const command_entry_t	cmd_rx_fifo_reset;
extern	const command_entry_t	cmd_rx_fifo_status;
extern	const command_entry_t	cmd_rx_fifo_underrun;
extern	const command_entry_t	cmd_rx_io_data_size;
extern	const command_entry_t	cmd_rx_io_mode;
extern	const command_entry_t	cmd_rx_io_overrun;
extern	const command_entry_t	cmd_rx_io_pio_threshold;
extern	const command_entry_t	cmd_rx_io_timeout;
extern	const command_entry_t	cmd_rx_io_underrun;
extern	const command_entry_t	cmd_rx_line_count;
extern	const command_entry_t	cmd_rx_status_count;

extern	const command_entry_t	cmd_sleep_ms;

extern	const command_entry_t	cmd_time_us;
extern	const command_entry_t	cmd_tristate_te_re;

extern	const command_entry_t	cmd_tx_auto_start;
extern	const command_entry_t	cmd_tx_auto_stop;
extern	const command_entry_t	cmd_tx_clock_divider;
extern	const command_entry_t	cmd_tx_enable;
extern	const command_entry_t	cmd_tx_fifo_ae;
extern	const command_entry_t	cmd_tx_fifo_af;
extern	const command_entry_t	cmd_tx_fifo_overrun;
extern	const command_entry_t	cmd_tx_fifo_reset;
extern	const command_entry_t	cmd_tx_fifo_status;
extern	const command_entry_t	cmd_tx_flow_control;
extern	const command_entry_t	cmd_tx_line_val_off_cnt;
extern	const command_entry_t	cmd_tx_line_val_on_cnt;
extern	const command_entry_t	cmd_tx_status_val_cnt;
extern	const command_entry_t	cmd_tx_io_data_size;
extern	const command_entry_t	cmd_tx_io_mode;
extern	const command_entry_t	cmd_tx_io_overrun;
extern	const command_entry_t	cmd_tx_io_pio_threshold;
extern	const command_entry_t	cmd_tx_io_timeout;
extern	const command_entry_t	cmd_tx_remote_throttle;
extern	const command_entry_t	cmd_tx_status_val_mir;

extern	const command_entry_t	cmd_user_jumpers;

extern	const command_entry_t	cmd_verbose;

extern	const command_entry_t	cmd_write;
extern	const command_entry_t	cmd_write_src;
extern	const command_entry_t	cmd_write_src_file;

extern	const arg_type_str_s32_t	firmware_arg_set[21];
extern	global_t					global;




// prototypes *****************************************************************

int		config_arg_qty_none_verify(const line_data_t* line_data);
int		config_arg_qty_verify(const line_data_t* line_data, int qty);
int		config_arg_validate_s32_range(const line_data_t* line_data, int index, const command_entry_t* cmd, s32* get);
int		config_arg_validate_u32_range(const line_data_t* line_data, int index, const command_entry_t* cmd, u32* get);
int		config_arg_validate_str_s32(const line_data_t* line_data, int index, const command_entry_t* cmd, s32* get);
void	config_display_verbose(int fd, const line_data_t* line_data, const command_entry_t* cmd, s32 set);
int		config_list_key_current(int fd);
int		config_list_key_desc(int fd);
int		config_list_key_values(int fd);
int		config_val_str_s32_print(s32 value, int qty, const arg_type_str_s32_t* list);

FILE*	file_close(const char* name, FILE* file);
FILE*	file_create(const char* name);
int		file_eof(const char* name, FILE* file, int* eof);
FILE*	file_open(const char* name);
int		file_seek(const char* name, FILE* file, long pos);
long	file_size(const char* name, FILE* file);
long	file_read(const char* name, FILE* file, char* buf, size_t size);
long	file_read_line(const char* name, FILE* file, char* buf, size_t size);
long	file_tell(const char* name, FILE* file);

int		perform_tests(const args_t* args);

void	read_dest_file_close(void);
int		read_dest_file_create(void);

char*	skip_ws(char* buf);

void	trunk_ws(char* buf);



#endif
