// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/perform.c $
// $Rev: 52305 $
// $Date: 2022-12-20 15:42:36 -0600 (Tue, 20 Dec 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static const command_entry_t*	_key_list[]	=
{
	&cmd_cable_cmd_mode_0,
	&cmd_cable_cmd_mode_1,
	&cmd_cable_cmd_mode_2,
	&cmd_cable_cmd_mode_3,
	&cmd_cable_cmd_mode_4,
	&cmd_cable_cmd_mode_5,
	&cmd_cable_cmd_mode_6,
	&cmd_cable_cmd_state_0,
	&cmd_cable_cmd_state_1,
	&cmd_cable_cmd_state_2,
	&cmd_cable_cmd_state_3,
	&cmd_cable_cmd_state_4,
	&cmd_cable_cmd_state_5,
	&cmd_cable_cmd_state_6,

	&cmd_echo,

	&cmd_initialize,
	&cmd_irq_config_edge,
	&cmd_irq_config_high,
	&cmd_irq_enable,
	&cmd_list,

	&cmd_read,
	&cmd_read_dest,
	&cmd_read_dest_file,

	&cmd_reg_list,
	&cmd_reg_mod,
	&cmd_reg_read,
	&cmd_reg_show,
	&cmd_reg_write,

	&cmd_rx_auto_start,
	&cmd_rx_enable,
	&cmd_rx_fifo_ae,
	&cmd_rx_fifo_af,
	&cmd_rx_fifo_overrun,
	&cmd_rx_fifo_reset,
	&cmd_rx_fifo_status,
	&cmd_rx_fifo_underrun,
	&cmd_rx_io_data_size,
	&cmd_rx_io_mode,
	&cmd_rx_io_overrun,
	&cmd_rx_io_pio_threshold,
	&cmd_rx_io_timeout,
	&cmd_rx_io_underrun,
	&cmd_rx_line_count,
	&cmd_rx_status_count,

	&cmd_sleep_ms,

	&cmd_time_us,
	&cmd_tristate_te_re,

	&cmd_tx_auto_start,
	&cmd_tx_auto_stop,
	&cmd_tx_clock_divider,
	&cmd_tx_enable,
	&cmd_tx_fifo_ae,
	&cmd_tx_fifo_af,
	&cmd_tx_fifo_overrun,
	&cmd_tx_fifo_reset,
	&cmd_tx_fifo_status,
	&cmd_tx_flow_control,
	&cmd_tx_io_data_size,
	&cmd_tx_io_mode,
	&cmd_tx_io_overrun,
	&cmd_tx_io_pio_threshold,
	&cmd_tx_io_timeout,
	&cmd_tx_line_val_off_cnt,
	&cmd_tx_line_val_on_cnt,
	&cmd_tx_remote_throttle,
	&cmd_tx_status_val_cnt,
	&cmd_tx_status_val_mir,

	&cmd_user_jumpers,
	&cmd_verbose,

	&cmd_write,
	&cmd_write_src,
	&cmd_write_src_file,

	NULL
};



//*****************************************************************************
static int _file_close(const char* name, FILE** file)
{
	int		errs	= 0;

	gsc_label("Closing File");
	file[0]	= file_close(name, file[0]);

	if (file[0] == NULL)
	{
		errs	= 0;
		printf("PASS  (%s)\n", name);
	}
	else
	{
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _file_open(const char* name, FILE** file)
{
	int		errs	= 0;

	gsc_label("Opening File");

	if (name)
	{
		file[0]	= file_open(name);

		if (file[0])
		{
			errs	= 0;
			printf("PASS  (%s)\n", name);
		}
		else
		{
			errs	= 1;
		}
	}
	else
	{
		errs	= 1;
		printf("FAIL <---  (No file specified.)\n");
	}

	return(errs);
}



//*****************************************************************************
static void _parse_line(char* buf, line_data_t* line_data)
{
	int		i;
	char*	psz;

	for (;;)	// A convenience loop.
	{
		memset(line_data, 0, sizeof(line_data_t));

		// Remove any training comments.
		psz	= strchr(buf, ';');

		if (psz)
			psz[0]	= 0;

		// Remove any surrounding white space.
		buf	= skip_ws(buf);
		trunk_ws(buf);

		if (buf[0] == 0)
			break;

		// Separate the key from the value(s).
		line_data->qty		= 1;
		line_data->text[0]	= buf;
		psz	= strchr(buf, '=');

		if (psz)
		{
			line_data->qty		= 2;
			line_data->text[1]	= psz + 1;
			psz[0]	= 0;
			psz++;
		}
		else
		{
			break;
		}

		// Parse any comma separated componants.

		for (buf = psz;;)
		{
			psz	= strchr(buf, ',');

			if (psz)
			{
				line_data->text[line_data->qty]	= psz + 1;
				line_data->qty++;
				psz[0]	= 0;
				buf		= psz + 1;
			}
			else
			{
				break;
			}
		}

		break;
	}

	// Trim surrounding white space from all componants.
	for (i = 0; i < line_data->qty; i++)
	{
		line_data->text[i]	= skip_ws(line_data->text[i]);
		trunk_ws(line_data->text[i]);
	}
}



//*****************************************************************************
static const command_entry_t* _command_locate(const char* text)
{
	const command_entry_t*	cmd	= NULL;
	int						i;

	for (i = 0; (cmd == NULL) && (_key_list[i]); i++)
	{
		if (strcmp(text, _key_list[i]->cmd) == 0)
		{
			cmd	= _key_list[i];
			break;
		}
	}

	return(cmd);
}



//*****************************************************************************
static int _process_line(int fd, const char* line_buf, long line_no)
{
	char					buf[1024];
	const command_entry_t*	cmd;
	int						errs		= 0;
	line_data_t				line_data;

	for (;;)		// A convenience loop.
	{
		strncpy(buf, line_buf, sizeof(buf));
		buf[sizeof(buf) - 1]	= 0;
		_parse_line(buf, &line_data);

		if (line_data.qty <= 0)
			break;

		cmd	= _command_locate(line_data.text[0]);

		if (cmd == NULL)
		{
			errs++;
			printf(	"ERROR: unrecognized command key on line %ld: '%s'\n",
					line_no,
					line_buf);
			break;
		}

		line_data.line_buf	= line_buf;
		line_data.line_no	= line_no;

		if (cmd->function)
		{
			errs	+= (cmd->function)(fd, &line_data);
		}
		else
		{
			errs++;
			printf(	"ERROR: command key on line %ld not implemented: '%s'\n",
					line_no,
					line_buf);
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
static int _process_file(int fd, const char* name, FILE* file)
{
	int		eof				= 0;
	int		errs			= 0;
	char	line_buf[1024];
	long	line_len;
	long	line_no			= 0;
	int		status;

	for (;;)
	{
		line_no++;
		line_len	= file_read_line(name, file, line_buf, sizeof(line_buf));

		if (line_len == 0)
		{
			status	= file_eof(name, file, &eof);

			if (status < 0)
			{
				errs++;
				break;
			}

			if (eof)
				break;
		}

		errs	+= _process_line(fd, line_buf, line_no);
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int			errs	= 0;
	FILE*		file;
	const char*	src		= args->cfg_file;

	if (src == NULL)
		src	= "(FIle name not specified.)";

	gsc_label("Reading Configuration File");
	printf("(%s)\n", src);
	gsc_label_level_inc();

	errs	+= _file_open(src, &file);

	if (errs == 0)
	{
		// Initialize settings.
		global.read_dest		= READ_DEST_IGNORE;
		global.rx_io_data_size	= 32;
		global.verbose			= 1;
		global.tx_io_data_size	= 32;
		global.write_src		= WRITE_SRC_INC;

		os_time_get_us(&global.tv);
		strcpy(global.read_dest_file, "read_dest.bin");
		strcpy(global.write_src_file, "write_src.bin");

		// Process the configuration file.
		errs	+= _process_file(args->fd, src, file);
		errs	+= _file_close(src, &file);

		gsc_label_level_dec();
		read_dest_file_close();
		gsc_label_level_inc();
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int config_arg_qty_none_verify(const line_data_t* line_data)
{
	int	errs	= 0;

	if (line_data->qty <= 1)				// cmd
		errs	= 0;
	else if (line_data->qty >= 3)			// cmd = xxx,...
		errs	= 1;
	else if (line_data->text[1] == NULL)	// cmd =
		errs	= 0;
	else if (line_data->text[1][0])			// cmd = xxx
		errs	= 1;
	else									// cmd =
		errs	= 0;

	if (errs)
	{
		printf(	"ERROR: too many arguments on line %ld: '%s'\n",
				line_data->line_no,
				line_data->line_buf);
	}

	return(errs);
}



//*****************************************************************************
int config_arg_qty_verify(const line_data_t* line_data, int qty)
{
	int	errs	= 0;
	int	i;

	for (;;)	// A convenience loop.				Examples with qty = 2.
	{
		if (line_data->qty < qty)					// cmd
		{
			errs	= 1;
			printf(	"ERROR: too few arguments on line %ld: '%s'\n",
					line_data->line_no,
					line_data->line_buf);
			break;
		}

		if (line_data->qty > qty)					// cmd = xxx,...
		{
			errs	= 1;
			printf(	"ERROR: too many arguments on line %ld: '%s'\n",
					line_data->line_no,
					line_data->line_buf);
			break;
		}

		for (i = 1; i < qty; i++)
		{
			if (line_data->text[i] == NULL)			// cmd =
				errs	= 1;
			else if (line_data->text[i][0] == 0)	// cmd =
				errs	= 1;

			if (errs)
			{
				printf(	"ERROR: argument %d missing on line %ld: '%s'\n",
						i,
						line_data->line_no,
						line_data->line_buf);
				break;
			}
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
int config_arg_validate_s32_range(
	const line_data_t*		line_data,
	int						index,
	const command_entry_t*	cmd,
	s32*					get)
{
	const arg_type_s32_range_t*	arg;
	const arg_entry_t*			arg_entry;
	const arg_entry_t**			arg_list;
	char						c;
	int							errs		= 0;
	int							i;
	int							j;
	long						l			= 0;
	const char*					psz			= line_data->text[index];

	arg_list	= cmd->arg_list;
	arg_entry	= arg_list[index - 1];
	arg			= (void*) arg_entry->list;

	if ((psz[0] == '0') && (psz[1] == 'x'))
	{
		if (psz[2])
		{
			j	= sscanf(psz + 2, "%lX%c", &l, &c);

			if (j == 1)
				get[0]	= l;
			else
				errs++;
		}
		else
		{
			errs++;
		}
	}
	else
	{
		j	= sscanf(psz, "%ld%c", &l, &c);

		if (j == 1)
			get[0]	= l;
		else
			errs++;
	}

	if (errs == 0)
	{
		errs	= 1;

		for (i = 0; i < arg_entry->qty; i++)
		{
			if ((l >= arg[i].min) && (l <= arg[i].max))
			{
				errs	= 0;
				break;
			}
		}
	}

	if (errs)
	{
		printf("ERROR: argument %d is invalid on line %ld: '%s'\n",
				index,
				(long) line_data->line_no,
				line_data->line_buf);
	}

	return(errs);
}



//*****************************************************************************
int config_arg_validate_u32_range(
	const line_data_t*		line_data,
	int						index,
	const command_entry_t*	cmd,
	u32*					get)
{
	const arg_type_s32_range_t*	arg;
	const arg_entry_t*			arg_entry;
	const arg_entry_t**			arg_list;
	char						c;
	int							errs		= 0;
	int							i;
	int							j;
	const char*					psz			= line_data->text[index];
	unsigned long				ul			= 0;

	arg_list	= cmd->arg_list;
	arg_entry	= arg_list[index - 1];
	arg			= (void*) arg_entry->list;

	if ((psz[0] == '0') && (psz[1] == 'x'))
	{
		if (psz[2])
		{
			j	= sscanf(psz + 2, "%lX%c", &ul, &c);

			if ((j == 1) && (ul >= 0) && (ul <= 0xFFFFFFFFUL))
				get[0]	= (u32) ul;
			else
				errs++;
		}
		else
		{
			errs++;
		}
	}
	else
	{
		j	= sscanf(psz, "%lu%c", &ul, &c);

		if ((j == 1) && (ul >= 0) && (ul <= 0xFFFFFFFFUL))
			get[0]	= (u32) ul;
		else
			errs++;
	}

	if (errs == 0)
	{
		errs	= 1;

		for (i = 0; i < arg_entry->qty; i++)
		{
			if ((ul >= (unsigned long) arg[i].min) &&
				(ul <= (unsigned long) arg[i].max))
			{
				errs	= 0;
				break;
			}
		}
	}

	if (errs)
	{
		printf("ERROR: argument %d is invalid on line %ld: '%s'\n",
				index,
				(long) line_data->line_no,
				line_data->line_buf);
	}

	return(errs);
}



//*****************************************************************************
int config_arg_validate_str_s32(const line_data_t* line_data, int index, const command_entry_t* cmd, s32* get)
{
	const arg_type_str_s32_t*	arg;
	const arg_entry_t*			arg_entry;
	const arg_entry_t**			arg_list;
	int							errs		= 0;
	int							i;

	arg_list	= cmd->arg_list;
	arg_entry	= arg_list[index - 1];
	arg			= (void*) arg_entry->list;

	for (i = 0; i < arg_entry->qty; i++)
	{
		if (strcmp(line_data->text[index], arg[i].name) == 0)
		{
			get[0]	= arg[i].value;
			break;
		}
	}

	if (i >= arg_entry->qty)
	{
		errs++;
		printf(	"ERROR: argument %d is invalid on line %ld: '%s'\n",
				index,
				line_data->line_no,
				line_data->line_buf);
	}

	return(errs);
}



//*****************************************************************************
void config_display_verbose(
	int						fd,
	const line_data_t*		line_data,
	const command_entry_t*	cmd,
	s32						set)
{
	int			i;
	const char*	space	= "";

	if ((set == -1) || (global.verbose))
	{
		gsc_label_suffix(line_data->text[0], " ");
		printf("= ");

		if (cmd->arg_qty <= 0)
		{
			if (set == -1)
				cmd->function(fd, NULL);
		}
		else if (cmd->arg_qty >= 1)
		{
			for (i = 1; i < line_data->qty; i++)
			{
				printf("%s%s", space, line_data->text[i]);
				space	= " , ";
			}

			if (set == -1)
			{
				printf("   ; ");
				cmd->function(fd, NULL);
			}
		}

		printf("\n");
	}
}



//*****************************************************************************
int config_list_key_current(int fd)
{
	int	errs	= 0;
	int	i;

	gsc_label_level_inc();

	for (i = 0; _key_list[i]; i++)
	{
		gsc_label_suffix(_key_list[i]->cmd, " ");

		printf("= ");

		if (_key_list[i]->function)
			errs	+= (_key_list[i]->function)(fd, NULL);
		else
			printf("  (not implemented)");

		printf("\n");
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int config_list_key_desc(int fd)
{
	int	i;

	gsc_label_level_inc();

	for (i = 0; _key_list[i]; i++)
	{
		gsc_label(_key_list[i]->cmd);

		if (_key_list[i]->desc)
			printf("%s", _key_list[i]->desc);

		if (_key_list[i]->function == NULL)
			printf("  (not implemented)");

		printf("\n");
	}

	gsc_label_level_dec();
	return(0);
}



//*****************************************************************************
int config_list_key_values(int fd)
{
	int						arg_no;
	int						arg_qty;
	const arg_entry_t*		arg_entry;
	const arg_entry_t**		arg_list;
	char					buf[64];
	const command_entry_t*	cmd;
	int						cmd_no;
	int						errs		= 0;
	int						i;
	int						val_qty;

	gsc_label_level_inc();

	for (cmd_no = 0; _key_list[cmd_no]; cmd_no++)
	{
		cmd	= _key_list[cmd_no];
		gsc_label(cmd->cmd);

		if (cmd->desc)
			printf("%s", cmd->desc);

		if (cmd->function == NULL)
			printf("  (not implemented)");

		printf("\n");

		// List the value options.
		gsc_label_level_inc();
		arg_qty		= cmd->arg_qty;
		arg_list	= cmd->arg_list;

		for (arg_no = 0; arg_no < arg_qty; arg_no++)
		{
			sprintf(buf, "Argument %d", arg_no + 1);
			gsc_label(buf);
			printf("\n");
			arg_entry	= arg_list[arg_no];
			val_qty		= arg_entry->qty;

			gsc_label_level_inc();

			switch (arg_entry->type)
			{
				default:
				case ARG_TYPE_U32_RANGE:

					printf(	"INTERNAL ERROR: current type not recognized: %ld"
							"  (line %d, file '%s)\n",
							(long) arg_entry->type,
							__LINE__,
							__FILE__);
					break;

				case ARG_TYPE_S32_RANGE:

					{
						const arg_type_s32_range_t* val_list	= (void*) arg_entry->list;

						for (i = 0; i < val_qty; i++)
						{
							if (val_list[i].min == val_list[i].max)
							{
								sprintf(buf, "%ld", (long) val_list[i].min);
							}
							else if (val_list[i].max < 4096)
							{
								sprintf(buf,
										"%ld - %ld",
										(long) val_list[i].min,
										(long) val_list[i].max);
							}
							else
							{
								sprintf(buf,
										"0x%lX - 0x%lX",
										(long) val_list[i].min,
										(long) val_list[i].max);
							}

							gsc_label(buf);
							printf("%s\n", val_list[i].desc);
						}

						break;
					}

				case ARG_TYPE_STR_S32:

					{
						const arg_type_str_s32_t* val_list	= (void*) arg_entry->list;

						for (i = 0; i < val_qty; i++)
						{
							gsc_label(val_list[i].name);
							printf("%s\n", val_list[i].desc);
						}

						break;
					}
			}

			gsc_label_level_dec();
		}

		gsc_label_level_dec();
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int config_val_str_s32_print(s32 value, int qty, const arg_type_str_s32_t* list)
{
	int	errs	= 1;
	int	i;

	for (i = 0; i < qty; i++)
	{
		if (value == list[i].value)
		{
			errs	= 0;
			printf("%s  (%s)", list[i].name, list[i].desc);
			break;
		}
	}

	if (errs)
	{
		printf(	"INTERNAL ERROR: current value not recognized: %ld"
				"  (line %d, file '%s)\n",
				(long) value,
				__LINE__,
				__FILE__);
	}

	return(errs);
}


//*****************************************************************************
char* skip_ws(char* buf)
{
	for (;;)
	{
		if (buf[0] == 0)
			break;

		if (isspace(buf[0]))
			buf++;
		else
			break;
	}

	return(buf);
}



//*****************************************************************************
void trunk_ws(char* buf)
{
	char*	end	= buf + strlen(buf) - 1;

	for (;;)
	{
		if (buf[0] == 0)
			break;

		if (isspace(end[0]))
		{
			end[0]	= 0;
			end--;
		}
		else
		{
			break;
		}
	}
}



