// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/read_dest.c $
// $Rev: 51587 $
// $Date: 2022-10-05 07:00:11 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_type_str_s32_t	_arg_1_set[]	=
{
	// name		value				desc
	{ "ignore",	READ_DEST_IGNORE,	"Ignore the data read from the device."			},
	{ "file",	READ_DEST_FILE,		"Save the data read from a device to a file."	},
	{ "-1",		-1,					"Query the current setting."					}
};

static const arg_entry_t		_arg_1	=
{
	/* type	*/	ARG_TYPE_STR_S32,
	/* qty	*/	SIZEOF_ARRAY(_arg_1_set),
	/* list	*/	(void*) _arg_1_set
};

static const arg_entry_t*		_arg_list[]	=
{
	&_arg_1
};

const command_entry_t			cmd_read_dest	=
{
	/* cmd		*/	"read_dest",
	/* desc		*/	"Decide what to do with the data read from the device.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	int	errs	= 0;
	s32	last	= global.read_dest;
	s32	set		= 0;
	s32	tmp		= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 2);	// cmd = arg

		if (errs == 0)
			errs	= config_arg_validate_str_s32(line_data, 1, &cmd_read_dest, &set);

		if (errs == 0)
		{
			tmp	= set;

			if (set != -1)
				global.read_dest	= set;
		}

		if (errs == 0)
			config_display_verbose(fd, line_data, &cmd_read_dest, tmp);

		if (errs)
		{
			// Do nothing.
		}
		else if (global.read_dest == last)
		{
			// Change nothing.
		}
		else if (global.read_dest == READ_DEST_FILE)
		{
			read_dest_file_create();
		}
		else
		{
			read_dest_file_close();
		}
	}
	else
	{
		// Print the current value.
		errs	= config_val_str_s32_print(
					global.read_dest,
					SIZEOF_ARRAY(_arg_1_set),
					_arg_1_set);
	}

	return(errs);
}


