// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/read_dest_file.c $
// $Rev: 43022 $
// $Date: 2018-06-22 17:36:23 -0500 (Fri, 22 Jun 2018) $

// HPDI32: Sample Application: source file

#include "main.h"



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_type_str_s32_t	_arg_1_set[]	=
{
	// name		value	desc
	{ "name",	0,		"Save the read data to this file."	},
	{ "-1",		-1,		"Query the current setting."		}
};

static const arg_entry_t		_arg_1	=
{
	/* type	*/	ARG_TYPE_STR_S32,
	/* qty	*/	SIZEOF_ARRAY(_arg_1_set),
	/* list	*/	(void*) _arg_1_set
};

static const arg_entry_t*		_arg_list[]	=
{
	&_arg_1
};

const command_entry_t			cmd_read_dest_file	=
{
	/* cmd		*/	"read_dest_file",
	/* desc		*/	"When saved, read data is written to this file.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	int	errs	= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 2);	// cmd = arg

		if (errs)
		{
			// Do nothing.
		}
		else if (strcmp(line_data->text[1], "-1") == 0)
		{
			config_display_verbose(fd, line_data, &cmd_read_dest_file, -1);
		}
		else if (global.read_dest != READ_DEST_FILE)
		{
			config_display_verbose(fd, line_data, &cmd_read_dest_file, 0);
			strncpy(global.read_dest_file, line_data->text[1], sizeof(global.read_dest_file));
			global.read_dest_file[sizeof(global.read_dest_file) - 1]	= 0;
		}
		else if (strcmp(line_data->text[1], global.read_dest_file) == 0)
		{
			config_display_verbose(fd, line_data, &cmd_read_dest_file, 0);
		}
		else
		{
			config_display_verbose(fd, line_data, &cmd_read_dest_file, 0);
			read_dest_file_close();
			strncpy(global.read_dest_file, line_data->text[1], sizeof(global.read_dest_file));
			global.read_dest_file[sizeof(global.read_dest_file) - 1]	= 0;
			read_dest_file_create();
		}
	}
	else
	{
		// Print the current value.
		printf("%s", global.read_dest_file);
	}

	return(errs);
}



//*****************************************************************************
void read_dest_file_close(void)
{
	if (global.read_dest_ptr)
	{
		gsc_label_level_inc();
		gsc_label("Closing File");
		file_close(global.read_dest_file, global.read_dest_ptr);
		printf("PASS  (%s)\n", global.read_dest_file);
		global.read_dest_ptr	= NULL;
		gsc_label_level_dec();
	}
}



//*****************************************************************************
int read_dest_file_create(void)
{
	int	errs	= 0;

	if (global.read_dest == READ_DEST_FILE)
	{
		gsc_label_level_inc();
		gsc_label("Creating File");

		global.read_dest_ptr	= file_create(global.read_dest_file);

		if (global.read_dest_ptr)
			printf("PASS  (%s)\n", global.read_dest_file);

		gsc_label_level_dec();
	}

	return(errs);
}


