// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/reg_list.c $
// $Rev: 51587 $
// $Date: 2022-10-05 07:00:11 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	LIST_BASIC		1
#define	LIST_DETAIL		2
#define	LIST_PCI		3
#define	LIST_PLX		4



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_type_str_s32_t	_arg_1_set[]	=
{
	// name		value			desc
	{ "basic",	LIST_BASIC,		"Give a basic listing of the firmware registers and their values."		},
	{ "detail",	LIST_DETAIL,	"List a detailed listing of the firmware registers and their values."	},
	{ "pci",	LIST_PCI,		"Give a basic listing of the PCI registers and their values."			},
	{ "plx",	LIST_PLX,		"Give a basic listing of the PLX registers and their values."			},
};

static const arg_entry_t		_arg_1	=
{
	/* type	*/	ARG_TYPE_STR_S32,
	/* qty	*/	SIZEOF_ARRAY(_arg_1_set),
	/* list	*/	(void*) _arg_1_set
};

static const arg_entry_t*		_arg_list[]	=
{
	&_arg_1
};

const command_entry_t			cmd_reg_list	=
{
	/* cmd		*/	"reg_list",
	/* desc		*/	"List registers and their content.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	u32	didr;
	int	errs	= 0;
	s32	set		= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 2);	// cmd = arg

		if (errs == 0)
			errs	= config_arg_validate_str_s32(line_data, 1, &cmd_reg_list, &set);

		if (errs == 0)
		{
			config_display_verbose(fd, line_data, &cmd_reg_list, set);

			switch (set)
			{
				default:

					errs	= 1;
					printf("INTERNAL ERROR: line %d, file %s\n", __LINE__, __FILE__);
					break;

				case LIST_BASIC:

					printf("\n");
					errs	= hpdi32_reg_list(fd, 0);
					printf("\n");
					break;

				case LIST_DETAIL:

					printf("\n");
					errs	= hpdi32_reg_list(fd, 1);
					printf("\n");
					break;

				case LIST_PCI:

					hpdi32_reg_read(fd, -1, 0, GSC_PCI_9080_DIDR, &didr);
					printf("\n");

					if (didr == 0x9080)
						gsc_reg_plx9080_list_pci(fd, hpdi32_reg_read);
					else
						gsc_reg_plx9656_list_pci(fd, hpdi32_reg_read);

					printf("\n");
					break;

				case LIST_PLX:

					hpdi32_reg_read(fd, -1, 0, GSC_PCI_9080_DIDR, &didr);
					printf("\n");

					if (didr == 0x9080)
						gsc_reg_plx9080_list_plx(fd, hpdi32_reg_read);
					else
						gsc_reg_plx9656_list_plx(fd, hpdi32_reg_read);

					printf("\n");
					break;
			}
		}
	}
	else
	{
		// Print the current value.
		// There is no current value.
	}

	return(errs);
}


