// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/reg_mod.c $
// $Rev: 43022 $
// $Date: 2018-06-22 17:36:23 -0500 (Fri, 22 Jun 2018) $

// HPDI32: Sample Application: source file

#include "main.h"



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_entry_t			_arg_1	=
{
	/* type	*/	ARG_TYPE_STR_S32,
	/* qty	*/	SIZEOF_ARRAY(firmware_arg_set),
	/* list	*/	(void*) firmware_arg_set
};

static const arg_type_u32_range_t	_arg_2_set[]	=
{
	// min	max			desc
	{ 0,	0xFFFFFFFF,	"Apply this value to the register."	},
};

static const arg_entry_t			_arg_2	=
{
	/* type	*/	ARG_TYPE_U32_RANGE,
	/* qty	*/	SIZEOF_ARRAY(_arg_2_set),
	/* list	*/	(void*) _arg_2_set
};

static const arg_type_u32_range_t	_arg_3_set[]	=
{
	// min	max			desc
	{ 0,	0xFFFFFFFF,	"Apply this mask for the operation."	},
};

static const arg_entry_t			_arg_3	=
{
	/* type	*/	ARG_TYPE_U32_RANGE,
	/* qty	*/	SIZEOF_ARRAY(_arg_3_set),
	/* list	*/	(void*) _arg_3_set
};

static const arg_entry_t*			_arg_list[]	=
{
	&_arg_1,
	&_arg_2,
	&_arg_3
};

const command_entry_t				cmd_reg_mod	=
{
	/* cmd		*/	"reg_mod",
	/* desc		*/	"Perform read-modify-write on a firmware register.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	gsc_reg_t	arg;
	int			errs	= 0;
	u32			mask	= 0;
	s32			reg		= 0;
	int			ret;
	u32			val		= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 4);	// cmd = reg, val, mask

		if (errs == 0)
			errs	= config_arg_validate_str_s32(line_data, 1, &cmd_reg_mod, &reg);

		if (errs == 0)
			errs	= config_arg_validate_u32_range(line_data, 2, &cmd_reg_mod, &val);

		if (errs == 0)
			errs	= config_arg_validate_u32_range(line_data, 3, &cmd_reg_mod, &mask);

		if (errs == 0)
		{
			arg.reg		= reg;
			arg.value	= val;
			arg.mask	= mask;
			ret			= hpdi32_ioctl(fd, HPDI32_IOCTL_REG_MOD, &arg);
			errs		+= ret ? 1 : 0;
		}

		if (errs == 0)
			config_display_verbose(fd, line_data, &cmd_reg_mod, 0);
	}
	else
	{
		// Print the current value.
		// We don't print current register contents.
	}

	return(errs);
}


