// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/reg_read.c $
// $Rev: 46631 $
// $Date: 2019-12-17 14:39:14 -0600 (Tue, 17 Dec 2019) $

// HPDI32: Sample Application: source file

#include "main.h"



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_entry_t			_arg_1	=
{
	/* type	*/	ARG_TYPE_STR_S32,
	/* qty	*/	SIZEOF_ARRAY(firmware_arg_set),
	/* list	*/	(void*) firmware_arg_set
};

static const arg_type_u32_range_t	_arg_2_set[]	=
{
	// min	max			desc
	{ 1,	0x20000,	"Read from the register this number of times."	},
};

static const arg_entry_t			_arg_2	=
{
	/* type	*/	ARG_TYPE_U32_RANGE,
	/* qty	*/	SIZEOF_ARRAY(_arg_2_set),
	/* list	*/	(void*) _arg_2_set
};

static const arg_entry_t*			_arg_list[]	=
{
	&_arg_1,
	&_arg_2
};

const command_entry_t				cmd_reg_read	=
{
	/* cmd		*/	"reg_read",
	/* desc		*/	"Read from a register a number of times.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	gsc_reg_t	arg;
	int			errs	= 0;
	long		l;
	u32			qty		= 0;
	s32			reg		= 0;
	int			ret;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 3);	// cmd = reg, qty

		if (errs == 0)
			errs	= config_arg_validate_str_s32(line_data, 1, &cmd_reg_read, &reg);

		if (errs == 0)
			errs	= config_arg_validate_u32_range(line_data, 2, &cmd_reg_read, &qty);

		if (errs == 0)
		{
			arg.reg		= reg;
			arg.value	= 0xDEADBEEF;
			arg.mask	= 0;

			for (l = 0; (errs == 0) && (l < (long) qty); l++)
			{
				ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_REG_READ, &arg);
				errs	+= ret ? 1 : 0;
			}
		}

		if (errs == 0)
			config_display_verbose(fd, line_data, &cmd_reg_read, 0);
	}
	else
	{
		// Print the current value.
		// We don't print current register contents.
	}

	return(errs);
}


