// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/tx_io_data_size.c $
// $Rev: 43022 $
// $Date: 2018-06-22 17:36:23 -0500 (Fri, 22 Jun 2018) $

// HPDI32: Sample Application: source file

#include "main.h"



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_type_str_s32_t	_arg_1_set[]	=
{
	// name	value							desc
	{ "32",	HPDI32_IO_DATA_SIZE_32_BITS,	"The data is 32-bits wide (cable signals D0-D31)."	},
	{ "16",	HPDI32_IO_DATA_SIZE_16_BITS,	"The data is 16-bits wide (cable signals D0-D15)."	},
	{ "8",	HPDI32_IO_DATA_SIZE_8_BITS,		"The data is 8-bits wide (cable signals D0-D7)."	},
	{ "-1",	-1,								"Query the current setting."						}
};

static const arg_entry_t		_arg_1	=
{
	/* type	*/	ARG_TYPE_STR_S32,
	/* qty	*/	SIZEOF_ARRAY(_arg_1_set),
	/* list	*/	(void*) _arg_1_set
};

static const arg_entry_t*		_arg_list[]	=
{
	&_arg_1
};

const command_entry_t			cmd_tx_io_data_size	=
{
	/* cmd		*/	"tx_io_data_size",
	/* desc		*/	"Specify the Tx data size at the cable interface.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	int	errs	= 0;
	int	ret;
	s32	set		= 0;
	s32	tmp		= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 2);	// cmd = arg

		if (errs == 0)
			errs	= config_arg_validate_str_s32(line_data, 1, &cmd_tx_io_data_size, &set);

		if (errs == 0)
		{
			tmp		= set;
			ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_IO_DATA_SIZE, &set);
			errs	+= ret ? 1 : 0;
		}

		if ((errs == 0) && (set > 0))
			global.tx_io_data_size	= set;

		if (errs == 0)
			config_display_verbose(fd, line_data, &cmd_tx_io_data_size, tmp);
	}
	else
	{
		// Print the current value.
		set		= -1;
		ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_IO_DATA_SIZE, &set);
		errs	+= ret ? 1 : 0;

		if (errs == 0)
		{
			errs	= config_val_str_s32_print(
						set,
						SIZEOF_ARRAY(_arg_1_set),
						_arg_1_set);
		}
	}

	return(errs);
}


