// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/tx_io_overrun.c $
// $Rev: 43022 $
// $Date: 2018-06-22 17:36:23 -0500 (Fri, 22 Jun 2018) $

// HPDI32: Sample Application: source file

#include "main.h"



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_type_str_s32_t	_arg_1_set[]	=
{
	// name		value					desc
	{ "check",	HPDI32_IO_ERROR_CHECK,	"Check for Tx Overruns with each write request."		},
	{ "ignore",	HPDI32_IO_ERROR_IGNORE,	"Do not check for Tx Overruns with each write request."	},
	{ "-1",		-1,						"Query the current setting."							}
};

static const arg_entry_t		_arg_1	=
{
	/* type	*/	ARG_TYPE_STR_S32,
	/* qty	*/	SIZEOF_ARRAY(_arg_1_set),
	/* list	*/	(void*) _arg_1_set
};

static const arg_entry_t*		_arg_list[]	=
{
	&_arg_1
};

const command_entry_t			cmd_tx_io_overrun	=
{
	/* cmd		*/	"tx_io_overrun",
	/* desc		*/	"Control how write requests response to Tx Overruns.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	int	errs	= 0;
	int	ret;
	s32	set		= 0;
	s32	tmp		= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 2);	// cmd = arg

		if (errs == 0)
			errs	= config_arg_validate_str_s32(line_data, 1, &cmd_tx_io_overrun, &set);

		if (errs == 0)
		{
			tmp		= set;
			ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_IO_OVERRUN, &set);
			errs	+= ret ? 1 : 0;
		}

		if (errs == 0)
			config_display_verbose(fd, line_data, &cmd_tx_io_overrun, tmp);
	}
	else
	{
		// Print the current value.
		set		= -1;
		ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TX_IO_OVERRUN, &set);
		errs	+= ret ? 1 : 0;

		if (errs == 0)
		{
			errs	= config_val_str_s32_print(
						set,
						SIZEOF_ARRAY(_arg_1_set),
						_arg_1_set);
		}
	}

	return(errs);
}


