// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/write.c $
// $Rev: 46888 $
// $Date: 2020-02-24 15:10:26 -0600 (Mon, 24 Feb 2020) $

// HPDI32: Sample Application: source file

#include "main.h"



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_type_s32_range_t	_arg_1_set[]	=
{
	// min	max		desc
	{ 1,	_32M,	"Write blocks containing this many samples."	},
};

static const arg_entry_t		_arg_1	=
{
	/* type	*/	ARG_TYPE_S32_RANGE,
	/* qty	*/	SIZEOF_ARRAY(_arg_1_set),
	/* list	*/	(void*) _arg_1_set
};

static const arg_type_s32_range_t	_arg_2_set[]	=
{
	// min	max		desc
	{ 1,	_32M,	"Write this number of blocks."	},
};

static const arg_entry_t		_arg_2	=
{
	/* type	*/	ARG_TYPE_S32_RANGE,
	/* qty	*/	SIZEOF_ARRAY(_arg_2_set),
	/* list	*/	(void*) _arg_2_set
};

static const arg_entry_t*		_arg_list[]	=
{
	&_arg_1,
	&_arg_2
};

const command_entry_t			cmd_write	=
{
	/* cmd		*/	"write",
	/* desc		*/	"Write data to the device.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static void _show_data_set(s32 samples, s32 blocks)
{
	long long	qty;

	gsc_label("Data Set");
	qty	= (long long) samples * blocks;
	gsc_label_long_comma(qty);
	printf(" samples, ");
	qty	*=  (global.tx_io_data_size / 8);
	gsc_label_long_comma(qty);
	printf(" bytes\n");
}



//*****************************************************************************
static int _src_file_access(const char* name, FILE** file, long* bytes)
{
	int	errs	= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label("Accessing Source File");
		file[0]	= file_open(name);

		if (file[0] == NULL)
		{
			errs++;
			break;
		}

		bytes[0]	= file_size(name, file[0]);

		if (bytes[0] < 0)
		{
			errs++;
			break;
		}

		if (bytes[0] == 0)
		{
			errs++;
			printf("ERROR: source file is empty: '%s'\n", name);
			break;
		}

		printf("PASS  (");
		gsc_label_long_comma(bytes[0]);
		printf(" bytes, %s)\n", name);
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _src_file_close(const char* name, FILE** file)
{
	int		errs	= 0;

	gsc_label("Closing File");
	file[0]	= file_close(name, file[0]);

	if (file[0] == NULL)
	{
		errs	= 0;
		printf("PASS  (%s)\n", name);
	}
	else
	{
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _src_file_read(const char* name, FILE* file, void* ptr, long bytes)
{
	int		errs	= 0;
	long	l;

	for (;;)	// A convenience loop.
	{
		gsc_label("Reading Source File");
		l	= file_read(name, file, ptr, bytes);

		if (l < 0)
			break;

		if (l != bytes)
		{
			errs++;
			printf(	"ERROR: read error: expected %ld bytes, got %ld\n",
					bytes,
					l);
			break;
		}

		printf("PASS  (");
		gsc_label_long_comma(bytes);
		printf(" bytes, %s)\n", name);
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _buffer_init(void* ptr, long bytes)
{
	int		errs	= 0;
	long	l;
	long	m;
	u8*		_8;
	u16*	_16;
	u32*	_32;

	gsc_label("Initializing Buffer");
	memset(ptr, 0, bytes);

	if (bytes == 0)
	{
		errs++;
		printf("ERROR: source image is too small.\n");
	}
	else
	{
		switch (global.tx_io_data_size)
		{
			default:

				break;

			case 8:

				l	= bytes;
				_8	= ptr;

				for (m = 0; m < l; m++)
					_8[m]	= (u8) (m & 0xFF);

				break;

			case 16:

				l	= bytes / 2;
				_16	= ptr;

				for (m = 0; m < l; m++)
					_16[m]	= (u16) (m & 0xFFFF);

				break;

			case 32:

				l	= bytes / 4;
				_32	= ptr;

				for (m = 0; m < l; m++)
					_32[m]	= (u32) m;

				break;
		}

		printf("Done  (");
		gsc_label_long_comma(bytes);
		printf(" bytes, %ld-bit samples)\n", (long) global.tx_io_data_size);
	}

	return(errs);
}



//*****************************************************************************
static void* _buffer_alloc(long bytes)
{
	void*	ptr;

	gsc_label("Allocate Buffer");
	ptr			= malloc(bytes);
	printf("%s  (", ptr ? "PASS" : "FAIL");
	gsc_label_long_comma(bytes);
	printf(" bytes)\n");
	return(ptr);
}



//*****************************************************************************
static void* _buffer_free(void* ptr)
{
	gsc_label("Free Buffer");
	free(ptr);
	printf("Done\n");
	return(NULL);
}



//*****************************************************************************
static int _tx_block(int fd, void* src, long bytes)
{
	int		errs	= 0;
	time_t	now;
	long	ret;
	time_t	start;

	start	= time(NULL);

	for (;;)	// A convenience loop.
	{
		ret	= hpdi32_write(fd, src, bytes);

		if (ret < 0)
		{
			errs++;
			printf("ERROR: hpdi32_write(): error = %ld\n", ret);
			break;
		}

		if (ret == 0)
		{
			now	= time(NULL);

			if (now >= (start + 2))
			{
				errs++;
				printf("ERROR: hpdi32_write() is sending no data\n");
				break;
			}

			os_sleep_ms(1);
			continue;
		}

		bytes	-= ret;

		if (bytes <= 0)
			break;

		start	= time(NULL);
	}

	return(errs);
}



//*****************************************************************************
static int _write_data(int errs, int fd, long samples, long blocks, long bytes, void* ptr)
{
	long			blk;
	long			blk_full;	// The number of full blocks to transfer.
	long			blk_part;	// The size of the partial block, if at all.
	long			blk_qty;
	long			blk_size;	// The size of a block of data.
	double			rate;
	double			secs;
	void*			src;
	os_time_us_t	t1;
	os_time_us_t	t2;
	long long		total	= 0;

	gsc_label("Writing Data");

	if (errs)
	{
		errs	= 0;
		printf("SKIPPED  (Aborted due to errors.)\n");
	}
	else
	{
		os_time_get_us(&t1);
		errs		= 0;
		blk_qty		= 0;
		blk_size	= samples * (global.tx_io_data_size / 8);
		blk_full	= bytes / blk_size;
		blk_part	= bytes - blk_full * blk_size;

		for (;;)	// Continue sending blocks until we finish.
		{
			src	= ptr;

			for (blk = 0; blk < blk_full; blk++)
			{
				errs	= _tx_block(fd, src, blk_size);

				if (errs)
					break;

				blk_qty++;
				total	+= blk_size;

				if (blk_qty >= blocks)
					break;

				src	= (char*) src + blk_size;
			}

			if ((errs) || (blk_qty >= blocks))
				break;

			if (blk_part)
			{
				errs	= _tx_block(fd, src, blk_part);

				if (errs)
					break;

				blk_qty++;
				total	+= blk_part;

				if (blk_qty >= blocks)
					break;
			}
		}

		if (errs == 0)
		{
			os_time_get_us(&t2);
			printf("Done\n");

			gsc_label("Performance");
			secs	= (float) (t2.tv_sec - t1.tv_sec)
					+ (float) (0.000001 * t2.tv_usec)
					- (float) (0.000001 * t1.tv_usec);
			rate	= (long double) total / (global.tx_io_data_size / 8) / secs;
			gsc_label_long_comma(blk_qty);
			printf(" blocks, ");
			gsc_label_long_comma(total / (global.tx_io_data_size / 8));
			printf(	" samples, ");
			gsc_label_float_comma((double) secs, 5, 3);
			printf(	" seconds, rate ");
			gsc_label_float_comma((double) rate, 5, 3);
			printf(	" S/S\n");
		}
	}

	return(errs);
}



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	long	blk_size;
	s32		blocks	= 0;
	long	bytes;
	int		errs	= 0;
	FILE*	file;
	void*	ptr		= NULL;
	s32		samples	= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 3);	// cmd = qty, times

		if (errs == 0)
			errs	= config_arg_validate_s32_range(line_data, 1, &cmd_write, &samples);

		if (errs == 0)
			errs	= config_arg_validate_s32_range(line_data, 2, &cmd_write, &blocks);

		if (errs == 0)
			config_display_verbose(fd, line_data, &cmd_write, 0);

		if (errs == 0)
		{
			gsc_label_level_inc();
			blk_size	= samples * (global.tx_io_data_size / 8);

			if (global.write_src == WRITE_SRC_FILE)
			{
				errs	= _src_file_access(global.write_src_file, &file, &bytes);

				if (errs == 0)
				{
					ptr		= _buffer_alloc(bytes);

					if (ptr == NULL)
						errs++;
				}

				if (errs == 0)
					errs	+= _src_file_read(global.write_src_file, file, ptr, bytes);

				if (errs == 0)
					errs	+= _src_file_close(global.write_src_file, &file);

				samples	= bytes / (global.tx_io_data_size / 8);
				bytes	= samples * (global.tx_io_data_size / 8);

				if (errs)
				{
				}
				else if (blk_size > bytes)
				{
					_show_data_set(samples, blocks);
					printf("         NOTE: The block size is larger than the file size.\n");
					printf("         NOTE: Block sizes will be reduced to the size of the file.\n");
				}
				else if (blk_size % bytes)
				{
					_show_data_set(samples, blocks);
					printf("         NOTE: The file size is not a multiple of the block size.\n");
					printf("         NOTE: Some block sizes will be smaller than expected.\n");
				}
				else
				{
					_show_data_set(samples, blocks);
				}
			}
			else
			{
				_show_data_set(samples, blocks);
				bytes	= samples * (global.tx_io_data_size / 8);
				ptr		= _buffer_alloc(bytes);

				if (ptr)
					_buffer_init(ptr, bytes);
				else
					errs++;
			}

			errs	+= _write_data(errs, fd, samples, blocks, bytes, ptr);

			if (ptr)
				_buffer_free(ptr);

			gsc_label_level_dec();
		}
	}
	else
	{
		// Print the current value.
		// There isn't anything to print.
	}

	return(errs);
}


