// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/samples/xfer/write_src.c $
// $Rev: 51587 $
// $Date: 2022-10-05 07:00:11 -0500 (Wed, 05 Oct 2022) $

// HPDI32: Sample Application: source file

#include "main.h"



// prototypes *****************************************************************

static int _callback(int fd, line_data_t* line_data);



// variables ******************************************************************

static const arg_type_str_s32_t	_arg_1_set[]	=
{
	// name		value			desc
	{ "inc",	WRITE_SRC_INC,	"The data written is an incrementing 32-bit pattern."	},
	{ "file",	WRITE_SRC_FILE,	"Write the data from an existing file."					},
	{ "-1",		-1,				"Query the current setting."							}
};

static const arg_entry_t		_arg_1	=
{
	/* type	*/	ARG_TYPE_STR_S32,
	/* qty	*/	SIZEOF_ARRAY(_arg_1_set),
	/* list	*/	(void*) _arg_1_set
};

static const arg_entry_t*		_arg_list[]	=
{
	&_arg_1
};

const command_entry_t			cmd_write_src	=
{
	/* cmd		*/	"write_src",
	/* desc		*/	"Decide what data to write to the device.",
	/* function	*/	_callback,
	/* arg_qty	*/	SIZEOF_ARRAY(_arg_list),
	/* arg_list	*/	_arg_list
};



//*****************************************************************************
static int _callback(int fd, line_data_t* line_data)
{
	int	errs	= 0;
	s32	set		= 0;
	s32	tmp		= 0;

	if (line_data)
	{
		errs	= config_arg_qty_verify(line_data, 2);	// cmd = arg

		if (errs == 0)
			errs	= config_arg_validate_str_s32(line_data, 1, &cmd_write_src, &set);

		if (errs == 0)
		{
			tmp	= set;

			if (set != -1)
				global.write_src	= set;
		}

		if (errs == 0)
			config_display_verbose(fd, line_data, &cmd_write_src, tmp);

		// We do nothing with the source selection at this point.
		// That is handled at the time of the "write" operation.
	}
	else
	{
		// Print the current value.
		errs	= config_val_str_s32_print(
					global.write_src,
					SIZEOF_ARRAY(_arg_1_set),
					_arg_1_set);
	}

	return(errs);
}


