// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/utils/util_query.c $
// $Rev: 53382 $
// $Date: 2023-07-12 11:12:16 -0500 (Wed, 12 Jul 2023) $

// HPDI32: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hpdi32_query
*
*	Purpose:
*
*		Provide a visual wrapper for the HPDI32_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hpdi32_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case HPDI32_QUERY_BSR_D18_XCVR:		ptr	= "Xcvr w/ BSR D18";	break;
		case HPDI32_QUERY_BUS_WIDTH:		ptr	= "Bus Width";			break;
		case HPDI32_QUERY_CLOCK_MAX:		ptr	= "Max Clock rate";		break;
		case HPDI32_QUERY_COUNT:			ptr	= "Query Options";		break;
		case HPDI32_QUERY_DEVICE_TYPE:		ptr	= "Device Type";		break;
		case HPDI32_QUERY_DMDMA_1:			ptr	= "Channel 1 DMDMA";	break;
		case HPDI32_QUERY_FIFO_RESET_FAST:	ptr	= "FIFO Fast Reset";	break;
		case HPDI32_QUERY_FIFO_SIZE_RX:		ptr	= "Rx FIFO Size";		break;
		case HPDI32_QUERY_FIFO_SIZE_TX:		ptr	= "Tx FIFO Size";		break;
		case HPDI32_QUERY_FORM_FACTOR:		ptr	= "Form Factor";		break;
		case HPDI32_QUERY_GPIO_0_5:			ptr	= "GPIO 0-5";			break;
		case HPDI32_QUERY_GPIO_6:			ptr	= "GPIO 6";				break;
		case HPDI32_QUERY_GPIO_D32:			ptr	= "D32 GPIO";			break;
		case HPDI32_QUERY_MODEL:			ptr	= "Model";				break;
		case HPDI32_QUERY_OVER_UNDER_RUN:	ptr	= "Over/Under Run";		break;
		case HPDI32_QUERY_REG_ACR:			ptr	= "Reg: ACR";			break;
		case HPDI32_QUERY_REG_FSR:			ptr	= "Reg: FSR";			break;
		case HPDI32_QUERY_REG_FWR_UPDATE:	ptr	= "Reg: FWR updated";	break;
		case HPDI32_QUERY_REG_GIR:			ptr	= "Reg: GIR";			break;
		case HPDI32_QUERY_REG_GOR:			ptr	= "Reg: GOR";			break;
		case HPDI32_QUERY_REG_IELR:			ptr	= "Reg: IELR";			break;
		case HPDI32_QUERY_REG_IHLR:			ptr	= "Reg: IHLR";			break;
		case HPDI32_QUERY_REG_RFSR:			ptr	= "Reg: RFSR";			break;
		case HPDI32_QUERY_REG_RFWR:			ptr	= "Reg: RFWR";			break;
		case HPDI32_QUERY_REG_TCDR:			ptr	= "Reg: TCDR";			break;
		case HPDI32_QUERY_REG_TFSR:			ptr	= "Reg: TFSR";			break;
		case HPDI32_QUERY_REG_TFWR:			ptr	= "Reg: TFWR";			break;
		case HPDI32_QUERY_SINGLE_CYC_DIS:	ptr	= "Single Cycle Dis";	break;
		case HPDI32_QUERY_TRISTATE_TE_RE:	ptr	= "Tristate Tx/Rx Ena";	break;
		case HPDI32_QUERY_TX_AUTO_STOP:		ptr	= "Tx Auto Stop";		break;
		case HPDI32_QUERY_TX_CLOCK_DIV_MAX:	ptr	= "Tx Clk Div Max";		break;
		case HPDI32_QUERY_TX_CLOCK_DIV_MIN:	ptr	= "Tx Clk Div Min";		break;
		case HPDI32_QUERY_USER_JUMPERS:		ptr	= "User Jumpers";		break;
		case HPDI32_QUERY_XCVR_TYPE:		ptr	= "Xcvr Type";			break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;
	ptr		= buf;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case HPDI32_QUERY_COUNT:
		case HPDI32_QUERY_TX_CLOCK_DIV_MAX:
		case HPDI32_QUERY_TX_CLOCK_DIV_MIN:
		case HPDI32_QUERY_USER_JUMPERS:

			gsc_label_long_comma_buf(set, buf);
			break;

		case HPDI32_QUERY_BUS_WIDTH:

			sprintf(buf, "%ld bits", (long) set);
			break;

		case HPDI32_QUERY_CLOCK_MAX:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case HPDI32_QUERY_BSR_D18_XCVR:
		case HPDI32_QUERY_DMDMA_1:
		case HPDI32_QUERY_FIFO_RESET_FAST:
		case HPDI32_QUERY_GPIO_0_5:
		case HPDI32_QUERY_GPIO_6:
		case HPDI32_QUERY_GPIO_D32:
		case HPDI32_QUERY_OVER_UNDER_RUN:
		case HPDI32_QUERY_REG_ACR:
		case HPDI32_QUERY_REG_FSR:
		case HPDI32_QUERY_REG_FWR_UPDATE:
		case HPDI32_QUERY_REG_GIR:
		case HPDI32_QUERY_REG_GOR:
		case HPDI32_QUERY_REG_IELR:
		case HPDI32_QUERY_REG_IHLR:
		case HPDI32_QUERY_REG_RFSR:
		case HPDI32_QUERY_REG_RFWR:
		case HPDI32_QUERY_REG_TCDR:
		case HPDI32_QUERY_REG_TFSR:
		case HPDI32_QUERY_REG_TFWR:
		case HPDI32_QUERY_SINGLE_CYC_DIS:
		case HPDI32_QUERY_TRISTATE_TE_RE:
		case HPDI32_QUERY_TX_AUTO_STOP:

			sprintf(buf, "%s", set ? "Supported" : "Not Supported");
			break;

		case HPDI32_QUERY_FIFO_SIZE_RX:
		case HPDI32_QUERY_FIFO_SIZE_TX:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Words");

			if ((set) && ((set % 1024) == 0))
			{
				strcat(buf, " (");
				gsc_label_long_comma_buf(set / 1024, buf + strlen(buf));
				strcat(buf, "K)");
			}

			break;

		case HPDI32_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_HPDI32)
			{
				strcpy(buf, "HPDI32");
			}
			else
			{
				errs	= 1;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case HPDI32_QUERY_FORM_FACTOR:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case HPDI32_QUERY_FF_UNKNOWN:	ptr	= "Not Known";	break;
				case HPDI32_QUERY_FF_PCI:		ptr	= "PCI";		break;
				case HPDI32_QUERY_FF_PMC:		ptr	= "PMC";		break;
				case HPDI32_QUERY_FF_CPCI:		ptr	= "cPCI";		break;
				case HPDI32_QUERY_FF_PC104P:	ptr	= "PX/104+";	break;
			}

			break;

		case HPDI32_QUERY_MODEL:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case HPDI32_QUERY_MODEL_HPDI32:		ptr	= "HPDI32";		break;
				case HPDI32_QUERY_MODEL_HPDI32A:	ptr	= "HPDI32A";		break;
				case HPDI32_QUERY_MODEL_HPDI32AL:	ptr	= "HPDI32AL";		break;
				case HPDI32_QUERY_MODEL_HPDI32ALT:	ptr	= "HPDI32ALT";		break;
				case HPDI32_QUERY_MODEL_HPDI32B:	ptr	= "HPDI32B";		break;
			}

			break;

		case HPDI32_QUERY_XCVR_TYPE:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case HPDI32_QUERY_XCVR_UNKNOWN:	ptr	= "Not Known";	break;
				case HPDI32_QUERY_XCVR_PECL:	ptr	= "PECL";		break;
				case HPDI32_QUERY_XCVR_RS485:	ptr	= "RS-485";		break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", ptr);
		else
			printf("%s\n", ptr);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


