// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/utils/util_reg.c $
// $Rev: 53382 $
// $Date: 2023-07-12 11:12:16 -0500 (Wed, 12 Jul 2023) $

// HPDI32: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)		"GSC " #a, HPDI32_GSC_##a, 0, 0
#define	WIDTH			24

#define	SHOW_FIELD(begin,end,eol,opts,desc)	\
	gsc_reg_field_show(	\
		width + 10,		\
		WIDTH,			\
		value,			\
		(begin),		\
		(end),			\
		(eol),			\
		(opts),			\
		(desc))



//*****************************************************************************
static int _frr_detail_new(int fd, int supported, u32 value, int width)
{
	static const char*	form[]	=
	{
		"RESERVED",
		"PCI",
		"PMC",
		"cPCI",
		"PC104+",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	fsr[]		= { "Absent",	"Present"	};
	static const char*	fw_size[]	= { "32-bit",	"64-bit"	};
	static const char*	layout[]	= { "Legacy",	"Updated"	};

	long	v;

	SHOW_FIELD(31, 31, 1, fsr,		"Features Register"	);
	SHOW_FIELD(30, 30, 1, layout,	"Spec Compliant"	);
	SHOW_FIELD(29, 29, 1, NULL,		"Reserved"			);
	SHOW_FIELD(28, 28, 1, fw_size,	"Firmware Size"		);
	SHOW_FIELD(27, 24, 1, form,		"Form Factor"		);
	SHOW_FIELD(23, 16, 0, NULL,		"PCB Revision"		);
	v	= (value & 0xFF0000) >> 16;

	if (v == 0)
		printf("No Revision\n");
	else
		printf("Rev. %c\n", (char) ('A' + v - 1));

	SHOW_FIELD(15,  8, 1, NULL,		"Sub ID"			);
	SHOW_FIELD( 7,  0, 0, NULL,		"Firmware Revision"	);
	v	= value & 0xFF;
	printf("%02lX\n", v);
	return(0);
}



//*****************************************************************************
static int _frr_detail_old(int fd, int supported, u32 value, int width, u32 didr)
{
	static const char*	fsr[]		= { "Absent",	"Present"	};
	static const char*	form[]		= { "PCI",		"PMC"		};

	long	v;

	SHOW_FIELD(31, 31, 1, fsr,	"Features Register"	);
	SHOW_FIELD(30, 25, 1, NULL,	"Reserved"			);
	SHOW_FIELD(24, 24, 1, form,	"Form Factor"		);
	SHOW_FIELD(23, 16, 0, NULL,	"PCB Revision"		);
	v	= (value & 0xFF0000) >> 16;

	if (v)
		printf("Rev %c\n", (char) ('A' + v - 1));
	else
		printf("N/R\n");

	SHOW_FIELD(15,  8, 1, NULL,	"Sub ID"			);
	SHOW_FIELD( 7,  0, 0, NULL,	"Firmware Revision"	);
	v	= value & 0xFF;
	printf("%02lX\n", v);
	return(0);
}



//*****************************************************************************
static int _frr_detail(int fd, int supported, u32 value, int width)
{
	u32	didr;
	int	ret;

	hpdi32_reg_read(fd, -1, 0, GSC_PCI_9080_DIDR, &didr);

	if (value & 0x40000000)
		ret	= _frr_detail_new(fd, supported, value, width);
	else
		ret	= _frr_detail_old(fd, supported, value, width, didr);

	return(ret);
}



//*****************************************************************************
static int _bcr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	dir[]		= { "Rx",				"Tx"				};
	static const char*	disabled[]	= { "Disabled",			"Enabled"			};
	static const char*	enabled[]	= { "Enabled",			"Disabled"			};
	static const char*	fc[]		= { "Flow Control",		"GPIO Input"		};
	static const char*	fc_out[]	= { "Flow Control",		"GPIO Output"		};
	static const char*	gpio_out[]	= { "GPIO Output Low",	"GPIO Output High"	};
	static const char*	idle[]		= { "Idle",				"Active"			};

	s32	dmdma;
	int	errs	= 0;
	s32	gpio_0_5;
	s32	gpio_6;
	s32	scd;
	s32	stop;

	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_DMDMA_1, &dmdma);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_GPIO_0_5, &gpio_0_5);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_GPIO_6, &gpio_6);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_SINGLE_CYC_DIS, &scd);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_TX_AUTO_STOP, &stop);

	SHOW_FIELD(31, 31, 1, disabled,				"Test Mode"				);

	if (gpio_0_5)
	{
		if (value & 0x400000)
			SHOW_FIELD(30, 30, 1, gpio_out,		"Cable Command D6"		);
		else
			SHOW_FIELD(30, 30, 1, fc,			"Cable Command D6"		);

		if (value & 0x200000)
			SHOW_FIELD(29, 29, 1, gpio_out,		"Cable Command D5"		);
		else
			SHOW_FIELD(29, 29, 1, fc,			"Cable Command D5"		);

		if (value & 0x100000)
			SHOW_FIELD(28, 28, 1, gpio_out,		"Cable Command D4"		);
		else
			SHOW_FIELD(28, 28, 1, fc,			"Cable Command D4"		);

		if (value & 0x80000)
			SHOW_FIELD(27, 27, 1, gpio_out,		"Cable Command D3"		);
		else
			SHOW_FIELD(27, 27, 1, fc,			"Cable Command D3"		);

		if (value & 0x40000)
			SHOW_FIELD(26, 26, 1, gpio_out,		"Cable Command D2"		);
		else
			SHOW_FIELD(26, 26, 1, fc,			"Cable Command D2"		);

		if (value & 0x20000)
			SHOW_FIELD(25, 25, 1, gpio_out,		"Cable Command D1"		);
		else
			SHOW_FIELD(25, 25, 1, fc,			"Cable Command D1"		);
	}
	else
	{
			SHOW_FIELD(30, 25, 1, NULL,			"Reserved"				);
	}

	if (gpio_6)
	{
		if (value & 0x10000)
			SHOW_FIELD(24, 24, 1, gpio_out,		"Cable Command D0"		);
		else
			SHOW_FIELD(24, 24, 1, fc,			"Cable Command D0"		);
	}
	else
	{
			SHOW_FIELD(24, 24, 1, NULL,			"Reserved"				);
	}

	SHOW_FIELD(			23, 23, 1, NULL,		"Reserved"				);

	if (gpio_0_5)
	{
		SHOW_FIELD(		22, 22, 1, fc_out,		"Cable Command D6"		);
		SHOW_FIELD(		21, 21, 1, fc_out,		"Cable Command D5"		);
		SHOW_FIELD(		20, 20, 1, fc_out,		"Cable Command D4"		);
		SHOW_FIELD(		19, 19, 1, fc_out,		"Cable Command D3"		);
		SHOW_FIELD(		18, 18, 1, fc_out,		"Cable Command D2"		);
		SHOW_FIELD(		17, 17, 1, fc_out,		"Cable Command D1"		);
	}
	else
	{
		SHOW_FIELD(		22, 17, 1, NULL,		"Reserved"				);
	}

	if (gpio_6)
	{
		SHOW_FIELD(		16, 16, 1, fc_out,		"Cable Command D0"		);
	}
	else
	{
		SHOW_FIELD(		16, 16, 1, NULL,		"Reserved"				);
	}

	if (scd)
	{
		SHOW_FIELD(		15, 15, 1, enabled,		"Single Cycles"			);
	}
	else
	{
		SHOW_FIELD(		15, 15, 1, NULL,		"Reserved"				);
	}

	SHOW_FIELD(			14, 10, 1, NULL,		"Reserved"				);
	SHOW_FIELD(			 9,  9, 1, disabled,	"Cable Throttle"		);
	SHOW_FIELD(			 8,  8, 1, disabled,	"Tx Start"				);
	SHOW_FIELD(			 7,  7, 1, disabled,	"LV Hi on SV Hi"		);

	if (stop)
	{
		SHOW_FIELD(		 6,  6, 1, enabled,		"Tx Start Auto Clear"	);
	}
	else if (dmdma)
	{
		SHOW_FIELD(		 6,  6, 1, dir,			"DMDMA Direction"		);
	}
	else
	{
		SHOW_FIELD(		 6,  6, 1, NULL,		"Reserved"				);
	}

	SHOW_FIELD(			 5,  5, 1, disabled,	"Rx Enable"				);
	SHOW_FIELD(			 4,  4, 1, disabled,	"Tx Enable"				);
	SHOW_FIELD(			 3,  3, 1, NULL,		"Reserved"				);
	SHOW_FIELD(			 2,  2, 1, idle,		"Tx FIFO Reset"			);
	SHOW_FIELD(			 1,  1, 1, idle,		"Rx FIFO Reset"			);
	SHOW_FIELD(			 0,  0, 1, idle,		"Board Reset"			);

	return(errs);
}



//*****************************************************************************
static int _bsr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	idle[]	= { "Idle",	"Active"	};
	static const char*	low[]	= { "Low",	"High"		};
	static const char*	no[]	= { "No",	"Yes"		};
	static const char*	off[]	= { "Off",	"On"		};
	static const char*	yes[]	= { "Yes",	"No"		};

	int	errs	= 0;
	s32	jumpers;
	s32	run_err;

	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_OVER_UNDER_RUN, &run_err);
	errs	+= hpdi32_query(fd, -1, 0, HPDI32_QUERY_USER_JUMPERS, &jumpers);

	SHOW_FIELD(		31, 24, 1, NULL,	"Reserved"				);

	if (run_err)
	{
		SHOW_FIELD(	23, 23, 1, no,		"Rx Overrun"			);
		SHOW_FIELD(	22, 22, 1, no,		"Rx Underrun"			);
		SHOW_FIELD(	21, 21, 1, no,		"Tx Overrun"			);
	}
	else
	{
		SHOW_FIELD(	23, 21, 1, NULL,	"Reserved"				);
	}

	SHOW_FIELD(		20, 18, 1, NULL,	"Reserved"				);

	if (jumpers)
	{
		SHOW_FIELD(	17, 17, 1, off,		"Board Jumper 1"		);
		SHOW_FIELD(	16, 16, 1, off,		"Board Jumper 0"		);
	}
	else
	{
		SHOW_FIELD(	17, 16, 1, NULL,	"Reserved"				);
	}

	SHOW_FIELD(		15, 15, 1, yes,		"Rx FIFO Full"			);
	SHOW_FIELD(		14, 14, 1, yes,		"Rx FIFO Almost Full"	);
	SHOW_FIELD(		13, 13, 1, yes,		"Rx FIFO Almost Empty"	);
	SHOW_FIELD(		12, 12, 1, yes,		"Rx FIFO Empty"			);
	SHOW_FIELD(		11, 11, 1, yes,		"Tx FIFO Full"			);
	SHOW_FIELD(		10, 10, 1, yes,		"Tx FIFO Almost Full"	);
	SHOW_FIELD(		 9,  9, 1, yes,		"Tx FIFO Almost Empty"	);
	SHOW_FIELD(		 8,  8, 1, yes,		"Tx FIFO Empty"			);
	SHOW_FIELD(		 7,  7, 1, idle,	"Tx Start"				);
	SHOW_FIELD(		 6,  6, 1, low,		"Cable Command D6"		);
	SHOW_FIELD(		 5,  5, 1, low,		"Cable Command D5"		);
	SHOW_FIELD(		 4,  4, 1, low,		"Cable Command D4"		);
	SHOW_FIELD(		 3,  3, 1, low,		"Cable Command D3"		);
	SHOW_FIELD(		 2,  2, 1, low,		"Cable Command D2"		);
	SHOW_FIELD(		 1,  1, 1, low,		"Cable Command D1"		);
	SHOW_FIELD(		 0,  0, 1, low,		"Cable Command D0"		);

	return(errs);
}



//*****************************************************************************
static int _xar_detail(int fd, int supported, u32 value, int width)
{
	u32	v;

	SHOW_FIELD(31, 16, 0, NULL,	"Almost Full"	);
	v	= (value & 0xFFFF0000) >> 16;
	printf("%ld  (0x%lX)\n", (long) v, (long) v);

	SHOW_FIELD(15,  0, 0, NULL,	"Almost Empty"	);
	v	= value & 0xFFFF;
	printf("%ld  (0x%lX)\n", (long) v, (long) v);

	return(0);
}



//*****************************************************************************
static int _fsr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	not[]	= { "Not Supported",	"Supported"		};

	s32	get;
	int	i;
	s32	model;
	int	ret;

	if (supported)
	{
		i	= hpdi32_query(fd, -1, 0, HPDI32_QUERY_REG_FSR, &get);

		if (i)
			ret	= -1;
		else
			ret	= get ? 1 : 0;
	}
	else
	{
		ret	= 0;

		hpdi32_query(fd, -1, 0, HPDI32_QUERY_MODEL, &model);

		switch (model)
		{
			default:
			case HPDI32_QUERY_MODEL_HPDI32:

				SHOW_FIELD(	31,  6, 1, NULL,	"Reserved"					);
				break;

			case HPDI32_QUERY_MODEL_HPDI32A:
			case HPDI32_QUERY_MODEL_HPDI32AL:
			case HPDI32_QUERY_MODEL_HPDI32B:

				SHOW_FIELD(	31,  9, 1, NULL,	"Reserved"					);
				SHOW_FIELD(	 8,  8, 1, not,		"Test Reg"					);
				SHOW_FIELD(	 7,  7, 1, not,		"Single Cycle Disable"		);
				SHOW_FIELD(	 6,  6, 1, not,		"GPIO 6/Tx Auto Stop"		);
				break;

			case HPDI32_QUERY_MODEL_HPDI32ALT:

				SHOW_FIELD(	31, 10, 1, NULL,	"Reserved"					);
				SHOW_FIELD(	 9,  9, 1, not,		"GPIO D32"					);
				SHOW_FIELD(	 8,  8, 1, not,		"Test Reg + FWR Update"		);
				SHOW_FIELD(	 7,  7, 1, not,		"Single Cycle Disable"		);
				SHOW_FIELD(	 6,  6, 1, not,		"GPIO 6/Tx Auto Stop"		);
				break;
		}

		SHOW_FIELD(			 5,  5, 1, not,		"Overrun/Underrun Flags"	);
		SHOW_FIELD(			 4,  4, 1, not,		"DMDMA Channel 1"			);
		SHOW_FIELD(			 3,  3, 1, not,		"GPIO 0-5"					);
		SHOW_FIELD(			 2,  2, 1, not,		"IELR & IHLR Registers"		);
		SHOW_FIELD(			 1,  1, 1, not,		"TFWR & RFWR Registers"		);
		SHOW_FIELD(			 0,  0, 1, not,		"TFSR & RFSR Registers"		);
	}

	return(ret);
}



//*****************************************************************************
static int _fdr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  8, 1, NULL,	"Reserved"	);
	SHOW_FIELD( 7,  0, 1, NULL,	"Data"		);

	return(0);
}



//*****************************************************************************
static int _txvlcr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 0, NULL,	"Clock Cycles"	);
	printf("%ld  (0x%lX)\n", (long) value, (long) value);

	return(0);
}



//*****************************************************************************
static int _tlilcr_detail(int fd, int supported, u32 value, int width)
{
	u32 v;

	SHOW_FIELD(31, 16, 1, NULL,	"Reserved"		);

	SHOW_FIELD(15,  0, 0, NULL,	"Clock Cycles"	);
	v	= value & 0xFFFF;
	printf("%ld  (0x%lX)\n", (long) v, (long) v);

	return(0);
}



//*****************************************************************************
static int _icr_irq_detail(int fd, int supported, u32 value, int width, const char** list)
{
	SHOW_FIELD(31, 16, 1, NULL,	"Reserved"				);
	SHOW_FIELD(15, 15, 1, list,	"Rx FIFO Full"			);
	SHOW_FIELD(14, 14, 1, list,	"Rx FIFO Almost Full"	);
	SHOW_FIELD(13, 13, 1, list,	"Rx FIFO Almost Empty"	);
	SHOW_FIELD(12, 12, 1, list,	"Rx FIFO Empty"			);
	SHOW_FIELD(11, 11, 1, list,	"Tx FIFO Full"			);
	SHOW_FIELD(10, 10, 1, list,	"Tx FIFO Almost Full"	);
	SHOW_FIELD( 9,  9, 1, list,	"Tx FIFO Almost Empty"	);
	SHOW_FIELD( 8,  8, 1, list,	"Tx FIFO Empty"			);
	SHOW_FIELD( 7,  7, 1, list,	"Cable Command D6"		);
	SHOW_FIELD( 6,  6, 1, list,	"Cable Command D5"		);
	SHOW_FIELD( 5,  5, 1, list,	"Cable Command D4"		);
	SHOW_FIELD( 4,  4, 1, list,	"Cable Command D3"		);
	SHOW_FIELD( 3,  3, 1, list,	"Cable Command D2"		);
	SHOW_FIELD( 2,  2, 1, list,	"Cable Command D1"		);
	SHOW_FIELD( 1,  1, 1, list,	"Frame Valid End"		);
	SHOW_FIELD( 0,  0, 1, list,	"Frame Valid Begin"		);

	return(0);
}



//*****************************************************************************
static int _icr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	list[]	= { "Disabled",	"Enabled"	};

	_icr_irq_detail(fd, supported, value, width, list);
	return(0);
}



//*****************************************************************************
static int _isr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	list[]	= { "Idle",	"Active"	};

	_icr_irq_detail(fd, supported, value, width, list);
	return(0);
}



//*****************************************************************************
static int _tcdr_detail(int fd, int supported, u32 value, int width)
{
	s32	get;
	int	i;
	int	ret		= 0;
	u32	v;

	if (supported)
	{
		i	= hpdi32_query(fd, -1, 0, HPDI32_QUERY_REG_TCDR, &get);

		if (i)
			ret	= -1;
		else
			ret	= get ? 1 : 0;
	}
	else
	{
		SHOW_FIELD(31, 16, 1, NULL,	"Reserved"			);

		SHOW_FIELD(15,  0, 0, NULL,	"Tx Clock Divider"	);
		v	= value & 0xFFFF;
		printf("%ld  (/%ld)\n", (long) v, (long) ((v <= 1) ? 1 : v));
	}

	return(ret);
}



//*****************************************************************************
static int _xfsr_detail(int fd, int supported, u32 value, int width, int query)
{
	s32	get;
	int	i;
	int	ret;
	u32	v;

	if (supported)
	{
		i	= hpdi32_query(fd, -1, 0, query, &get);

		if (i)
			ret	= -1;
		else
			ret	= get ? 1 : 0;
	}
	else
	{
		ret	= 0;

		SHOW_FIELD(31, 20, 1, NULL,	"Reserved"	);

		SHOW_FIELD(19,  0, 0, NULL,	"FIFO Size"	);
		v	= value & 0xFFFFF;

		if (v)
		{
			printf("%ld Words", (long) v);

			if ((v) && ((v % 1024) == 0))
				printf("  (%ldK)", (long) (v / 1024));

		}
		else
		{
			printf("Unknown");
		}

		printf("\n");
	}

	return(ret);
}



//*****************************************************************************
static int _rfsr_detail(int fd, int supported, u32 value, int width)
{
	int	ret;

	ret	= _xfsr_detail(fd, supported, value, width, HPDI32_QUERY_REG_RFSR);
	return(ret);
}



//*****************************************************************************
static int _tfsr_detail(int fd, int supported, u32 value, int width)
{
	int	ret;

	ret	= _xfsr_detail(fd, supported, value, width, HPDI32_QUERY_REG_TFSR);
	return(ret);
}



//*****************************************************************************
static int _xfwr_detail(int fd, int supported, u32 value, int width, int query)
{
	s32	get;
	int	i;
	int	ret;
	u32	v;

	if (supported)
	{
		i	= hpdi32_query(fd, -1, 0, query, &get);
		if (i)
			ret	= -1;
		else
			ret	= get ? 1 : 0;
	}
	else
	{
		ret	= 0;

		SHOW_FIELD(31, 20, 1, NULL,	"Reserved"	);

		SHOW_FIELD(19,  0, 0, NULL,	"FIFO Words"	);
		v	= value & 0xFFFFF;
		printf("%ld Words", (long) v);

		if ((v) && ((v % 1024) == 0))
			printf("  (%ldK)", (long) (v / 1024));

		printf("\n");
	}

	return(ret);
}



//*****************************************************************************
static int _rfwr_detail(int fd, int supported, u32 value, int width)
{
	int	ret;

	ret	= _xfwr_detail(fd, supported, value, width, HPDI32_QUERY_REG_RFWR);
	return(ret);
}



//*****************************************************************************
static int _tfwr_detail(int fd, int supported, u32 value, int width)
{
	int	ret;

	ret	= _xfwr_detail(fd, supported, value, width, HPDI32_QUERY_REG_TFWR);
	return(ret);
}



//*****************************************************************************
static int _ielr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	list[]	= { "Level Triggerred",	"Edge Triggerred"	};

	s32	get;
	int	i;
	int	ret;

	if (supported)
	{
		i	= hpdi32_query(fd, -1, 0, HPDI32_QUERY_REG_IELR, &get);

		if (i)
			ret	= -1;
		else
			ret	= get ? 1 : 0;
	}
	else
	{
		ret		= 0;
		_icr_irq_detail(fd, supported, value, width, list);
	}

	return(ret);
}



//*****************************************************************************
static int _ihlr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	list[]	= { "Low",	"High"	};

	s32	get;
	int	i;
	int	ret;

	if (supported)
	{
		i	= hpdi32_query(fd, -1, 0, HPDI32_QUERY_REG_IHLR, &get);

		if (i)
			ret	= -1;
		else
			ret	= get ? 1 : 0;
	}
	else
	{
		ret		= 0;
		_icr_irq_detail(fd, supported, value, width, list);
	}

	return(ret);
}



//*****************************************************************************
static int _acr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	disabled[]	= { "Disabled",			"Enabled"			};

	s32	get;
	int	i;
	int	ret		= 0;

	if (supported)
	{
		i	= hpdi32_query(fd, -1, 0, HPDI32_QUERY_REG_ACR, &get);

		if (i)
			ret	= -1;
		else
			ret	= get ? 1 : 0;
	}
	else
	{
		SHOW_FIELD(31, 1, 1, NULL,		"Reserved"			);
		SHOW_FIELD( 0, 0, 1, disabled,	"GPIO D32 Output"	);
	}

	return(ret);
}



//*****************************************************************************
static int _gpio_d32_detail(int fd, int supported, int reg, u32 value, int width)
{
	static const char*	_0_1[] = { "0 (Low)", "1 (High"	};

	char	buf[16];
	s32		get;
	int		i;
	int		ret		= 0;

	if (supported)
	{
		i	= hpdi32_query(fd, -1, 0, reg, &get);

		if (i)
			ret	= -1;
		else
			ret	= get ? 1 : 0;
	}
	else
	{
		for (i = 31; i >= 0; i--)
		{
			sprintf(buf, "D%d", i);
			SHOW_FIELD(i, i, 1, _0_1, buf);
		}
	}

	return(ret);
}



//*****************************************************************************
static int _gor_detail(int fd, int supported, u32 value, int width)
{
	int	ret;

	ret	= _gpio_d32_detail(fd, supported, HPDI32_QUERY_REG_GOR, value, width);
	return(ret);
}



//*****************************************************************************
static int _gir_detail(int fd, int supported, u32 value, int width)
{
	int	ret;

	ret	= _gpio_d32_detail(fd, supported, HPDI32_QUERY_REG_GIR, value, width);
	return(ret);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(FRR),	0,	_frr_detail,	"Firmware Revision Register"				},
	{ _GSC_REG(BCR),	0,	_bcr_detail,	"Board Control Register"					},
	{ _GSC_REG(BSR),	0,	_bsr_detail,	"Board Status Register"						},
	{ _GSC_REG(TAR),	0,	_xar_detail,	"Tx Almost Register"						},
	{ _GSC_REG(RAR),	0,	_xar_detail,	"Rx Almost Register"						},
	{ _GSC_REG(FSR),	1,	_fsr_detail,	"Feature Set Register"						},
	{ _GSC_REG(FDR),	0,	_fdr_detail,	"FIFO Data Register"						},
	{ _GSC_REG(TSVLCR),	0,	_txvlcr_detail,	"Tx Status Valid Counter Register"			},
	{ _GSC_REG(TLVLCR),	0,	_txvlcr_detail,	"Tx Line Valid Length Counter Register"		},
	{ _GSC_REG(TLILCR),	0,	_tlilcr_detail,	"Tx Line Invalid Length Counter Register"	},
	{ _GSC_REG(RSCR),	0,	_txvlcr_detail,	"Rx Status Count Register"					},
	{ _GSC_REG(RLCR),	0,	_txvlcr_detail,	"Rx Line Count Register"					},
	{ _GSC_REG(ICR),	0,	_icr_detail,	"Interrupt Control Register"				},
	{ _GSC_REG(ISR),	0,	_isr_detail,	"Interrupt Status Register"					},
	{ _GSC_REG(TCDR),	1,	_tcdr_detail,	"Tx CLock Divider Register"					},
	{ _GSC_REG(TFSR),	1,	_tfsr_detail,	"Tx FIFO Size Register"						},
	{ _GSC_REG(RFSR),	1,	_rfsr_detail,	"Rx FIFO Size Register"						},
	{ _GSC_REG(TFWR),	1,	_tfwr_detail,	"Tx FIFO Words Register"					},
	{ _GSC_REG(RFWR),	1,	_rfwr_detail,	"Rx FIFO Words Register"					},
	{ _GSC_REG(IELR),	1,	_ielr_detail,	"Interrupt Edge/Level Register"				},
	{ _GSC_REG(IHLR),	1,	_ihlr_detail,	"Interrupt High/Low Register"				},
	{ _GSC_REG(ACR),	1,	_acr_detail,	"Auxiliary Control Register"				},
	{ _GSC_REG(GOR),	1,	_gor_detail,	"GPIO Output Register"						},
	{ _GSC_REG(GIR),	1,	_gir_detail,	"GPIO Input Register"						},

	{ NULL, 0, 0, 0, 0, NULL, NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	hpdi32_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* hpdi32_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	hpdi32_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* hpdi32_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	hpdi32_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* hpdi32_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	hpdi32_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* hpdi32_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	hpdi32_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hpdi32_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, hpdi32_reg_read);
	return(errs);
}



