// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/utils/util_reg_read.c $
// $Rev: 52295 $
// $Date: 2022-12-20 15:32:38 -0600 (Tue, 20 Dec 2022) $

// HPDI32: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hpdi32_reg_read
*
*	Purpose:
*
*		Provide a visual wrapper for the HPDI32_IOCTL_REG_READ service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		reg		The register to access.
*
*		value		The value read goes here.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hpdi32_reg_read(int fd, int index, int verbose, u32 reg, u32* value)
{
	gsc_reg_t	arg;
	int			errs;
	const char*	name;
	int			ret;

	if (verbose)
		gsc_label_index("Register Read", index);

	arg.reg		= reg;
	arg.value	= 0xDEADBEEF;
	arg.mask	= 0;	// unused here
	ret			= hpdi32_ioctl(fd, HPDI32_IOCTL_REG_READ, &arg);
	errs		= ret ? 1 : 0;

	if (verbose)
	{
		name	= hpdi32_reg_get_name(reg);
		printf(	"%s  (val 0x%08lX, %s)\n",
				errs ? "FAIL <---" : "PASS",
				(unsigned long) arg.value,
				name);
	}

	if (value)
		value[0]	= arg.value;

	return(errs);
}


