// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/utils/util_rx_fifo_status.c $
// $Rev: 49213 $
// $Date: 2021-03-15 16:42:01 -0500 (Mon, 15 Mar 2021) $

// HPDI32: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hpdi32_rx_fifo_status
*
*	Purpose:
*
*		Provide a visual wrapper for the HPDI32_IOCTL_RX_FIFO_STATUS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hpdi32_rx_fifo_status(int fd, int index, int verbose, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("Rx FIFO Status", index);

	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_RX_FIFO_STATUS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case HPDI32_FIFO_STATUS_EMPTY:

			ptr	= "Empty";
			break;

		case HPDI32_FIFO_STATUS_ALMOST_EMPTY:

			ptr	= "Almost Empty";
			break;

		case HPDI32_FIFO_STATUS_MEDIUM:

			ptr	= "Medium";
			break;

		case HPDI32_FIFO_STATUS_ALMOST_FULL:

			ptr	= "Almost Full";
			break;

		case HPDI32_FIFO_STATUS_FULL:

			ptr	= "Full";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


