// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_2.x.x.x_GSC_DN/utils/util_tristate_te_re.c $
// $Rev: 51575 $
// $Date: 2022-10-04 17:47:45 -0500 (Tue, 04 Oct 2022) $

// HPDI32: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hpdi32_tristate_te_re
*
*	Purpose:
*
*		Provide a visual wrapper for the HPDI32_IOCTL_TRISTATE_TE_RE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hpdi32_tristate_te_re(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= NULL;
	int			ret;

	if (verbose)
		gsc_label_index("Tx/Rx Enabled Tristate", index);

	ret		= hpdi32_ioctl(fd, HPDI32_IOCTL_TRISTATE_TE_RE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			break;

		case HPDI32_TRISTATE_TE_RE_NO:

			ptr	= "No, Drive when High";
			break;

		case HPDI32_TRISTATE_TE_RE_YES:

			ptr	= "Yes, Tristate when High";
			break;
	}

	if (verbose == 0)
		;
	else if (set == -1)
		printf("(Feature unsupported on this device.)\n");
	else
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


